@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2015-2016 Canonical Ltd
+ * Copyright (C) 2015-2020 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -1139,6 +1139,216 @@ func (safs *signAddFindSuite) TestFindMaxFormat(c *C) {
 	c.Check(err, ErrorMatches, `cannot find "test-only" assertions for format 3 higher than supported format 1`)
 }
 
+func (safs *signAddFindSuite) TestWithStackedBackstore(c *C) {
+	headers := map[string]interface{}{
+		"authority-id": "canonical",
+		"primary-key":  "one",
+	}
+	a1, err := safs.signingDB.Sign(asserts.TestOnlyType, headers, nil, safs.signingKeyID)
+	c.Assert(err, IsNil)
+
+	err = safs.db.Add(a1)
+	c.Assert(err, IsNil)
+
+	headers = map[string]interface{}{
+		"authority-id": "canonical",
+		"primary-key":  "two",
+	}
+	a2, err := safs.signingDB.Sign(asserts.TestOnlyType, headers, nil, safs.signingKeyID)
+	c.Assert(err, IsNil)
+
+	bs := asserts.NewMemoryBackstore()
+	stacked := safs.db.WithStackedBackstore(bs)
+
+	err = stacked.Add(a2)
+	c.Assert(err, IsNil)
+
+	_, err = stacked.Find(asserts.TestOnlyType, map[string]string{
+		"primary-key": "one",
+	})
+	c.Check(err, IsNil)
+
+	_, err = stacked.Find(asserts.TestOnlyType, map[string]string{
+		"primary-key": "two",
+	})
+	c.Check(err, IsNil)
+
+	_, err = safs.db.Find(asserts.TestOnlyType, map[string]string{
+		"primary-key": "two",
+	})
+	c.Check(asserts.IsNotFound(err), Equals, true)
+
+	_, err = stacked.Find(asserts.AccountKeyType, map[string]string{
+		"public-key-sha3-384": safs.signingKeyID,
+	})
+	c.Check(err, IsNil)
+
+	// stored in backstore
+	_, err = bs.Get(asserts.TestOnlyType, []string{"two"}, 0)
+	c.Check(err, IsNil)
+}
+
+func (safs *signAddFindSuite) TestWithStackedBackstoreSafety(c *C) {
+	stacked := safs.db.WithStackedBackstore(asserts.NewMemoryBackstore())
+
+	// usual add safety
+	pubKey0, err := safs.signingDB.PublicKey(safs.signingKeyID)
+	c.Assert(err, IsNil)
+	pubKey0Encoded, err := asserts.EncodePublicKey(pubKey0)
+	c.Assert(err, IsNil)
+
+	now := time.Now().UTC()
+	headers := map[string]interface{}{
+		"authority-id":        "canonical",
+		"account-id":          "canonical",
+		"public-key-sha3-384": safs.signingKeyID,
+		"name":                "default",
+		"since":               now.Format(time.RFC3339),
+		"until":               now.AddDate(1, 0, 0).Format(time.RFC3339),
+	}
+	tKey, err := safs.signingDB.Sign(asserts.AccountKeyType, headers, []byte(pubKey0Encoded), safs.signingKeyID)
+	c.Assert(err, IsNil)
+
+	err = stacked.Add(tKey)
+	c.Check(err, ErrorMatches, `cannot add "account-key" assertion with primary key clashing with a trusted assertion: .*`)
+
+	// cannot go back to old revisions
+	headers = map[string]interface{}{
+		"authority-id": "canonical",
+		"primary-key":  "one",
+	}
+	a0, err := safs.signingDB.Sign(asserts.TestOnlyType, headers, nil, safs.signingKeyID)
+	c.Assert(err, IsNil)
+
+	headers = map[string]interface{}{
+		"authority-id": "canonical",
+		"primary-key":  "one",
+		"revision":     "1",
+	}
+	a1, err := safs.signingDB.Sign(asserts.TestOnlyType, headers, nil, safs.signingKeyID)
+	c.Assert(err, IsNil)
+
+	err = safs.db.Add(a1)
+	c.Assert(err, IsNil)
+
+	err = stacked.Add(a0)
+	c.Assert(err, DeepEquals, &asserts.RevisionError{
+		Used:    0,
+		Current: 1,
+	})
+}
+
+func (safs *signAddFindSuite) TestFindSequence(c *C) {
+	headers := map[string]interface{}{
+		"authority-id": "canonical",
+		"n":            "s1",
+		"sequence":     "1",
+	}
+	sq1f0, err := safs.signingDB.Sign(asserts.TestOnlySeqType, headers, nil, safs.signingKeyID)
+	c.Assert(err, IsNil)
+
+	headers = map[string]interface{}{
+		"authority-id": "canonical",
+		"n":            "s1",
+		"sequence":     "2",
+	}
+	sq2f0, err := safs.signingDB.Sign(asserts.TestOnlySeqType, headers, nil, safs.signingKeyID)
+	c.Assert(err, IsNil)
+
+	headers = map[string]interface{}{
+		"authority-id": "canonical",
+		"format":       "1",
+		"n":            "s1",
+		"sequence":     "2",
+		"revision":     "1",
+	}
+	sq2f1, err := safs.signingDB.Sign(asserts.TestOnlySeqType, headers, nil, safs.signingKeyID)
+	c.Assert(err, IsNil)
+
+	headers = map[string]interface{}{
+		"authority-id": "canonical",
+		"format":       "1",
+		"n":            "s1",
+		"sequence":     "3",
+	}
+	sq3f1, err := safs.signingDB.Sign(asserts.TestOnlySeqType, headers, nil, safs.signingKeyID)
+	c.Assert(err, IsNil)
+
+	headers = map[string]interface{}{
+		"authority-id": "canonical",
+		"format":       "2",
+		"n":            "s1",
+		"sequence":     "3",
+		"revision":     "1",
+	}
+	sq3f2, err := safs.signingDB.Sign(asserts.TestOnlySeqType, headers, nil, safs.signingKeyID)
+	c.Assert(err, IsNil)
+
+	for _, a := range []asserts.Assertion{sq1f0, sq2f0, sq2f1, sq3f1} {
+
+		err = safs.db.Add(a)
+		c.Assert(err, IsNil)
+	}
+
+	// stack a backstore, for test completeness, this is an unlikely
+	// scenario atm
+	bs := asserts.NewMemoryBackstore()
+	db := safs.db.WithStackedBackstore(bs)
+	err = db.Add(sq3f2)
+	c.Assert(err, IsNil)
+
+	seqHeaders := map[string]string{
+		"n": "s1",
+	}
+	tests := []struct {
+		after     int
+		maxFormat int
+		sequence  int
+		format    int
+		revision  int
+	}{
+		{after: 0, maxFormat: 0, sequence: 1, format: 0, revision: 0},
+		{after: 0, maxFormat: 2, sequence: 1, format: 0, revision: 0},
+		{after: 1, maxFormat: 0, sequence: 2, format: 0, revision: 0},
+		{after: 1, maxFormat: 1, sequence: 2, format: 1, revision: 1},
+		{after: 1, maxFormat: 2, sequence: 2, format: 1, revision: 1},
+		{after: 2, maxFormat: 0, sequence: -1},
+		{after: 2, maxFormat: 1, sequence: 3, format: 1, revision: 0},
+		{after: 2, maxFormat: 2, sequence: 3, format: 2, revision: 1},
+		{after: 3, maxFormat: 0, sequence: -1},
+		{after: 3, maxFormat: 2, sequence: -1},
+		{after: 4, maxFormat: 2, sequence: -1},
+		{after: -1, maxFormat: 0, sequence: 2, format: 0, revision: 0},
+		{after: -1, maxFormat: 1, sequence: 3, format: 1, revision: 0},
+		{after: -1, maxFormat: 2, sequence: 3, format: 2, revision: 1},
+	}
+
+	for _, t := range tests {
+		a, err := db.FindSequence(asserts.TestOnlySeqType, seqHeaders, t.after, t.maxFormat)
+		if t.sequence == -1 {
+			c.Check(err, DeepEquals, &asserts.NotFoundError{
+				Type:    asserts.TestOnlySeqType,
+				Headers: seqHeaders,
+			})
+		} else {
+			c.Assert(err, IsNil)
+			c.Assert(a.HeaderString("n"), Equals, "s1")
+			c.Check(a.Sequence(), Equals, t.sequence)
+			c.Check(a.Format(), Equals, t.format)
+			c.Check(a.Revision(), Equals, t.revision)
+		}
+	}
+
+	seqHeaders = map[string]string{
+		"n": "s2",
+	}
+	_, err = db.FindSequence(asserts.TestOnlySeqType, seqHeaders, -1, 2)
+	c.Check(err, DeepEquals, &asserts.NotFoundError{
+		Type: asserts.TestOnlySeqType, Headers: seqHeaders,
+	})
+
+}
+
 type revisionErrorSuite struct{}
 
 func (res *revisionErrorSuite) TestErrorText(c *C) {
