@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2015-2018 Canonical Ltd
+ * Copyright (C) 2015-2020 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -21,6 +21,7 @@ package client
 
 import (
 	"bytes"
+	"context"
 	"encoding/json"
 	"fmt"
 	"io"
@@ -30,6 +31,7 @@ import (
 	"net/url"
 	"os"
 	"path"
+	"strconv"
 	"time"
 
 	"github.com/snapcore/snapd/dirs"
@@ -70,6 +72,9 @@ type Config struct {
 	// DisableKeepAlive indicates whether the connections should not be kept
 	// alive for later reuse
 	DisableKeepAlive bool
+
+	// User-Agent to sent to the snapd daemon
+	UserAgent string
 }
 
 // A Client knows how to talk to the snappy daemon.
@@ -84,6 +89,8 @@ type Client struct {
 
 	warningCount     int
 	warningTimestamp time.Time
+
+	userAgent string
 }
 
 // New returns a new instance of Client
@@ -103,6 +110,7 @@ func New(config *Config) *Client {
 			doer:        &http.Client{Transport: transport},
 			disableAuth: config.DisableAuth,
 			interactive: config.Interactive,
+			userAgent:   config.UserAgent,
 		}
 	}
 
@@ -115,6 +123,7 @@ func New(config *Config) *Client {
 		doer:        &http.Client{Transport: &http.Transport{DisableKeepAlives: config.DisableKeepAlive}},
 		disableAuth: config.DisableAuth,
 		interactive: config.Interactive,
+		userAgent:   config.UserAgent,
 	}
 }
 
@@ -172,10 +181,23 @@ func (e AuthorizationError) Error() string {
 	return fmt.Sprintf("cannot add authorization: %v", e.error)
 }
 
-type ConnectionError struct{ error }
+type ConnectionError struct{ Err error }
 
 func (e ConnectionError) Error() string {
-	return fmt.Sprintf("cannot communicate with server: %v", e.error)
+	var errStr string
+	switch e.Err {
+	case context.DeadlineExceeded:
+		errStr = "timeout exceeded while waiting for response"
+	case context.Canceled:
+		errStr = "request canceled"
+	default:
+		errStr = e.Err.Error()
+	}
+	return fmt.Sprintf("cannot communicate with server: %s", errStr)
+}
+
+func (e ConnectionError) Unwrap() error {
+	return e.Err
 }
 
 // AllowInteractionHeader is the HTTP request header used to indicate
@@ -183,9 +205,9 @@ func (e ConnectionError) Error() string {
 const AllowInteractionHeader = "X-Allow-Interaction"
 
 // raw performs a request and returns the resulting http.Response and
-// error you usually only need to call this directly if you expect the
+// error. You usually only need to call this directly if you expect the
 // response to not be JSON, otherwise you'd call Do(...) instead.
-func (client *Client) raw(method, urlpath string, query url.Values, headers map[string]string, body io.Reader) (*http.Response, error) {
+func (client *Client) raw(ctx context.Context, method, urlpath string, query url.Values, headers map[string]string, body io.Reader) (*http.Response, error) {
 	// fake a url to keep http.Client happy
 	u := client.baseURL
 	u.Path = path.Join(client.baseURL.Path, urlpath)
@@ -194,10 +216,23 @@ func (client *Client) raw(method, urlpath string, query url.Values, headers map[
 	if err != nil {
 		return nil, RequestError{err}
 	}
+	if client.userAgent != "" {
+		req.Header.Set("User-Agent", client.userAgent)
+	}
 
 	for key, value := range headers {
 		req.Header.Set(key, value)
 	}
+	// Content-length headers are special and need to be set
+	// directly to the request. Just setting it to the header
+	// will be ignored by go http.
+	if clStr := req.Header.Get("Content-Length"); clStr != "" {
+		cl, err := strconv.ParseInt(clStr, 10, 64)
+		if err != nil {
+			return nil, err
+		}
+		req.ContentLength = cl
+	}
 
 	if !client.disableAuth {
 		// set Authorization header if there are user's credentials
@@ -211,6 +246,10 @@ func (client *Client) raw(method, urlpath string, query url.Values, headers map[
 		req.Header.Set(AllowInteractionHeader, "true")
 	}
 
+	if ctx != nil {
+		req = req.WithContext(ctx)
+	}
+
 	rsp, err := client.doer.Do(req)
 	if err != nil {
 		return nil, ConnectionError{err}
@@ -219,13 +258,39 @@ func (client *Client) raw(method, urlpath string, query url.Values, headers map[
 	return rsp, nil
 }
 
+// rawWithTimeout is like raw(), but sets a timeout based on opts for
+// the whole of request and response (including rsp.Body() read) round
+// trip. If opts is nil the default doTimeout is used.
+// The caller is responsible for canceling the internal context
+// to release the resources associated with the request by calling the
+// returned cancel function.
+func (client *Client) rawWithTimeout(ctx context.Context, method, urlpath string, query url.Values, headers map[string]string, body io.Reader, opts *doOptions) (*http.Response, context.CancelFunc, error) {
+	opts = ensureDoOpts(opts)
+	if opts.Timeout <= 0 {
+		return nil, nil, fmt.Errorf("internal error: timeout not set in options for rawWithTimeout")
+	}
+
+	ctx, cancel := context.WithTimeout(ctx, opts.Timeout)
+	rsp, err := client.raw(ctx, method, urlpath, query, headers, body)
+	if err != nil && ctx.Err() != nil {
+		cancel()
+		return nil, nil, ConnectionError{ctx.Err()}
+	}
+
+	return rsp, cancel, err
+}
+
 var (
-	doRetry   = 250 * time.Millisecond
-	doTimeout = 5 * time.Second
+	doRetry = 250 * time.Millisecond
+	// snapd may need to reach out to the store, where it uses a fixed 10s
+	// timeout for the whole of a single request to complete, requests are
+	// retried for up to 38s in total, make sure that the client timeout is
+	// not shorter than that
+	doTimeout = 120 * time.Second
 )
 
-// MockDoRetry mocks the delays used by the do retry loop.
-func MockDoRetry(retry, timeout time.Duration) (restore func()) {
+// MockDoTimings mocks the delay used by the do retry loop and request timeout.
+func MockDoTimings(retry, timeout time.Duration) (restore func()) {
 	oldRetry := doRetry
 	oldTimeout := doTimeout
 	doRetry = retry
@@ -249,39 +314,86 @@ func (client *Client) Hijack(f func(*http.Request) (*http.Response, error)) {
 	client.doer = hijacked{f}
 }
 
+type doOptions struct {
+	// Timeout is the overall request timeout
+	Timeout time.Duration
+	// Retry interval.
+	// Note for a request with a Timeout but without a retry, Retry should just
+	// be set to something larger than the Timeout.
+	Retry time.Duration
+}
+
+func ensureDoOpts(opts *doOptions) *doOptions {
+	if opts == nil {
+		// defaults
+		opts = &doOptions{
+			Timeout: doTimeout,
+			Retry:   doRetry,
+		}
+	}
+	return opts
+}
+
+// doNoTimeoutAndRetry can be passed to the do family to not have timeout
+// nor retries.
+var doNoTimeoutAndRetry = &doOptions{
+	Timeout: time.Duration(-1),
+}
+
 // do performs a request and decodes the resulting json into the given
 // value. It's low-level, for testing/experimenting only; you should
 // usually use a higher level interface that builds on this.
-func (client *Client) do(method, path string, query url.Values, headers map[string]string, body io.Reader, v interface{}) error {
-	retry := time.NewTicker(doRetry)
-	defer retry.Stop()
-	timeout := time.After(doTimeout)
+func (client *Client) do(method, path string, query url.Values, headers map[string]string, body io.Reader, v interface{}, opts *doOptions) (statusCode int, err error) {
+	opts = ensureDoOpts(opts)
+
+	client.checkMaintenanceJSON()
+
 	var rsp *http.Response
-	var err error
-	for {
-		rsp, err = client.raw(method, path, query, headers, body)
-		if err == nil || method != "GET" {
+	var ctx context.Context = context.Background()
+	if opts.Timeout <= 0 {
+		// no timeout and retries
+		rsp, err = client.raw(ctx, method, path, query, headers, body)
+	} else {
+		if opts.Retry <= 0 {
+			return 0, fmt.Errorf("internal error: retry setting %s invalid", opts.Retry)
+		}
+		retry := time.NewTicker(opts.Retry)
+		defer retry.Stop()
+		timeout := time.NewTimer(opts.Timeout)
+		defer timeout.Stop()
+
+		for {
+			var cancel context.CancelFunc
+			// use the same timeout as for the whole of the retry
+			// loop to error out the whole do() call when a single
+			// request exceeds the deadline
+			rsp, cancel, err = client.rawWithTimeout(ctx, method, path, query, headers, body, opts)
+			if err == nil {
+				defer cancel()
+			}
+			if err == nil || method != "GET" {
+				break
+			}
+			select {
+			case <-retry.C:
+				continue
+			case <-timeout.C:
+			}
 			break
 		}
-		select {
-		case <-retry.C:
-			continue
-		case <-timeout:
-		}
-		break
 	}
 	if err != nil {
-		return err
+		return 0, err
 	}
 	defer rsp.Body.Close()
 
 	if v != nil {
 		if err := decodeInto(rsp.Body, v); err != nil {
-			return err
+			return rsp.StatusCode, err
 		}
 	}
 
-	return nil
+	return rsp.StatusCode, nil
 }
 
 func decodeInto(reader io.Reader, v interface{}) error {
@@ -302,11 +414,64 @@ func decodeInto(reader io.Reader, v interface{}) error {
 // response payload into the given value using the "UseNumber" json decoding
 // which produces json.Numbers instead of float64 types for numbers.
 func (client *Client) doSync(method, path string, query url.Values, headers map[string]string, body io.Reader, v interface{}) (*ResultInfo, error) {
+	return client.doSyncWithOpts(method, path, query, headers, body, v, nil)
+}
+
+// checkMaintenanceJSON checks if there is a maintenance.json file written by
+// snapd the daemon that positively identifies snapd as being unavailable due to
+// maintenance, either for snapd restarting itself to update, or rebooting the
+// system to update the kernel or base snap, etc. If there is ongoing
+// maintenance, then the maintenance object on the client is set appropriately.
+// note that currently checkMaintenanceJSON does not return errors, such that
+// if the file is missing or corrupt or empty, nothing will happen and it will
+// be silently ignored
+func (client *Client) checkMaintenanceJSON() {
+	f, err := os.Open(dirs.SnapdMaintenanceFile)
+	// just continue if we can't read the maintenance file
+	if err != nil {
+		return
+	}
+	defer f.Close()
+
+	// we have a maintenance file, try to read it
+	maintenance := &Error{}
+
+	if err := json.NewDecoder(f).Decode(&maintenance); err != nil {
+		// if the json is malformed, just ignore it for now, we only use it for
+		// positive identification of snapd down for maintenance
+		return
+	}
+
+	if maintenance != nil {
+		switch maintenance.Kind {
+		case ErrorKindDaemonRestart:
+			client.maintenance = maintenance
+		case ErrorKindSystemRestart:
+			client.maintenance = maintenance
+		}
+		// don't set maintenance for other kinds, as we don't know what it
+		// is yet
+
+		// this also means an empty json object in maintenance.json doesn't get
+		// treated as a real maintenance downtime for example
+	}
+}
+
+func (client *Client) doSyncWithOpts(method, path string, query url.Values, headers map[string]string, body io.Reader, v interface{}, opts *doOptions) (*ResultInfo, error) {
+	// first check maintenance.json to see if snapd is down for a restart, and
+	// set cli.maintenance as appropriate, then perform the request
+	// TODO: it would be a nice thing to skip the request if we know that snapd
+	// won't respond and return a specific error, but that's a big behavior
+	// change we probably shouldn't make right now, not to mention it probably
+	// requires adjustments in other areas too
+	client.checkMaintenanceJSON()
+
 	var rsp response
-	if err := client.do(method, path, query, headers, body, &rsp); err != nil {
+	statusCode, err := client.do(method, path, query, headers, body, &rsp, opts)
+	if err != nil {
 		return nil, err
 	}
-	if err := rsp.err(client); err != nil {
+	if err := rsp.err(client, statusCode); err != nil {
 		return nil, err
 	}
 	if rsp.Type != "sync" {
@@ -326,23 +491,23 @@ func (client *Client) doSync(method, path string, query url.Values, headers map[
 }
 
 func (client *Client) doAsync(method, path string, query url.Values, headers map[string]string, body io.Reader) (changeID string, err error) {
-	_, changeID, err = client.doAsyncFull(method, path, query, headers, body)
+	_, changeID, err = client.doAsyncFull(method, path, query, headers, body, nil)
 	return
 }
 
-func (client *Client) doAsyncFull(method, path string, query url.Values, headers map[string]string, body io.Reader) (result json.RawMessage, changeID string, err error) {
+func (client *Client) doAsyncFull(method, path string, query url.Values, headers map[string]string, body io.Reader, opts *doOptions) (result json.RawMessage, changeID string, err error) {
 	var rsp response
-
-	if err := client.do(method, path, query, headers, body, &rsp); err != nil {
+	statusCode, err := client.do(method, path, query, headers, body, &rsp, opts)
+	if err != nil {
 		return nil, "", err
 	}
-	if err := rsp.err(client); err != nil {
+	if err := rsp.err(client, statusCode); err != nil {
 		return nil, "", err
 	}
 	if rsp.Type != "async" {
 		return nil, "", fmt.Errorf("expected async response for %q on %q, got %q", method, path, rsp.Type)
 	}
-	if rsp.StatusCode != 202 {
+	if statusCode != 202 {
 		return nil, "", fmt.Errorf("operation not accepted")
 	}
 	if rsp.Change == "" {
@@ -359,7 +524,9 @@ type ServerVersion struct {
 	OSVersionID string
 	OnClassic   bool
 
-	KernelVersion string
+	KernelVersion  string
+	Architecture   string
+	Virtualization string
 }
 
 func (client *Client) ServerVersion() (*ServerVersion, error) {
@@ -375,18 +542,18 @@ func (client *Client) ServerVersion() (*ServerVersion, error) {
 		OSVersionID: sysInfo.OSRelease.VersionID,
 		OnClassic:   sysInfo.OnClassic,
 
-		KernelVersion: sysInfo.KernelVersion,
+		KernelVersion:  sysInfo.KernelVersion,
+		Architecture:   sysInfo.Architecture,
+		Virtualization: sysInfo.Virtualization,
 	}, nil
 }
 
 // A response produced by the REST API will usually fit in this
 // (exceptions are the icons/ endpoints obvs)
 type response struct {
-	Result     json.RawMessage `json:"result"`
-	Status     string          `json:"status"`
-	StatusCode int             `json:"status-code"`
-	Type       string          `json:"type"`
-	Change     string          `json:"change"`
+	Result json.RawMessage `json:"result"`
+	Type   string          `json:"type"`
+	Change string          `json:"change"`
 
 	WarningCount     int       `json:"warning-count"`
 	WarningTimestamp time.Time `json:"warning-timestamp"`
@@ -398,7 +565,7 @@ type response struct {
 
 // Error is the real value of response.Result when an error occurs.
 type Error struct {
-	Kind    string      `json:"kind"`
+	Kind    ErrorKind   `json:"kind"`
 	Value   interface{} `json:"value"`
 	Message string      `json:"message"`
 
@@ -409,53 +576,12 @@ func (e *Error) Error() string {
 	return e.Message
 }
 
-const (
-	ErrorKindTwoFactorRequired = "two-factor-required"
-	ErrorKindTwoFactorFailed   = "two-factor-failed"
-	ErrorKindLoginRequired     = "login-required"
-	ErrorKindInvalidAuthData   = "invalid-auth-data"
-	ErrorKindTermsNotAccepted  = "terms-not-accepted"
-	ErrorKindNoPaymentMethods  = "no-payment-methods"
-	ErrorKindPaymentDeclined   = "payment-declined"
-	ErrorKindPasswordPolicy    = "password-policy"
-
-	ErrorKindSnapAlreadyInstalled   = "snap-already-installed"
-	ErrorKindSnapNotInstalled       = "snap-not-installed"
-	ErrorKindSnapNotFound           = "snap-not-found"
-	ErrorKindAppNotFound            = "app-not-found"
-	ErrorKindSnapLocal              = "snap-local"
-	ErrorKindSnapNeedsDevMode       = "snap-needs-devmode"
-	ErrorKindSnapNeedsClassic       = "snap-needs-classic"
-	ErrorKindSnapNeedsClassicSystem = "snap-needs-classic-system"
-	ErrorKindSnapNotClassic         = "snap-not-classic"
-	ErrorKindNoUpdateAvailable      = "snap-no-update-available"
-
-	ErrorKindRevisionNotAvailable     = "snap-revision-not-available"
-	ErrorKindChannelNotAvailable      = "snap-channel-not-available"
-	ErrorKindArchitectureNotAvailable = "snap-architecture-not-available"
-
-	ErrorKindChangeConflict = "snap-change-conflict"
-
-	ErrorKindNotSnap = "snap-not-a-snap"
-
-	ErrorKindNetworkTimeout = "network-timeout"
-	ErrorKindDNSFailure     = "dns-failure"
-
-	ErrorKindInterfacesUnchanged = "interfaces-unchanged"
-
-	ErrorKindBadQuery           = "bad-query"
-	ErrorKindConfigNoSuchOption = "option-not-found"
-
-	ErrorKindSystemRestart = "system-restart"
-	ErrorKindDaemonRestart = "daemon-restart"
-)
-
 // IsRetryable returns true if the given error is an error
 // that can be retried later.
 func IsRetryable(err error) bool {
 	switch e := err.(type) {
 	case *Error:
-		return e.Kind == ErrorKindChangeConflict
+		return e.Kind == ErrorKindSnapChangeConflict
 	}
 	return false
 }
@@ -481,6 +607,17 @@ func IsInterfacesUnchangedError(err error) bool {
 	return e.Kind == ErrorKindInterfacesUnchanged
 }
 
+// IsAssertionNotFoundError returns whether the given error means that the
+// assertion wasn't found and thus the device isn't ready/seeded.
+func IsAssertionNotFoundError(err error) bool {
+	e, ok := err.(*Error)
+	if !ok || e == nil {
+		return false
+	}
+
+	return e.Kind == ErrorKindAssertionNotFound
+}
+
 // OSRelease contains information about the system extracted from /etc/os-release.
 type OSRelease struct {
 	ID        string `json:"id"`
@@ -507,14 +644,16 @@ type SysInfo struct {
 	OnClassic bool      `json:"on-classic"`
 	Managed   bool      `json:"managed"`
 
-	KernelVersion string `json:"kernel-version,omitempty"`
+	KernelVersion  string `json:"kernel-version,omitempty"`
+	Architecture   string `json:"architecture,omitempty"`
+	Virtualization string `json:"virtualization,omitempty"`
 
 	Refresh         RefreshInfo         `json:"refresh,omitempty"`
 	Confinement     string              `json:"confinement"`
 	SandboxFeatures map[string][]string `json:"sandbox-features,omitempty"`
 }
 
-func (rsp *response) err(cli *Client) error {
+func (rsp *response) err(cli *Client, statusCode int) error {
 	if cli != nil {
 		maintErr := rsp.Maintenance
 		// avoid setting to (*client.Error)(nil)
@@ -530,9 +669,9 @@ func (rsp *response) err(cli *Client) error {
 	var resultErr Error
 	err := json.Unmarshal(rsp.Result, &resultErr)
 	if err != nil || resultErr.Message == "" {
-		return fmt.Errorf("server error: %q", rsp.Status)
+		return fmt.Errorf("server error: %q", http.StatusText(statusCode))
 	}
-	resultErr.StatusCode = rsp.StatusCode
+	resultErr.StatusCode = statusCode
 
 	return &resultErr
 }
@@ -548,7 +687,7 @@ func parseError(r *http.Response) error {
 		return fmt.Errorf("cannot unmarshal error: %v", err)
 	}
 
-	err := rsp.err(nil)
+	err := rsp.err(nil, r.StatusCode)
 	if err == nil {
 		return fmt.Errorf("server error: %q", r.Status)
 	}
@@ -559,112 +698,17 @@ func parseError(r *http.Response) error {
 func (client *Client) SysInfo() (*SysInfo, error) {
 	var sysInfo SysInfo
 
-	if _, err := client.doSync("GET", "/v2/system-info", nil, nil, nil, &sysInfo); err != nil {
+	opts := &doOptions{
+		Timeout: 25 * time.Second,
+		Retry:   doRetry,
+	}
+	if _, err := client.doSyncWithOpts("GET", "/v2/system-info", nil, nil, nil, &sysInfo, opts); err != nil {
 		return nil, fmt.Errorf("cannot obtain system details: %v", err)
 	}
 
 	return &sysInfo, nil
 }
 
-// CreateUserResult holds the result of a user creation.
-type CreateUserResult struct {
-	Username string   `json:"username"`
-	SSHKeys  []string `json:"ssh-keys"`
-}
-
-// CreateUserOptions holds options for creating a local system user.
-//
-// If Known is false, the provided email is used to query the store for
-// username and SSH key details.
-//
-// If Known is true, the user will be created by looking through existing
-// system-user assertions and looking for a matching email. If Email is
-// empty then all such assertions are considered and multiple users may
-// be created.
-type CreateUserOptions struct {
-	Email        string `json:"email,omitempty"`
-	Sudoer       bool   `json:"sudoer,omitempty"`
-	Known        bool   `json:"known,omitempty"`
-	ForceManaged bool   `json:"force-managed,omitempty"`
-}
-
-// CreateUser creates a local system user. See CreateUserOptions for details.
-func (client *Client) CreateUser(options *CreateUserOptions) (*CreateUserResult, error) {
-	if options.Email == "" {
-		return nil, fmt.Errorf("cannot create a user without providing an email")
-	}
-
-	var result CreateUserResult
-	data, err := json.Marshal(options)
-	if err != nil {
-		return nil, err
-	}
-
-	if _, err := client.doSync("POST", "/v2/create-user", nil, nil, bytes.NewReader(data), &result); err != nil {
-		return nil, fmt.Errorf("while creating user: %v", err)
-	}
-	return &result, nil
-}
-
-// CreateUsers creates multiple local system users. See CreateUserOptions for details.
-//
-// Results may be provided even if there are errors.
-func (client *Client) CreateUsers(options []*CreateUserOptions) ([]*CreateUserResult, error) {
-	for _, opts := range options {
-		if opts.Email == "" && !opts.Known {
-			return nil, fmt.Errorf("cannot create user from store details without an email to query for")
-		}
-	}
-
-	var results []*CreateUserResult
-	var errs []error
-
-	for _, opts := range options {
-		data, err := json.Marshal(opts)
-		if err != nil {
-			return nil, err
-		}
-
-		if opts.Email == "" {
-			var result []*CreateUserResult
-			if _, err := client.doSync("POST", "/v2/create-user", nil, nil, bytes.NewReader(data), &result); err != nil {
-				errs = append(errs, err)
-			} else {
-				results = append(results, result...)
-			}
-		} else {
-			var result *CreateUserResult
-			if _, err := client.doSync("POST", "/v2/create-user", nil, nil, bytes.NewReader(data), &result); err != nil {
-				errs = append(errs, err)
-			} else {
-				results = append(results, result)
-			}
-		}
-	}
-
-	if len(errs) == 1 {
-		return results, errs[0]
-	}
-	if len(errs) > 1 {
-		var buf bytes.Buffer
-		for _, err := range errs {
-			fmt.Fprintf(&buf, "\n- %s", err)
-		}
-		return results, fmt.Errorf("while creating users:%s", buf.Bytes())
-	}
-	return results, nil
-}
-
-// Users returns the local users.
-func (client *Client) Users() ([]*User, error) {
-	var result []*User
-
-	if _, err := client.doSync("GET", "/v2/users", nil, nil, nil, &result); err != nil {
-		return nil, fmt.Errorf("while getting users: %v", err)
-	}
-	return result, nil
-}
-
 type debugAction struct {
 	Action string      `json:"action"`
 	Params interface{} `json:"params,omitempty"`
@@ -684,3 +728,22 @@ func (client *Client) Debug(action string, params interface{}, result interface{
 	_, err = client.doSync("POST", "/v2/debug", nil, nil, bytes.NewReader(body), result)
 	return err
 }
+
+func (client *Client) DebugGet(aspect string, result interface{}, params map[string]string) error {
+	urlParams := url.Values{"aspect": []string{aspect}}
+	for k, v := range params {
+		urlParams.Set(k, v)
+	}
+	_, err := client.doSync("GET", "/v2/debug", urlParams, nil, nil, &result)
+	return err
+}
+
+type SystemRecoveryKeysResponse struct {
+	RecoveryKey  string `json:"recovery-key"`
+	ReinstallKey string `json:"reinstall-key"`
+}
+
+func (client *Client) SystemRecoveryKeys(result interface{}) error {
+	_, err := client.doSync("GET", "/v2/system-recovery-keys", nil, nil, nil, &result)
+	return err
+}
