@@ -21,6 +21,7 @@ package client
 
 import (
 	"bytes"
+	"context"
 	"encoding/json"
 	"fmt"
 	"io"
@@ -32,12 +33,26 @@ import (
 type SnapOptions struct {
 	Channel          string `json:"channel,omitempty"`
 	Revision         string `json:"revision,omitempty"`
+	CohortKey        string `json:"cohort-key,omitempty"`
+	LeaveCohort      bool   `json:"leave-cohort,omitempty"`
 	DevMode          bool   `json:"devmode,omitempty"`
 	JailMode         bool   `json:"jailmode,omitempty"`
 	Classic          bool   `json:"classic,omitempty"`
 	Dangerous        bool   `json:"dangerous,omitempty"`
 	IgnoreValidation bool   `json:"ignore-validation,omitempty"`
+	IgnoreRunning    bool   `json:"ignore-running,omitempty"`
 	Unaliased        bool   `json:"unaliased,omitempty"`
+	Purge            bool   `json:"purge,omitempty"`
+	Amend            bool   `json:"amend,omitempty"`
+
+	Users []string `json:"users,omitempty"`
+}
+
+func writeFieldBool(mw *multipart.Writer, key string, val bool) error {
+	if !val {
+		return nil
+	}
+	return mw.WriteField(key, "true")
 }
 
 func (opts *SnapOptions) writeModeFields(mw *multipart.Writer) error {
@@ -51,11 +66,7 @@ func (opts *SnapOptions) writeModeFields(mw *multipart.Writer) error {
 		{"dangerous", opts.Dangerous},
 	}
 	for _, o := range fields {
-		if !o.b {
-			continue
-		}
-		err := mw.WriteField(o.f, "true")
-		if err != nil {
+		if err := writeFieldBool(mw, o.f, o.b); err != nil {
 			return err
 		}
 	}
@@ -63,6 +74,13 @@ func (opts *SnapOptions) writeModeFields(mw *multipart.Writer) error {
 	return nil
 }
 
+func (opts *SnapOptions) writeOptionFields(mw *multipart.Writer) error {
+	if err := writeFieldBool(mw, "ignore-running", opts.IgnoreRunning); err != nil {
+		return err
+	}
+	return writeFieldBool(mw, "unaliased", opts.Unaliased)
+}
+
 type actionData struct {
 	Action   string `json:"action"`
 	Name     string `json:"name,omitempty"`
@@ -73,6 +91,7 @@ type actionData struct {
 type multiActionData struct {
 	Action string   `json:"action"`
 	Snaps  []string `json:"snaps,omitempty"`
+	Users  []string `json:"users,omitempty"`
 }
 
 // Install adds the snap with the given name from the given channel (or
@@ -122,6 +141,24 @@ func (client *Client) Switch(name string, options *SnapOptions) (changeID string
 	return client.doSnapAction("switch", name, options)
 }
 
+// SnapshotMany snapshots many snaps (all, if names empty) for many users (all, if users is empty).
+func (client *Client) SnapshotMany(names []string, users []string) (setID uint64, changeID string, err error) {
+	result, changeID, err := client.doMultiSnapActionFull("snapshot", names, &SnapOptions{Users: users})
+	if err != nil {
+		return 0, "", err
+	}
+	if len(result) == 0 {
+		return 0, "", fmt.Errorf("server result does not contain snapshot set identifier")
+	}
+	var x struct {
+		SetID uint64 `json:"set-id"`
+	}
+	if err := json.Unmarshal(result, &x); err != nil {
+		return 0, "", err
+	}
+	return x.SetID, changeID, nil
+}
+
 var ErrDangerousNotApplicable = fmt.Errorf("dangerous option only meaningful when installing from a local file")
 
 func (client *Client) doSnapAction(actionName string, snapName string, options *SnapOptions) (changeID string, err error) {
@@ -149,25 +186,34 @@ func (client *Client) doMultiSnapAction(actionName string, snaps []string, optio
 	if options != nil {
 		return "", fmt.Errorf("cannot use options for multi-action") // (yet)
 	}
+	_, changeID, err = client.doMultiSnapActionFull(actionName, snaps, options)
+
+	return changeID, err
+}
+
+func (client *Client) doMultiSnapActionFull(actionName string, snaps []string, options *SnapOptions) (result json.RawMessage, changeID string, err error) {
 	action := multiActionData{
 		Action: actionName,
 		Snaps:  snaps,
 	}
+	if options != nil {
+		action.Users = options.Users
+	}
 	data, err := json.Marshal(&action)
 	if err != nil {
-		return "", fmt.Errorf("cannot marshal multi-snap action: %s", err)
+		return nil, "", fmt.Errorf("cannot marshal multi-snap action: %s", err)
 	}
 
 	headers := map[string]string{
 		"Content-Type": "application/json",
 	}
 
-	return client.doAsync("POST", "/v2/snaps", nil, headers, bytes.NewBuffer(data))
+	return client.doAsyncFull("POST", "/v2/snaps", nil, headers, bytes.NewBuffer(data), nil)
 }
 
-// InstallPath sideloads the snap with the given path, returning the UUID
-// of the background operation upon success.
-func (client *Client) InstallPath(path string, options *SnapOptions) (changeID string, err error) {
+// InstallPath sideloads the snap with the given path under optional provided name,
+// returning the UUID of the background operation upon success.
+func (client *Client) InstallPath(path, name string, options *SnapOptions) (changeID string, err error) {
 	f, err := os.Open(path)
 	if err != nil {
 		return "", fmt.Errorf("cannot open: %q", path)
@@ -175,6 +221,7 @@ func (client *Client) InstallPath(path string, options *SnapOptions) (changeID s
 
 	action := actionData{
 		Action:      "install",
+		Name:        name,
 		SnapPath:    path,
 		SnapOptions: options,
 	}
@@ -187,7 +234,8 @@ func (client *Client) InstallPath(path string, options *SnapOptions) (changeID s
 		"Content-Type": mw.FormDataContentType(),
 	}
 
-	return client.doAsync("POST", "/v2/snaps", nil, headers, pr)
+	_, changeID, err = client.doAsyncFull("POST", "/v2/snaps", nil, headers, pr, doNoTimeoutAndRetry)
+	return changeID, err
 }
 
 // Try
@@ -243,6 +291,11 @@ func sendSnapFile(snapPath string, snapFile *os.File, pw *io.PipeWriter, mw *mul
 		return
 	}
 
+	if err := action.writeOptionFields(mw); err != nil {
+		pw.CloseWithError(err)
+		return
+	}
+
 	fw, err := mw.CreateFormFile("snap", filepath.Base(snapPath))
 	if err != nil {
 		pw.CloseWithError(err)
@@ -258,3 +311,89 @@ func sendSnapFile(snapPath string, snapFile *os.File, pw *io.PipeWriter, mw *mul
 	mw.Close()
 	pw.Close()
 }
+
+type snapRevisionOptions struct {
+	Channel   string `json:"channel,omitempty"`
+	Revision  string `json:"revision,omitempty"`
+	CohortKey string `json:"cohort-key,omitempty"`
+}
+
+type downloadAction struct {
+	SnapName string `json:"snap-name"`
+
+	snapRevisionOptions
+
+	HeaderPeek  bool   `json:"header-peek,omitempty"`
+	ResumeToken string `json:"resume-token,omitempty"`
+}
+
+type DownloadInfo struct {
+	SuggestedFileName string
+	Size              int64
+	Sha3_384          string
+	ResumeToken       string
+}
+
+type DownloadOptions struct {
+	SnapOptions
+
+	HeaderPeek  bool
+	ResumeToken string
+	Resume      int64
+}
+
+// Download will stream the given snap to the client
+func (client *Client) Download(name string, options *DownloadOptions) (dlInfo *DownloadInfo, r io.ReadCloser, err error) {
+	if options == nil {
+		options = &DownloadOptions{}
+	}
+	action := downloadAction{
+		SnapName: name,
+		snapRevisionOptions: snapRevisionOptions{
+			Channel:   options.Channel,
+			CohortKey: options.CohortKey,
+			Revision:  options.Revision,
+		},
+		HeaderPeek:  options.HeaderPeek,
+		ResumeToken: options.ResumeToken,
+	}
+	data, err := json.Marshal(&action)
+	if err != nil {
+		return nil, nil, fmt.Errorf("cannot marshal snap action: %s", err)
+	}
+	headers := map[string]string{
+		"Content-Type": "application/json",
+	}
+	if options.Resume > 0 {
+		headers["range"] = fmt.Sprintf("bytes: %d-", options.Resume)
+	}
+
+	// no deadline for downloads
+	ctx := context.Background()
+	rsp, err := client.raw(ctx, "POST", "/v2/download", nil, headers, bytes.NewBuffer(data))
+	if err != nil {
+		return nil, nil, err
+	}
+
+	if rsp.StatusCode != 200 {
+		var r response
+		defer rsp.Body.Close()
+		if err := decodeInto(rsp.Body, &r); err != nil {
+			return nil, nil, err
+		}
+		return nil, nil, r.err(client, rsp.StatusCode)
+	}
+	matches := contentDispositionMatcher(rsp.Header.Get("Content-Disposition"))
+	if matches == nil || matches[1] == "" {
+		return nil, nil, fmt.Errorf("cannot determine filename")
+	}
+
+	dlInfo = &DownloadInfo{
+		SuggestedFileName: matches[1],
+		Size:              rsp.ContentLength,
+		Sha3_384:          rsp.Header.Get("Snap-Sha3-384"),
+		ResumeToken:       rsp.Header.Get("Snap-Download-Token"),
+	}
+
+	return dlInfo, rsp.Body, nil
+}
