@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2017 Canonical Ltd
+ * Copyright (C) 2017-2020 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -38,6 +38,7 @@ import (
 	"syscall"
 	"time"
 
+	"github.com/mvo5/goconfigparser"
 	"gopkg.in/retry.v1"
 
 	"github.com/snapcore/snapd/arch"
@@ -49,6 +50,8 @@ import (
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/release"
+	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snapdenv"
 	"github.com/snapcore/snapd/strutil"
 )
 
@@ -168,6 +171,8 @@ func (r *Repair) Run() error {
 		env = append(env, "PATH=/usr/sbin:/usr/bin:/sbin:/bin:"+repairToolsDir)
 	}
 
+	// TODO:UC20: add SNAPD_RECOVER_MODE if the repair assertion is for uc20
+
 	workdir := filepath.Join(rundir, "work")
 	if err := os.MkdirAll(workdir, 0700); err != nil {
 		return err
@@ -188,7 +193,7 @@ func (r *Repair) Run() error {
 	// wait for repair to finish or timeout
 	var scriptErr error
 	killTimerCh := time.After(defaultRepairTimeout)
-	doneCh := make(chan error)
+	doneCh := make(chan error, 1)
 	go func() {
 		doneCh <- cmd.Wait()
 		close(doneCh)
@@ -281,11 +286,15 @@ func NewRunner() *Runner {
 	run := &Runner{
 		sequenceNext: make(map[string]int),
 	}
-	opts := httputil.ClientOpts{
-		MayLogBody: false,
+	opts := httputil.ClientOptions{
+		MayLogBody:         false,
+		ProxyConnectHeader: http.Header{"User-Agent": []string{snapdenv.UserAgent()}},
 		TLSConfig: &tls.Config{
 			Time: run.now,
 		},
+		ExtraSSLCerts: &httputil.ExtraSSLCertsFromDir{
+			Dir: dirs.SnapdStoreSSLCertsDir,
+		},
 	}
 	run.cli = httputil.NewHTTPClient(&opts)
 	return run
@@ -299,9 +308,9 @@ var (
 		},
 	))
 
-	peekRetryStrategy = retry.LimitCount(5, retry.LimitTime(44*time.Second,
+	peekRetryStrategy = retry.LimitCount(6, retry.LimitTime(44*time.Second,
 		retry.Exponential{
-			Initial: 300 * time.Millisecond,
+			Initial: 500 * time.Millisecond,
 			Factor:  2.5,
 		},
 	))
@@ -332,6 +341,7 @@ func (run *Runner) Fetch(brandID string, repairID int, revision int) (*asserts.R
 		if err != nil {
 			return nil, err
 		}
+		req.Header.Set("User-Agent", snapdenv.UserAgent())
 		req.Header.Set("Accept", "application/x.ubuntu.assertion")
 		if revision >= 0 {
 			req.Header.Set("If-None-Match", fmt.Sprintf(`"%d"`, revision))
@@ -340,6 +350,8 @@ func (run *Runner) Fetch(brandID string, repairID int, revision int) (*asserts.R
 	}, func(resp *http.Response) error {
 		if resp.StatusCode == 200 {
 			logger.Debugf("fetching repair %s-%d", brandID, repairID)
+
+			// TODO: use something like TransferSpeedMonitoringWriter to avoid stalling here
 			// decode assertions
 			dec := asserts.NewDecoderWithTypeMaxBodySize(resp.Body, map[*asserts.AssertionType]int{
 				asserts.RepairType: maxRepairScriptSize,
@@ -432,10 +444,13 @@ func (run *Runner) Peek(brandID string, repairID int) (headers map[string]interf
 	var rsp peekResp
 
 	resp, err := httputil.RetryRequest(u.String(), func() (*http.Response, error) {
+		// TODO: setup a overall request timeout using contexts
+		// can be many minutes but not unlimited like now
 		req, err := http.NewRequest("GET", u.String(), nil)
 		if err != nil {
 			return nil, err
 		}
+		req.Header.Set("User-Agent", snapdenv.UserAgent())
 		req.Header.Set("Accept", "application/json")
 		return run.cli.Do(req)
 	}, func(resp *http.Response) error {
@@ -481,6 +496,8 @@ func (run *Runner) Peek(brandID string, repairID int) (headers map[string]interf
 type deviceInfo struct {
 	Brand string `json:"brand"`
 	Model string `json:"model"`
+	Base  string `json:"base"`
+	Mode  string `json:"mode"`
 }
 
 // RepairStatus represents the possible statuses of a repair.
@@ -568,11 +585,9 @@ func (run *Runner) initState() error {
 	os.Remove(dirs.SnapRepairStateFile)
 	run.state = state{}
 	// initialize time lower bound with image built time/seed.yaml time
-	info, err := os.Stat(filepath.Join(dirs.SnapSeedDir, "seed.yaml"))
-	if err != nil {
+	if err := run.findTimeLowerBound(); err != nil {
 		return err
 	}
-	run.moveTimeLowerBound(info.ModTime())
 	// initialize device info
 	if err := run.initDeviceInfo(); err != nil {
 		return err
@@ -648,16 +663,87 @@ func verifySignatures(a asserts.Assertion, workBS asserts.Backstore, trusted ass
 	return nil
 }
 
-func (run *Runner) initDeviceInfo() error {
-	const errPrefix = "cannot set device information: "
+func (run *Runner) findTimeLowerBound() error {
+	timeLowerBoundSources := []string{
+		// uc16
+		filepath.Join(dirs.SnapSeedDir, "seed.yaml"),
+		// uc20+
+		dirs.SnapModeenvFile,
+	}
+	// add all model files from uc20 seeds
+	allModels, err := filepath.Glob(filepath.Join(dirs.SnapSeedDir, "systems/*/model"))
+	if err != nil {
+		return err
+	}
+	timeLowerBoundSources = append(timeLowerBoundSources, allModels...)
 
+	// use all files as potential time inputs
+	for _, p := range timeLowerBoundSources {
+		info, err := os.Stat(p)
+		if os.IsNotExist(err) {
+			continue
+		}
+		if err != nil {
+			return err
+		}
+		run.moveTimeLowerBound(info.ModTime())
+	}
+	return nil
+}
+
+func findBrandAndModel() (*deviceInfo, error) {
+	if osutil.FileExists(dirs.SnapModeenvFile) {
+		return findDevInfo20()
+	}
+	return findDevInfo16()
+}
+
+func findDevInfo20() (*deviceInfo, error) {
+	cfg := goconfigparser.New()
+	cfg.AllowNoSectionHeader = true
+	if err := cfg.ReadFile(dirs.SnapModeenvFile); err != nil {
+		return nil, err
+	}
+	brandAndModel, err := cfg.Get("", "model")
+	if err != nil {
+		return nil, err
+	}
+	l := strings.SplitN(brandAndModel, "/", 2)
+	if len(l) != 2 {
+		return nil, fmt.Errorf("cannot find brand/model in modeenv model string %q", brandAndModel)
+	}
+
+	mode, err := cfg.Get("", "mode")
+	if err != nil {
+		return nil, err
+	}
+
+	baseName, err := cfg.Get("", "base")
+	if err != nil {
+		return nil, err
+	}
+
+	baseSn, err := snap.ParsePlaceInfoFromSnapFileName(baseName)
+	if err != nil {
+		return nil, err
+	}
+
+	return &deviceInfo{
+		Brand: l[0],
+		Model: l[1],
+		Base:  baseSn.SnapName(),
+		Mode:  mode,
+	}, nil
+}
+
+func findDevInfo16() (*deviceInfo, error) {
 	workBS := asserts.NewMemoryBackstore()
 	assertSeedDir := filepath.Join(dirs.SnapSeedDir, "assertions")
 	dc, err := ioutil.ReadDir(assertSeedDir)
 	if err != nil {
-		return err
+		return nil, err
 	}
-	var model *asserts.Model
+	var modelAs *asserts.Model
 	for _, fi := range dc {
 		fn := filepath.Join(assertSeedDir, fi.Name())
 		f, err := os.Open(fn)
@@ -674,37 +760,53 @@ func (run *Runner) initDeviceInfo() error {
 			}
 			switch a.Type() {
 			case asserts.ModelType:
-				if model != nil {
-					return fmt.Errorf(errPrefix + "multiple models in seed assertions")
+				if modelAs != nil {
+					return nil, fmt.Errorf("multiple models in seed assertions")
 				}
-				model = a.(*asserts.Model)
+				modelAs = a.(*asserts.Model)
 			case asserts.AccountType, asserts.AccountKeyType:
 				workBS.Put(a.Type(), a)
 			}
 		}
 	}
-	if model == nil {
-		return fmt.Errorf(errPrefix + "no model assertion in seed data")
+	if modelAs == nil {
+		return nil, fmt.Errorf("no model assertion in seed data")
 	}
 	trustedBS := trustedBackstore(sysdb.Trusted())
-	if err := verifySignatures(model, workBS, trustedBS); err != nil {
-		return fmt.Errorf(errPrefix+"%v", err)
+	if err := verifySignatures(modelAs, workBS, trustedBS); err != nil {
+		return nil, err
 	}
-	acctPK := []string{model.BrandID()}
+	acctPK := []string{modelAs.BrandID()}
 	acctMaxSupFormat := asserts.AccountType.MaxSupportedFormat()
 	acct, err := trustedBS.Get(asserts.AccountType, acctPK, acctMaxSupFormat)
 	if err != nil {
 		var err error
 		acct, err = workBS.Get(asserts.AccountType, acctPK, acctMaxSupFormat)
 		if err != nil {
-			return fmt.Errorf(errPrefix + "no brand account assertion in seed data")
+			return nil, fmt.Errorf("no brand account assertion in seed data")
 		}
 	}
 	if err := verifySignatures(acct, workBS, trustedBS); err != nil {
-		return fmt.Errorf(errPrefix+"%v", err)
+		return nil, err
 	}
-	run.state.Device.Brand = model.BrandID()
-	run.state.Device.Model = model.Model()
+
+	// get the base snap as well
+
+	return &deviceInfo{
+		Brand: modelAs.BrandID(),
+		Model: modelAs.Model(),
+		Base:  modelAs.Base(),
+		// Mode is unset on uc16/uc18
+	}, nil
+}
+
+func (run *Runner) initDeviceInfo() error {
+	dev, err := findBrandAndModel()
+	if err != nil {
+		return fmt.Errorf("cannot set device information: %v", err)
+	}
+	run.state.Device = *dev
+
 	return nil
 }
 
@@ -774,7 +876,7 @@ func (run *Runner) Applicable(headers map[string]interface{}) bool {
 	if err != nil {
 		return false
 	}
-	if len(archs) != 0 && !strutil.ListContains(archs, arch.UbuntuArchitecture()) {
+	if len(archs) != 0 && !strutil.ListContains(archs, arch.DpkgArchitecture()) {
 		return false
 	}
 	brandModel := fmt.Sprintf("%s/%s", run.state.Device.Brand, run.state.Device.Model)
@@ -797,6 +899,10 @@ func (run *Runner) Applicable(headers map[string]interface{}) bool {
 			return false
 		}
 	}
+
+	// TODO:UC20: need to consider filtering by bases and modes in the assertion
+	// here
+
 	return true
 }
 
@@ -917,7 +1023,8 @@ func (run *Runner) makeReady(brandID string, sequenceNext int) (repair *asserts.
 	return repair, nil
 }
 
-// Next returns the next repair for the brand id sequence to run/retry or ErrRepairNotFound if there is none atm. It updates the state as required.
+// Next returns the next repair for the brand id sequence to run/retry or
+// ErrRepairNotFound if there is none atm. It updates the state as required.
 func (run *Runner) Next(brandID string) (*Repair, error) {
 	sequenceNext := run.sequenceNext[brandID]
 	if sequenceNext == 0 {
@@ -982,6 +1089,9 @@ func (run *Runner) Verify(repair *asserts.Repair, aux []asserts.Assertion) error
 		trustedBS.Put(asserts.AccountKeyType, t)
 	}
 	for _, t := range sysdb.Trusted() {
+		// we do *not* add the defalt sysdb trusted account
+		// keys here because the repair assertions have their
+		// own *dedicated* root of trust
 		if t.Type() == asserts.AccountType {
 			trustedBS.Put(asserts.AccountType, t)
 		}
