@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2017 Canonical Ltd
+ * Copyright (C) 2017-2020 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -41,13 +41,18 @@ import (
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/asserts/assertstest"
 	"github.com/snapcore/snapd/asserts/sysdb"
+	"github.com/snapcore/snapd/boot"
 	repair "github.com/snapcore/snapd/cmd/snap-repair"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/snapdenv"
+	"github.com/snapcore/snapd/testutil"
 )
 
 type baseRunnerSuite struct {
+	testutil.BaseTest
+
 	tmpdir string
 
 	seedTime time.Time
@@ -71,6 +76,21 @@ type baseRunnerSuite struct {
 	restoreLogger func()
 }
 
+func makeReadOnly(c *C, dir string) (restore func()) {
+	// skip tests that need this because uid==0 does not honor
+	// write permissions in directories (yay, unix)
+	if os.Getuid() == 0 {
+		// FIXME: we could use osutil.Chattr() here
+		c.Skip("too lazy to make path readonly as root")
+	}
+	err := os.Chmod(dir, 0555)
+	c.Assert(err, IsNil)
+	return func() {
+		err := os.Chmod(dir, 0755)
+		c.Assert(err, IsNil)
+	}
+}
+
 func (s *baseRunnerSuite) SetUpSuite(c *C) {
 	s.storeSigning = assertstest.NewStoreStack("canonical", nil)
 
@@ -108,31 +128,14 @@ func (s *baseRunnerSuite) SetUpSuite(c *C) {
 }
 
 func (s *baseRunnerSuite) SetUpTest(c *C) {
-	_, s.restoreLogger = logger.MockLogger()
+	s.BaseTest.SetUpTest(c)
+
+	_, restoreLogger := logger.MockLogger()
+	s.AddCleanup(restoreLogger)
 
 	s.tmpdir = c.MkDir()
 	dirs.SetRootDir(s.tmpdir)
-
-	s.seedAssertsDir = filepath.Join(dirs.SnapSeedDir, "assertions")
-
-	// dummy seed yaml
-	err := os.MkdirAll(dirs.SnapSeedDir, 0755)
-	c.Assert(err, IsNil)
-	seedYamlFn := filepath.Join(dirs.SnapSeedDir, "seed.yaml")
-	err = ioutil.WriteFile(seedYamlFn, nil, 0644)
-	c.Assert(err, IsNil)
-	seedTime, err := time.Parse(time.RFC3339, "2017-08-11T15:49:49Z")
-	c.Assert(err, IsNil)
-	err = os.Chtimes(filepath.Join(dirs.SnapSeedDir, "seed.yaml"), seedTime, seedTime)
-	c.Assert(err, IsNil)
-	s.seedTime = seedTime
-
-	s.t0 = time.Now().UTC().Truncate(time.Minute)
-}
-
-func (s *baseRunnerSuite) TearDownTest(c *C) {
-	dirs.SetRootDir("/")
-	s.restoreLogger()
+	s.AddCleanup(func() { dirs.SetRootDir("/") })
 }
 
 func (s *baseRunnerSuite) signSeqRepairs(c *C, repairs []string) []string {
@@ -151,7 +154,7 @@ func (s *baseRunnerSuite) signSeqRepairs(c *C, repairs []string) []string {
 	return seq
 }
 
-const freshStateJSON = `{"device":{"brand":"my-brand","model":"my-model"},"time-lower-bound":"2017-08-11T15:49:49Z"}`
+const freshStateJSON = `{"device":{"brand":"my-brand","model":"my-model","base":"","mode":""},"time-lower-bound":"2017-08-11T15:49:49Z"}`
 
 func (s *baseRunnerSuite) freshState(c *C) {
 	err := os.MkdirAll(dirs.SnapRepairDir, 0775)
@@ -162,6 +165,17 @@ func (s *baseRunnerSuite) freshState(c *C) {
 
 type runnerSuite struct {
 	baseRunnerSuite
+
+	restore func()
+}
+
+func (s *runnerSuite) SetUpSuite(c *C) {
+	s.baseRunnerSuite.SetUpSuite(c)
+	s.restore = snapdenv.SetUserAgentFromVersion("1", nil, "snap-repair")
+}
+
+func (s *runnerSuite) TearDownSuite(c *C) {
+	s.restore()
 }
 
 var _ = Suite(&runnerSuite{})
@@ -230,6 +244,8 @@ func (s *runnerSuite) checkBrokenTimeNowMitigated(c *C, runner *repair.Runner) {
 
 func (s *runnerSuite) TestFetchJustRepair(c *C) {
 	mockServer := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		ua := r.Header.Get("User-Agent")
+		c.Check(strings.Contains(ua, "snap-repair"), Equals, true)
 		c.Check(r.Header.Get("Accept"), Equals, "application/x.ubuntu.assertion")
 		c.Check(r.URL.Path, Equals, "/repairs/canonical/2")
 		io.WriteString(w, testRepair)
@@ -477,6 +493,8 @@ func (s *runnerSuite) TestFetchRepairPlusKey(c *C) {
 
 func (s *runnerSuite) TestPeek(c *C) {
 	mockServer := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		ua := r.Header.Get("User-Agent")
+		c.Check(strings.Contains(ua, "snap-repair"), Equals, true)
 		c.Check(r.Header.Get("Accept"), Equals, "application/json")
 		c.Check(r.URL.Path, Equals, "/repairs/canonical/2")
 		io.WriteString(w, testHeadersResp)
@@ -593,120 +611,15 @@ func (s *runnerSuite) TestLoadState(c *C) {
 	c.Check(model, Equals, "my-model")
 }
 
-func (s *runnerSuite) initSeed(c *C) {
-	err := os.MkdirAll(s.seedAssertsDir, 0775)
-	c.Assert(err, IsNil)
-}
-
-func (s *runnerSuite) writeSeedAssert(c *C, fname string, a asserts.Assertion) {
-	err := ioutil.WriteFile(filepath.Join(s.seedAssertsDir, fname), asserts.Encode(a), 0644)
-	c.Assert(err, IsNil)
-}
-
-func (s *runnerSuite) rmSeedAssert(c *C, fname string) {
-	err := os.Remove(filepath.Join(s.seedAssertsDir, fname))
-	c.Assert(err, IsNil)
-}
-
-func (s *runnerSuite) TestLoadStateInitState(c *C) {
-	// sanity
-	c.Check(osutil.IsDirectory(dirs.SnapRepairDir), Equals, false)
-	c.Check(osutil.FileExists(dirs.SnapRepairStateFile), Equals, false)
-	// setup realistic seed/assertions
-	r := sysdb.InjectTrusted(s.storeSigning.Trusted)
-	defer r()
-	s.initSeed(c)
-	s.writeSeedAssert(c, "store.account-key", s.storeSigning.StoreAccountKey(""))
-	s.writeSeedAssert(c, "brand.account", s.brandAcct)
-	s.writeSeedAssert(c, "brand.account-key", s.brandAcctKey)
-	s.writeSeedAssert(c, "model", s.modelAs)
-
-	runner := repair.NewRunner()
-	err := runner.LoadState()
-	c.Assert(err, IsNil)
-	c.Check(osutil.FileExists(dirs.SnapRepairStateFile), Equals, true)
-
-	brand, model := runner.BrandModel()
-	c.Check(brand, Equals, "my-brand")
-	c.Check(model, Equals, "my-model-2")
-
-	c.Check(runner.TimeLowerBound().Equal(s.seedTime), Equals, true)
-}
-
-func (s *runnerSuite) TestLoadStateInitDeviceInfoFail(c *C) {
-	// sanity
-	c.Check(osutil.IsDirectory(dirs.SnapRepairDir), Equals, false)
-	c.Check(osutil.FileExists(dirs.SnapRepairStateFile), Equals, false)
-	// setup realistic seed/assertions
-	r := sysdb.InjectTrusted(s.storeSigning.Trusted)
-	defer r()
-	s.initSeed(c)
-
-	const errPrefix = "cannot set device information: "
-	tests := []struct {
-		breakFunc   func()
-		expectedErr string
-	}{
-		{func() { s.rmSeedAssert(c, "model") }, errPrefix + "no model assertion in seed data"},
-		{func() { s.rmSeedAssert(c, "brand.account") }, errPrefix + "no brand account assertion in seed data"},
-		{func() { s.rmSeedAssert(c, "brand.account-key") }, errPrefix + `cannot find public key.*`},
-		{func() {
-			// broken signature
-			blob := asserts.Encode(s.brandAcct)
-			err := ioutil.WriteFile(filepath.Join(s.seedAssertsDir, "brand.account"), blob[:len(blob)-3], 0644)
-			c.Assert(err, IsNil)
-		}, errPrefix + "cannot decode signature:.*"},
-		{func() { s.writeSeedAssert(c, "model2", s.modelAs) }, errPrefix + "multiple models in seed assertions"},
-	}
-
-	for _, test := range tests {
-		s.writeSeedAssert(c, "store.account-key", s.storeSigning.StoreAccountKey(""))
-		s.writeSeedAssert(c, "brand.account", s.brandAcct)
-		s.writeSeedAssert(c, "brand.account-key", s.brandAcctKey)
-		s.writeSeedAssert(c, "model", s.modelAs)
-
-		test.breakFunc()
-
-		runner := repair.NewRunner()
-		err := runner.LoadState()
-		c.Check(err, ErrorMatches, test.expectedErr)
-	}
-}
-
-func (s *runnerSuite) TestTLSTime(c *C) {
-	s.freshState(c)
-	runner := repair.NewRunner()
-	err := runner.LoadState()
-	c.Assert(err, IsNil)
-	epoch := time.Unix(0, 0)
-	r := repair.MockTimeNow(func() time.Time {
-		return epoch
-	})
-	defer r()
-	c.Check(runner.TLSTime().Equal(s.seedTime), Equals, true)
-}
-
-func makeReadOnly(c *C, dir string) (restore func()) {
-	// skip tests that need this because uid==0 does not honor
-	// write permissions in directories (yay, unix)
-	if os.Getuid() == 0 {
-		// FIXME: we could use osutil.Chattr() here
-		c.Skip("too lazy to make path readonly as root")
-	}
-	err := os.Chmod(dir, 0555)
-	c.Assert(err, IsNil)
-	return func() {
-		err := os.Chmod(dir, 0755)
-		c.Assert(err, IsNil)
-	}
-}
-
 func (s *runnerSuite) TestLoadStateInitStateFail(c *C) {
+	err := os.MkdirAll(dirs.SnapSeedDir, 0755)
+	c.Assert(err, IsNil)
+
 	restore := makeReadOnly(c, filepath.Dir(dirs.SnapSeedDir))
 	defer restore()
 
 	runner := repair.NewRunner()
-	err := runner.LoadState()
+	err = runner.LoadState()
 	c.Check(err, ErrorMatches, `cannot create repair state directory:.*`)
 }
 
@@ -747,9 +660,7 @@ func (s *runnerSuite) TestSaveState(c *C) {
 	err = runner.SaveState()
 	c.Assert(err, IsNil)
 
-	data, err := ioutil.ReadFile(dirs.SnapRepairStateFile)
-	c.Assert(err, IsNil)
-	c.Check(string(data), Equals, `{"device":{"brand":"my-brand","model":"my-model"},"sequences":{"canonical":[{"sequence":1,"revision":3,"status":0}]},"time-lower-bound":"2017-08-11T15:49:49Z"}`)
+	c.Check(dirs.SnapRepairStateFile, testutil.FileEquals, `{"device":{"brand":"my-brand","model":"my-model","base":"","mode":""},"sequences":{"canonical":[{"sequence":1,"revision":3,"status":0}]},"time-lower-bound":"2017-08-11T15:49:49Z"}`)
 }
 
 func (s *runnerSuite) TestApplicable(c *C) {
@@ -767,10 +678,10 @@ func (s *runnerSuite) TestApplicable(c *C) {
 		{map[string]interface{}{"series": []interface{}{"18", "16"}}, true},
 		{map[string]interface{}{"series": "18"}, false},
 		{map[string]interface{}{"series": []interface{}{18}}, false},
-		{map[string]interface{}{"architectures": []interface{}{arch.UbuntuArchitecture()}}, true},
+		{map[string]interface{}{"architectures": []interface{}{arch.DpkgArchitecture()}}, true},
 		{map[string]interface{}{"architectures": []interface{}{"other-arch"}}, false},
-		{map[string]interface{}{"architectures": []interface{}{"other-arch", arch.UbuntuArchitecture()}}, true},
-		{map[string]interface{}{"architectures": arch.UbuntuArchitecture()}, false},
+		{map[string]interface{}{"architectures": []interface{}{"other-arch", arch.DpkgArchitecture()}}, true},
+		{map[string]interface{}{"architectures": arch.DpkgArchitecture()}, false},
 		{map[string]interface{}{"models": []interface{}{"my-brand/my-model"}}, true},
 		{map[string]interface{}{"models": []interface{}{"other-brand/other-model"}}, false},
 		{map[string]interface{}{"models": []interface{}{"other-brand/other-model", "my-brand/my-model"}}, true},
@@ -875,6 +786,9 @@ AXNpZw==
 func makeMockServer(c *C, seqRepairs *[]string, redirectFirst bool) *httptest.Server {
 	var mockServer *httptest.Server
 	mockServer = httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		ua := r.Header.Get("User-Agent")
+		c.Check(strings.Contains(ua, "snap-repair"), Equals, true)
+
 		urlPath := r.URL.Path
 		if redirectFirst && r.Header.Get("Accept") == asserts.MediaType {
 			if !strings.HasPrefix(urlPath, "/final/") {
@@ -952,22 +866,6 @@ func (s *runnerSuite) TestVerify(c *C) {
 	c.Check(err, IsNil)
 }
 
-func (s *runnerSuite) signSeqRepairs(c *C, repairs []string) []string {
-	var seq []string
-	for _, rpr := range repairs {
-		decoded, err := asserts.Decode([]byte(rpr))
-		c.Assert(err, IsNil)
-		signed, err := s.repairsSigning.Sign(asserts.RepairType, decoded.Headers(), decoded.Body(), "")
-		c.Assert(err, IsNil)
-		buf := &bytes.Buffer{}
-		enc := asserts.NewEncoder(buf)
-		enc.Encode(signed)
-		enc.Encode(s.repairsAcctKey)
-		seq = append(seq, buf.String())
-	}
-	return seq
-}
-
 func (s *runnerSuite) loadSequences(c *C) map[string][]*repair.RepairState {
 	data, err := ioutil.ReadFile(dirs.SnapRepairStateFile)
 	c.Assert(err, IsNil)
@@ -1002,9 +900,7 @@ func (s *runnerSuite) testNext(c *C, redirectFirst bool) {
 	rpr, err = runner.Next("canonical")
 	c.Assert(err, IsNil)
 	c.Check(rpr.RepairID(), Equals, 3)
-	strm, err := ioutil.ReadFile(filepath.Join(dirs.SnapRepairAssertsDir, "canonical", "3", "r2.repair"))
-	c.Assert(err, IsNil)
-	c.Check(string(strm), Equals, seqRepairs[2])
+	c.Check(filepath.Join(dirs.SnapRepairAssertsDir, "canonical", "3", "r2.repair"), testutil.FileEquals, seqRepairs[2])
 
 	// no more
 	rpr, err = runner.Next("canonical")
@@ -1221,20 +1117,16 @@ func (s *runnerSuite) TestNextNotFound(c *C) {
 	runner.LoadState()
 
 	// sanity
-	data, err := ioutil.ReadFile(dirs.SnapRepairStateFile)
-	c.Assert(err, IsNil)
-	c.Check(string(data), Equals, freshStateJSON)
+	c.Check(dirs.SnapRepairStateFile, testutil.FileEquals, freshStateJSON)
 
-	_, err = runner.Next("canonical")
+	_, err := runner.Next("canonical")
 	c.Assert(err, Equals, repair.ErrRepairNotFound)
 
 	// we saved new time lower bound
 	t1 := runner.TimeLowerBound()
 	expected := strings.Replace(freshStateJSON, "2017-08-11T15:49:49Z", t1.Format(time.RFC3339), 1)
 	c.Check(expected, Not(Equals), freshStateJSON)
-	data, err = ioutil.ReadFile(dirs.SnapRepairStateFile)
-	c.Assert(err, IsNil)
-	c.Check(string(data), Equals, expected)
+	c.Check(dirs.SnapRepairStateFile, testutil.FileEquals, expected)
 }
 
 func (s *runnerSuite) TestNextSaveStateError(c *C) {
@@ -1423,9 +1315,10 @@ summary: repair one
 series:
   - 16
 timestamp: 2017-07-02T12:00:00Z
-body-length: 7
+body-length: 17
 sign-key-sha3-384: KPIl7M4vQ9d4AUjkoU41TGAwtOMLc_bWUCeW8AvdRWD4_xcP60Oo4ABsFNo6BtXj
 
+#!/bin/sh
 exit 0
 
 
@@ -1448,10 +1341,9 @@ AXNpZw==`}
 	rpr, err := runner.Next("canonical")
 	c.Assert(err, IsNil)
 
-	rpr.Run()
-	scrpt, err := ioutil.ReadFile(filepath.Join(dirs.SnapRepairRunDir, "canonical", "1", "r0.script"))
+	err = rpr.Run()
 	c.Assert(err, IsNil)
-	c.Check(string(scrpt), Equals, "exit 0\n")
+	c.Check(filepath.Join(dirs.SnapRepairRunDir, "canonical", "1", "r0.script"), testutil.FileEquals, "#!/bin/sh\nexit 0\n")
 }
 
 func makeMockRepair(script string) string {
@@ -1472,9 +1364,7 @@ AXNpZw==`, len(script), script)
 }
 
 func verifyRepairStatus(c *C, status repair.RepairStatus) {
-	data, err := ioutil.ReadFile(dirs.SnapRepairStateFile)
-	c.Assert(err, IsNil)
-	c.Check(string(data), Matches, fmt.Sprintf(`{"device":{"brand":"","model":""},"sequences":{"canonical":\[{"sequence":1,"revision":0,"status":%d}.*`, status))
+	c.Check(dirs.SnapRepairStateFile, testutil.FileContains, fmt.Sprintf(`{"device":{"brand":"","model":"","base":"","mode":""},"sequences":{"canonical":[{"sequence":1,"revision":0,"status":%d}`, status))
 }
 
 // tests related to correct execution of script
@@ -1503,6 +1393,7 @@ func (s *runScriptSuite) SetUpTest(c *C) {
 	s.baseRunnerSuite.SetUpTest(c)
 
 	s.mockServer = makeMockServer(c, &s.seqRepairs, false)
+	s.AddCleanup(func() { s.mockServer.Close() })
 
 	s.runner = repair.NewRunner()
 	s.runner.BaseURL = mustParseURL(s.mockServer.URL)
@@ -1510,14 +1401,8 @@ func (s *runScriptSuite) SetUpTest(c *C) {
 
 	s.runDir = filepath.Join(dirs.SnapRepairRunDir, "canonical", "1")
 
-	s.restoreErrTrackerReportRepair = repair.MockErrtrackerReportRepair(s.errtrackerReportRepair)
-}
-
-func (s *runScriptSuite) TearDownTest(c *C) {
-	s.baseRunnerSuite.TearDownTest(c)
-
-	s.restoreErrTrackerReportRepair()
-	s.mockServer.Close()
+	restoreErrTrackerReportRepair := repair.MockErrtrackerReportRepair(s.errtrackerReportRepair)
+	s.AddCleanup(restoreErrTrackerReportRepair)
 }
 
 func (s *runScriptSuite) errtrackerReportRepair(repair, errMsg, dupSig string, extra map[string]string) (string, error) {
@@ -1543,9 +1428,7 @@ func (s *runScriptSuite) testScriptRun(c *C, mockScript string) *repair.Repair {
 	err = rpr.Run()
 	c.Assert(err, IsNil)
 
-	scrpt, err := ioutil.ReadFile(filepath.Join(s.runDir, "r0.script"))
-	c.Assert(err, IsNil)
-	c.Check(string(scrpt), Equals, mockScript)
+	c.Check(filepath.Join(s.runDir, "r0.script"), testutil.FileEquals, mockScript)
 
 	return rpr
 }
@@ -1566,9 +1449,7 @@ func (m byMtime) Less(i, j int) bool { return m[i].ModTime().Before(m[j].ModTime
 func (m byMtime) Swap(i, j int)      { m[i], m[j] = m[j], m[i] }
 
 func (s *runScriptSuite) verifyOutput(c *C, name, expectedOutput string) {
-	output, err := ioutil.ReadFile(filepath.Join(s.runDir, name))
-	c.Assert(err, IsNil)
-	c.Check(string(output), Equals, expectedOutput)
+	c.Check(filepath.Join(s.runDir, name), testutil.FileEquals, expectedOutput)
 	// ensure correct permissions
 	fi, err := os.Stat(filepath.Join(s.runDir, name))
 	c.Assert(err, IsNil)
@@ -1764,13 +1645,227 @@ ls -l ${PATH##*:}/repair
 	err = rpr.Run()
 	c.Assert(err, IsNil)
 
-	output, err := ioutil.ReadFile(filepath.Join(s.runDir, "r0.retry"))
-	c.Assert(err, IsNil)
-	c.Check(string(output), Matches, fmt.Sprintf(`(?ms).*^PATH=.*:.*/run/snapd/repair/tools.*`))
-	c.Check(string(output), Matches, `(?ms).*/repair -> /usr/lib/snapd/snap-repair`)
+	c.Check(filepath.Join(s.runDir, "r0.retry"), testutil.FileMatches, fmt.Sprintf(`(?ms).*^PATH=.*:.*/run/snapd/repair/tools.*`))
+	c.Check(filepath.Join(s.runDir, "r0.retry"), testutil.FileContains, `/repair -> /usr/lib/snapd/snap-repair`)
 
 	// run again and ensure no error happens
 	err = rpr.Run()
 	c.Assert(err, IsNil)
 
 }
+
+// shared1620RunnerSuite is embedded by runner16Suite and
+// runner20Suite and the tests are run once with a simulated uc16 and
+// once with a simulated uc20 environment
+type shared1620RunnerSuite struct {
+	baseRunnerSuite
+
+	writeSeedAssert func(c *C, fname string, a asserts.Assertion)
+}
+
+func (s *shared1620RunnerSuite) TestTLSTime(c *C) {
+	s.freshState(c)
+	runner := repair.NewRunner()
+	err := runner.LoadState()
+	c.Assert(err, IsNil)
+	epoch := time.Unix(0, 0)
+	r := repair.MockTimeNow(func() time.Time {
+		return epoch
+	})
+	defer r()
+	c.Check(runner.TLSTime().Equal(s.seedTime), Equals, true)
+}
+
+func (s *shared1620RunnerSuite) TestLoadStateInitState(c *C) {
+	// sanity
+	c.Check(osutil.IsDirectory(dirs.SnapRepairDir), Equals, false)
+	c.Check(osutil.FileExists(dirs.SnapRepairStateFile), Equals, false)
+	// setup realistic seed/assertions
+	r := sysdb.InjectTrusted(s.storeSigning.Trusted)
+	defer r()
+	s.writeSeedAssert(c, "store.account-key", s.storeSigning.StoreAccountKey(""))
+	s.writeSeedAssert(c, "brand.account", s.brandAcct)
+	s.writeSeedAssert(c, "brand.account-key", s.brandAcctKey)
+	s.writeSeedAssert(c, "model", s.modelAs)
+
+	runner := repair.NewRunner()
+	err := runner.LoadState()
+	c.Assert(err, IsNil)
+	c.Check(osutil.FileExists(dirs.SnapRepairStateFile), Equals, true)
+
+	brand, model := runner.BrandModel()
+	c.Check(brand, Equals, "my-brand")
+	c.Check(model, Equals, "my-model-2")
+
+	c.Check(runner.TimeLowerBound().Equal(s.seedTime), Equals, true)
+}
+
+type runner16Suite struct {
+	shared1620RunnerSuite
+}
+
+var _ = Suite(&runner16Suite{})
+
+func (s *runner16Suite) SetUpTest(c *C) {
+	s.shared1620RunnerSuite.SetUpTest(c)
+
+	s.seedAssertsDir = filepath.Join(dirs.SnapSeedDir, "assertions")
+
+	// dummy seed yaml
+	err := os.MkdirAll(s.seedAssertsDir, 0755)
+	c.Assert(err, IsNil)
+	seedYamlFn := filepath.Join(dirs.SnapSeedDir, "seed.yaml")
+	err = ioutil.WriteFile(seedYamlFn, nil, 0644)
+	c.Assert(err, IsNil)
+	seedTime, err := time.Parse(time.RFC3339, "2017-08-11T15:49:49Z")
+	c.Assert(err, IsNil)
+	err = os.Chtimes(filepath.Join(dirs.SnapSeedDir, "seed.yaml"), seedTime, seedTime)
+	c.Assert(err, IsNil)
+	s.seedTime = seedTime
+
+	s.t0 = time.Now().UTC().Truncate(time.Minute)
+
+	s.writeSeedAssert = s.writeSeedAssert16
+}
+
+func (s *runner16Suite) writeSeedAssert16(c *C, fname string, a asserts.Assertion) {
+	err := ioutil.WriteFile(filepath.Join(s.seedAssertsDir, fname), asserts.Encode(a), 0644)
+	c.Assert(err, IsNil)
+}
+
+func (s *runner16Suite) rmSeedAssert16(c *C, fname string) {
+	err := os.Remove(filepath.Join(s.seedAssertsDir, fname))
+	c.Assert(err, IsNil)
+}
+
+func (s *runner16Suite) TestLoadStateInitDeviceInfoFail(c *C) {
+	// sanity
+	c.Check(osutil.IsDirectory(dirs.SnapRepairDir), Equals, false)
+	c.Check(osutil.FileExists(dirs.SnapRepairStateFile), Equals, false)
+	// setup realistic seed/assertions
+	r := sysdb.InjectTrusted(s.storeSigning.Trusted)
+	defer r()
+
+	const errPrefix = "cannot set device information: "
+	tests := []struct {
+		breakFunc   func()
+		expectedErr string
+	}{
+		{func() { s.rmSeedAssert16(c, "model") }, errPrefix + "no model assertion in seed data"},
+		{func() { s.rmSeedAssert16(c, "brand.account") }, errPrefix + "no brand account assertion in seed data"},
+		{func() { s.rmSeedAssert16(c, "brand.account-key") }, errPrefix + `cannot find public key.*`},
+		{func() {
+			// broken signature
+			blob := asserts.Encode(s.brandAcct)
+			err := ioutil.WriteFile(filepath.Join(s.seedAssertsDir, "brand.account"), blob[:len(blob)-3], 0644)
+			c.Assert(err, IsNil)
+		}, errPrefix + "cannot decode signature:.*"},
+		{func() { s.writeSeedAssert(c, "model2", s.modelAs) }, errPrefix + "multiple models in seed assertions"},
+	}
+
+	for _, test := range tests {
+		s.writeSeedAssert(c, "store.account-key", s.storeSigning.StoreAccountKey(""))
+		s.writeSeedAssert(c, "brand.account", s.brandAcct)
+		s.writeSeedAssert(c, "brand.account-key", s.brandAcctKey)
+		s.writeSeedAssert(c, "model", s.modelAs)
+
+		test.breakFunc()
+
+		runner := repair.NewRunner()
+		err := runner.LoadState()
+		c.Check(err, ErrorMatches, test.expectedErr)
+	}
+}
+
+type runner20Suite struct {
+	shared1620RunnerSuite
+}
+
+var _ = Suite(&runner20Suite{})
+
+var mockModeenv = []byte(`
+mode=run
+model=my-brand/my-model-2
+base=core20_1.snap
+`)
+
+func (s *runner20Suite) SetUpTest(c *C) {
+	s.shared1620RunnerSuite.SetUpTest(c)
+
+	s.seedAssertsDir = filepath.Join(dirs.SnapSeedDir, "/systems/20201212/assertions")
+	err := os.MkdirAll(s.seedAssertsDir, 0755)
+	c.Assert(err, IsNil)
+
+	// write dummy modeenv
+	err = os.MkdirAll(filepath.Dir(dirs.SnapModeenvFile), 0755)
+	c.Assert(err, IsNil)
+	err = ioutil.WriteFile(dirs.SnapModeenvFile, mockModeenv, 0644)
+	c.Assert(err, IsNil)
+	// validate that modeenv is actually valid
+	_, err = boot.ReadModeenv("")
+	c.Assert(err, IsNil)
+
+	seedTime, err := time.Parse(time.RFC3339, "2017-08-11T15:49:49Z")
+	c.Assert(err, IsNil)
+	err = os.Chtimes(dirs.SnapModeenvFile, seedTime, seedTime)
+	c.Assert(err, IsNil)
+	s.seedTime = seedTime
+	s.t0 = time.Now().UTC().Truncate(time.Minute)
+
+	s.writeSeedAssert = s.writeSeedAssert20
+}
+
+func (s *runner20Suite) writeSeedAssert20(c *C, fname string, a asserts.Assertion) {
+	var fn string
+	if _, ok := a.(*asserts.Model); ok {
+		fn = filepath.Join(s.seedAssertsDir, "../model")
+	} else {
+		fn = filepath.Join(s.seedAssertsDir, fname)
+	}
+	err := ioutil.WriteFile(fn, asserts.Encode(a), 0644)
+	c.Assert(err, IsNil)
+
+	// ensure model assertion file has the correct seed time
+	if _, ok := a.(*asserts.Model); ok {
+		err = os.Chtimes(fn, s.seedTime, s.seedTime)
+		c.Assert(err, IsNil)
+	}
+}
+
+func (s *runner20Suite) TestLoadStateInitDeviceInfoModeenvInvalidContent(c *C) {
+	runner := repair.NewRunner()
+
+	for _, tc := range []struct {
+		modelStr    string
+		expectedErr string
+	}{
+		{
+			`invalid-key-value`,
+			"cannot set device information: No option model in section ",
+		}, {
+			`model=`,
+			`cannot set device information: cannot find brand/model in modeenv model string ""`,
+		}, {
+			`model=brand-but-no-model`,
+			`cannot set device information: cannot find brand/model in modeenv model string "brand-but-no-model"`,
+		},
+	} {
+		err := ioutil.WriteFile(dirs.SnapModeenvFile, []byte(tc.modelStr), 0644)
+		c.Assert(err, IsNil)
+		err = runner.LoadState()
+		c.Check(err, ErrorMatches, tc.expectedErr)
+	}
+}
+
+func (s *runner20Suite) TestLoadStateInitDeviceInfoModeenvIncorrectPermissions(c *C) {
+	runner := repair.NewRunner()
+
+	err := os.Chmod(dirs.SnapModeenvFile, 0300)
+	c.Assert(err, IsNil)
+	s.AddCleanup(func() {
+		err := os.Chmod(dirs.SnapModeenvFile, 0644)
+		c.Assert(err, IsNil)
+	})
+	err = runner.LoadState()
+	c.Check(err, ErrorMatches, "cannot set device information: open /.*/modeenv: permission denied")
+}
