@@ -28,8 +28,10 @@ import (
 	"strings"
 	"syscall"
 
+	"github.com/snapcore/snapd/features"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/osutil/mount"
 )
 
 // Action represents a mount action (mount, remount, unmount, etc).
@@ -93,10 +95,10 @@ func (c *Change) createPath(path string, pokeHoles bool, as *Assumptions) ([]*Ch
 
 	// In case we need to create something, some constants.
 	const (
-		mode = 0755
-		uid  = 0
-		gid  = 0
+		uid = 0
+		gid = 0
 	)
+	mode := as.ModeForPath(path)
 
 	// If the element doesn't exist we can attempt to create it.  We will
 	// create the parent directory and then the final element relative to it.
@@ -119,6 +121,7 @@ func (c *Change) createPath(path string, pokeHoles bool, as *Assumptions) ([]*Ch
 	if needsMimic, mimicPath := mimicRequired(err); needsMimic && pokeHoles {
 		// If the error can be recovered by using a writable mimic
 		// then construct one and try again.
+		logger.Debugf("need to create writable mimic needed to create path %q (original error: %v)", path, err)
 		changes, err = createWritableMimic(mimicPath, path, as)
 		if err != nil {
 			err = fmt.Errorf("cannot create writable mimic over %q: %s", mimicPath, err)
@@ -306,13 +309,23 @@ func (c *Change) lowLevelPerform(as *Assumptions) error {
 				flagsForMount = uintptr(maskedFlagsNotPropagationNotRecursive)
 				err = sysMount(c.Entry.Name, c.Entry.Dir, c.Entry.Type, uintptr(flagsForMount), strings.Join(unparsed, ","))
 			}
-			logger.Debugf("mount %q %q %q %d %q (error: %v)", c.Entry.Name, c.Entry.Dir, c.Entry.Type, flagsForMount, strings.Join(unparsed, ","), err)
+			mountOpts, unknownFlags := mount.MountFlagsToOpts(int(flagsForMount))
+			if unknownFlags != 0 {
+				mountOpts = append(mountOpts, fmt.Sprintf("%#x", unknownFlags))
+			}
+			logger.Debugf("mount name:%q dir:%q type:%q opts:%s unparsed:%q (error: %v)",
+				c.Entry.Name, c.Entry.Dir, c.Entry.Type, strings.Join(mountOpts, "|"), strings.Join(unparsed, ","), err)
 			if err == nil && maskedFlagsPropagation != 0 {
 				// now change mount propagation (shared/rshared, private/rprivate,
 				// slave/rslave, unbindable/runbindable).
 				flagsForMount := uintptr(maskedFlagsPropagation | maskedFlagsRecursive)
+				mountOpts, unknownFlags := mount.MountFlagsToOpts(int(flagsForMount))
+				if unknownFlags != 0 {
+					mountOpts = append(mountOpts, fmt.Sprintf("%#x", unknownFlags))
+				}
 				err = sysMount("none", c.Entry.Dir, "", flagsForMount, "")
-				logger.Debugf("mount %q %q %q %d %q (error: %v)", "none", c.Entry.Dir, "", flagsForMount, "", err)
+				logger.Debugf("mount name:%q dir:%q type:%q opts:%s unparsed:%q (error: %v)",
+					"none", c.Entry.Dir, "", strings.Join(mountOpts, "|"), strings.Join(unparsed, ","), err)
 			}
 			if err == nil {
 				as.AddChange(c)
@@ -343,25 +356,54 @@ func (c *Change) lowLevelPerform(as *Assumptions) error {
 			// Perform the raw unmount operation.
 			if err == nil {
 				err = sysUnmount(c.Entry.Dir, flags)
+				umountOpts, unknownFlags := mount.UnmountFlagsToOpts(flags)
+				if unknownFlags != 0 {
+					umountOpts = append(umountOpts, fmt.Sprintf("%#x", unknownFlags))
+				}
+				logger.Debugf("umount %q %s (error: %v)", c.Entry.Dir, strings.Join(umountOpts, "|"), err)
+				if err == syscall.EINVAL {
+					// We attempted to unmount but got an EINVAL, one of the
+					// possibilities and the only one unless we provided wrong
+					// flags, is that the mount no longer exists.
+					//
+					// We can verify that now by scanning mountinfo:
+					entries, _ := osutil.LoadMountInfo()
+					for _, entry := range entries {
+						if entry.MountDir == c.Entry.Dir {
+							// Mount point still exists, EINVAL was unexpected.
+							return err
+						}
+					}
+					// We didn't find a mount point at the location we tried to
+					// unmount. The EINVAL we observed indicates that the mount
+					// profile no longer agrees with reality. The mount point
+					// no longer exists. As such, consume the error and carry on.
+					logger.Debugf("ignoring EINVAL from unmount, %q is not mounted", c.Entry.Dir)
+					err = nil
+				}
+				if err != nil {
+					return err
+				}
 			}
 			if err == nil {
 				as.AddChange(c)
 			}
-			logger.Debugf("umount %q (error: %v)", c.Entry.Dir, err)
-			if err != nil {
-				return err
-			}
 
 			// Open a path of the file we are considering the removal of.
 			path := c.Entry.Dir
 			var fd int
 			fd, err = OpenPath(path)
+			// If the place does not exist anymore, we are done.
+			if os.IsNotExist(err) {
+				return nil
+			}
 			if err != nil {
 				return err
 			}
 			defer sysClose(fd)
 
 			// Don't attempt to remove anything from squashfs.
+			// Note that this is not a perfect check and we also handle EROFS below.
 			var statfsBuf syscall.Statfs_t
 			err = sysFstatfs(fd, &statfsBuf)
 			if err != nil {
@@ -388,10 +430,28 @@ func (c *Change) lowLevelPerform(as *Assumptions) error {
 			// no way to avoid a race here since there's no way to unlink a
 			// file solely by file descriptor.
 			err = osRemove(path)
+			logger.Debugf("remove %q (error: %v)", path, err)
 			// Unpack the low-level error that osRemove wraps into PathError.
 			if packed, ok := err.(*os.PathError); ok {
 				err = packed.Err
 			}
+			if err == syscall.EROFS {
+				// If the underlying medium is read-only then ignore the error.
+				// Instead of checking up front we just try to remove because
+				// of https://bugs.launchpad.net/snapd/+bug/1867752 which showed us
+				// two important properties:
+				// 1) inside containers we cannot detect squashfs reliably and
+				//    will always see FUSE instead. The problem is that there's no
+				//    indication as to what is really mounted via statfs(2) and
+				//    we would have to deduce that from mountinfo, trusting
+				//    that fuse.<name> is not spoofed (as in, the name is not
+				//    spoofed).
+				// 2) rmdir of a bind mount (from a normal writable filesystem like ext4)
+				//    over a read-only filesystem also yields EROFS without any indication
+				//    that this is to be expected.
+				logger.Debugf("cannot remove a mount point on read-only filesystem %q", path)
+				return nil
+			}
 			// If we were removing a directory but it was not empty then just
 			// ignore the error. This is the equivalent of the non-empty file
 			// check we do above. See rmdir(2) for explanation why we accept
@@ -399,6 +459,23 @@ func (c *Change) lowLevelPerform(as *Assumptions) error {
 			if kind == "" && (err == syscall.ENOTEMPTY || err == syscall.EEXIST) {
 				return nil
 			}
+			if features.RobustMountNamespaceUpdates.IsEnabled() {
+				// FIXME: This should not be necessary. It is necessary because
+				// mimic construction code is not considering all layouts in tandem
+				// and doesn't know enough about base file system to construct
+				// mimics in the order that would prevent them from nesting.
+				//
+				// By ignoring EBUSY here and by continuing to tear down the mimic
+				// tmpfs entirely (without any reuse) we guarantee that at the end
+				// of the day the nested mimic case is entirely removed.
+				//
+				// In an ideal world we would model this better and could do
+				// without this edge case.
+				if (kind == "" || kind == "file") && err == syscall.EBUSY {
+					logger.Debugf("cannot remove busy mount point %q", path)
+					return nil
+				}
+			}
 		}
 		return err
 	case Keep:
@@ -408,16 +485,9 @@ func (c *Change) lowLevelPerform(as *Assumptions) error {
 	return fmt.Errorf("cannot process mount change: unknown action: %q", c.Action)
 }
 
-// NeededChanges computes the changes required to change current to desired mount entries.
-//
-// The current and desired profiles is a fstab like list of mount entries. The
-// lists are processed and a "diff" of mount changes is produced. The mount
-// changes, when applied in order, transform the current profile into the
-// desired profile.
-var NeededChanges = neededChangesImpl
-
-// neededChangesImpl is the real implementation of NeededChanges
-func neededChangesImpl(currentProfile, desiredProfile *osutil.MountProfile) []*Change {
+// neededChangesOld is the real implementation of NeededChanges
+// This function is used when RobustMountNamespaceUpdate is not enabled.
+func neededChangesOld(currentProfile, desiredProfile *osutil.MountProfile) []*Change {
 	// Copy both profiles as we will want to mutate them.
 	current := make([]osutil.MountEntry, len(currentProfile.Entries))
 	copy(current, currentProfile.Entries)
@@ -530,3 +600,97 @@ func neededChangesImpl(currentProfile, desiredProfile *osutil.MountProfile) []*C
 
 	return changes
 }
+
+// neededChangesNew is the real implementation of NeededChanges
+// This function is used when RobustMountNamespaceUpdate is enabled.
+func neededChangesNew(currentProfile, desiredProfile *osutil.MountProfile) []*Change {
+	// Copy both profiles as we will want to mutate them.
+	current := make([]osutil.MountEntry, len(currentProfile.Entries))
+	copy(current, currentProfile.Entries)
+	desired := make([]osutil.MountEntry, len(desiredProfile.Entries))
+	copy(desired, desiredProfile.Entries)
+
+	// Clean the directory part of both profiles. This is done so that we can
+	// easily test if a given directory is a subdirectory with
+	// strings.HasPrefix coupled with an extra slash character.
+	for i := range current {
+		current[i].Dir = filepath.Clean(current[i].Dir)
+	}
+	for i := range desired {
+		desired[i].Dir = filepath.Clean(desired[i].Dir)
+	}
+
+	// Sort both lists by directory name with implicit trailing slash.
+	sort.Sort(byOriginAndMagicDir(current))
+	sort.Sort(byOriginAndMagicDir(desired))
+
+	// We are now ready to compute the necessary mount changes.
+	var changes []*Change
+
+	// Unmount entries in reverse order, so that the most nested element is
+	// always processed first.
+	for i := len(current) - 1; i >= 0; i-- {
+		var entry osutil.MountEntry = current[i]
+		entry.Options = append([]string(nil), entry.Options...)
+		switch {
+		case entry.XSnapdSynthetic() && entry.Type == "tmpfs":
+			// Synthetic changes are rooted under a tmpfs, detach that tmpfs to
+			// remove them all.
+			if !entry.XSnapdDetach() {
+				entry.Options = append(entry.Options, osutil.XSnapdDetach())
+			}
+		case entry.XSnapdSynthetic():
+			// Consume all other syn ethic entries without emitting either a
+			// mount, unmount or keep change.  This relies on the fact that all
+			// synthetic mounts are created by a mimic underneath a tmpfs that
+			// is detached, as coded above.
+			continue
+		case entry.OptBool("rbind") || entry.Type == "tmpfs":
+			// Recursive bind mounts and non-mimic tmpfs mounts need to be
+			// detached because they can contain other mount points that can
+			// otherwise propagate in a self-conflicting way.
+			if !entry.XSnapdDetach() {
+				entry.Options = append(entry.Options, osutil.XSnapdDetach())
+			}
+		case entry.OptBool("bind") && entry.XSnapdKind() == "file":
+			// Bind mounted files are detached. If a bind mounted file open or
+			// mapped into a process as a library, then attempting to unmount
+			// it will result in EBUSY.
+			//
+			// This can happen when a snap has a service, for example one using
+			// a library mounted via a bind mount and an absent content
+			// connection. Subsequent connection of the content connection will
+			// trigger re-population of the mount namespace, which will start
+			// by tearing down the existing file bind-mount. To prevent this,
+			// detach the mount instead.
+			if !entry.XSnapdDetach() {
+				entry.Options = append(entry.Options, osutil.XSnapdDetach())
+			}
+		}
+		// Unmount all changes that were not eliminated.
+		changes = append(changes, &Change{Action: Unmount, Entry: entry})
+	}
+
+	// Mount desired entries.
+	for i := range desired {
+		changes = append(changes, &Change{Action: Mount, Entry: desired[i]})
+	}
+
+	return changes
+}
+
+// NeededChanges computes the changes required to change current to desired mount entries.
+//
+// The algorithm differs depending on the value of the robust mount namespace
+// updates feature flag. If the flag is enabled then the current profile is
+// entirely undone and the desired profile is constructed from scratch.
+//
+// If the flag is disabled then a diff-like operation on the mount profile is
+// computed. Some of the mount entries from the current profile may be reused.
+// The diff approach doesn't function correctly in cases of nested mimics.
+var NeededChanges = func(current, desired *osutil.MountProfile) []*Change {
+	if features.RobustMountNamespaceUpdates.IsEnabled() {
+		return neededChangesNew(current, desired)
+	}
+	return neededChangesOld(current, desired)
+}
