@@ -23,8 +23,7 @@ import (
 	"os"
 	"syscall"
 
-	. "gopkg.in/check.v1"
-
+	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/sys"
 )
 
@@ -32,23 +31,33 @@ var (
 	// change
 	ValidateInstanceName = validateInstanceName
 	ProcessArguments     = processArguments
-	// freezer
-	FreezeSnapProcesses = freezeSnapProcesses
-	ThawSnapProcesses   = thawSnapProcesses
+
 	// utils
 	PlanWritableMimic = planWritableMimic
 	ExecWritableMimic = execWritableMimic
 
-	// main
-	ComputeAndSaveChanges = computeAndSaveChanges
-	ApplyUserFstab        = applyUserFstab
-
 	// bootstrap
 	ClearBootstrapError = clearBootstrapError
 
 	// trespassing
 	IsReadOnly                   = isReadOnly
 	IsPrivateTmpfsCreatedBySnapd = isPrivateTmpfsCreatedBySnapd
+
+	// system
+	DesiredSystemProfilePath = desiredSystemProfilePath
+	CurrentSystemProfilePath = currentSystemProfilePath
+
+	// user
+	DesiredUserProfilePath = desiredUserProfilePath
+	CurrentUserProfilePath = currentUserProfilePath
+
+	// xdg
+	XdgRuntimeDir        = xdgRuntimeDir
+	ExpandPrefixVariable = expandPrefixVariable
+	ExpandXdgRuntimeDir  = expandXdgRuntimeDir
+
+	// update
+	ExecuteMountProfileUpdate = executeMountProfileUpdate
 )
 
 // SystemCalls encapsulates various system interactions performed by this module.
@@ -134,18 +143,6 @@ func MockSystemCalls(sc SystemCalls) (restore func()) {
 	}
 }
 
-func MockFreezerCgroupDir(c *C) (restore func()) {
-	old := freezerCgroupDir
-	freezerCgroupDir = c.MkDir()
-	return func() {
-		freezerCgroupDir = old
-	}
-}
-
-func FreezerCgroupDir() string {
-	return freezerCgroupDir
-}
-
 func MockChangePerform(f func(chg *Change, as *Assumptions) ([]*Change, error)) func() {
 	origChangePerform := changePerform
 	changePerform = f
@@ -154,6 +151,14 @@ func MockChangePerform(f func(chg *Change, as *Assumptions) ([]*Change, error))
 	}
 }
 
+func MockNeededChanges(f func(old, new *osutil.MountProfile) []*Change) (restore func()) {
+	origNeededChanges := NeededChanges
+	NeededChanges = f
+	return func() {
+		NeededChanges = origNeededChanges
+	}
+}
+
 func MockReadDir(fn func(string) ([]os.FileInfo, error)) (restore func()) {
 	old := ioutilReadDir
 	ioutilReadDir = fn
@@ -181,3 +186,32 @@ func (as *Assumptions) PastChanges() []*Change {
 func (as *Assumptions) CanWriteToDirectory(dirFd int, dirName string) (bool, error) {
 	return as.canWriteToDirectory(dirFd, dirName)
 }
+
+func (as *Assumptions) UnrestrictedPaths() []string {
+	return as.unrestrictedPaths
+}
+
+func (upCtx *CommonProfileUpdateContext) CurrentProfilePath() string {
+	return upCtx.currentProfilePath
+}
+
+func (upCtx *CommonProfileUpdateContext) DesiredProfilePath() string {
+	return upCtx.desiredProfilePath
+}
+
+func (upCtx *CommonProfileUpdateContext) FromSnapConfine() bool {
+	return upCtx.fromSnapConfine
+}
+
+func (upCtx *CommonProfileUpdateContext) SetFromSnapConfine(v bool) {
+	upCtx.fromSnapConfine = v
+}
+
+func NewCommonProfileUpdateContext(instanceName string, fromSnapConfine bool, currentProfilePath, desiredProfilePath string) *CommonProfileUpdateContext {
+	return &CommonProfileUpdateContext{
+		instanceName:       instanceName,
+		fromSnapConfine:    fromSnapConfine,
+		currentProfilePath: currentProfilePath,
+		desiredProfilePath: desiredProfilePath,
+	}
+}
