@@ -26,15 +26,24 @@ import (
 )
 
 type cmdGetBaseDeclaration struct {
+	get bool
 	clientMixin
 }
 
 func init() {
 	cmd := addDebugCommand("get-base-declaration",
+		"(internal) obtain the base declaration for all interfaces (deprecated)",
+		"(internal) obtain the base declaration for all interfaces (deprecated)",
+		func() flags.Commander {
+			return &cmdGetBaseDeclaration{}
+		}, nil, nil)
+	cmd.hidden = true
+
+	cmd = addDebugCommand("base-declaration",
 		"(internal) obtain the base declaration for all interfaces",
 		"(internal) obtain the base declaration for all interfaces",
 		func() flags.Commander {
-			return &cmdGetBaseDeclaration{}
+			return &cmdGetBaseDeclaration{get: true}
 		}, nil, nil)
 	cmd.hidden = true
 }
@@ -46,7 +55,13 @@ func (x *cmdGetBaseDeclaration) Execute(args []string) error {
 	var resp struct {
 		BaseDeclaration string `json:"base-declaration"`
 	}
-	if err := x.client.Debug("get-base-declaration", nil, &resp); err != nil {
+	var err error
+	if x.get {
+		err = x.client.DebugGet("base-declaration", &resp, nil)
+	} else {
+		err = x.client.Debug("get-base-declaration", nil, &resp)
+	}
+	if err != nil {
 		return err
 	}
 	fmt.Fprintf(Stdout, "%s\n", resp.BaseDeclaration)
