@@ -31,16 +31,15 @@ import (
 	"strings"
 	"testing"
 
+	"github.com/jessevdk/go-flags"
+	"golang.org/x/crypto/ssh/terminal"
 	. "gopkg.in/check.v1"
 
-	"golang.org/x/crypto/ssh/terminal"
-
-	"github.com/snapcore/snapd/cmd"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
-	snapdsnap "github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snapdtool"
 	"github.com/snapcore/snapd/testutil"
 
 	snap "github.com/snapcore/snapd/cmd/snap"
@@ -67,6 +66,12 @@ func (s *BaseSnapSuite) SetUpTest(c *C) {
 	s.BaseTest.SetUpTest(c)
 	dirs.SetRootDir(c.MkDir())
 
+	path := os.Getenv("PATH")
+	s.AddCleanup(func() {
+		os.Setenv("PATH", path)
+	})
+	os.Setenv("PATH", path+":"+dirs.SnapBinariesDir)
+
 	s.stdin = bytes.NewBuffer(nil)
 	s.stdout = bytes.NewBuffer(nil)
 	s.stderr = bytes.NewBuffer(nil)
@@ -79,8 +84,6 @@ func (s *BaseSnapSuite) SetUpTest(c *C) {
 	s.AuthFile = filepath.Join(c.MkDir(), "json")
 	os.Setenv(TestAuthFileEnvKey, s.AuthFile)
 
-	s.AddCleanup(snapdsnap.MockSanitizePlugsSlots(func(snapInfo *snapdsnap.Info) {}))
-
 	s.AddCleanup(interfaces.MockSystemKey(`
 {
 "build-id": "7a94e9736c091b3984bd63f5aebfc883c4d859e0",
@@ -95,6 +98,10 @@ func (s *BaseSnapSuite) SetUpTest(c *C) {
 	s.AddCleanup(snap.MockIsStdinTTY(false))
 
 	s.AddCleanup(snap.MockSELinuxIsEnabled(func() (bool, error) { return false, nil }))
+
+	// mock an empty cmdline since we check the cmdline to check whether we are
+	// in install mode or not and we don't want to use the host's proc/cmdline
+	s.AddCleanup(osutil.MockProcCmdline(filepath.Join(c.MkDir(), "proc/cmdline")))
 }
 
 func (s *BaseSnapSuite) TearDownTest(c *C) {
@@ -171,12 +178,6 @@ func mockArgs(args ...string) (restore func()) {
 	return func() { os.Args = old }
 }
 
-func mockVersion(v string) (restore func()) {
-	old := cmd.Version
-	cmd.Version = v
-	return func() { cmd.Version = old }
-}
-
 func mockSnapConfine(libExecDir string) func() {
 	snapConfine := filepath.Join(libExecDir, "snap-confine")
 	if err := os.MkdirAll(libExecDir, 0755); err != nil {
@@ -234,7 +235,7 @@ func (s *SnapSuite) TestVersionOnClassic(c *C) {
 	})
 	restore := mockArgs("snap", "--version")
 	defer restore()
-	restore = mockVersion("4.56")
+	restore = snapdtool.MockVersion("4.56")
 	defer restore()
 
 	c.Assert(func() { snap.RunMain() }, PanicMatches, `internal error: exitStatus\{0\} .*`)
@@ -248,7 +249,7 @@ func (s *SnapSuite) TestVersionOnAllSnap(c *C) {
 	})
 	restore := mockArgs("snap", "--version")
 	defer restore()
-	restore = mockVersion("4.56")
+	restore = snapdtool.MockVersion("4.56")
 	defer restore()
 
 	c.Assert(func() { snap.RunMain() }, PanicMatches, `internal error: exitStatus\{0\} .*`)
@@ -261,7 +262,7 @@ func (s *SnapSuite) TestUnknownCommand(c *C) {
 	defer restore()
 
 	err := snap.RunMain()
-	c.Assert(err, ErrorMatches, `unknown command "unknowncmd", see 'snap help'`)
+	c.Assert(err, ErrorMatches, `unknown command "unknowncmd", see 'snap help'.`)
 }
 
 func (s *SnapSuite) TestResolveApp(c *C) {
@@ -364,6 +365,10 @@ func (s *SnapSuite) TestLintDesc(c *C) {
 	}
 	c.Check(fn, PanicMatches, `option on "command" has no name`)
 	log.Reset()
+
+	snap.LintDesc("snap-advise", "from-apt", "snap-advise will run as a hook", "")
+	c.Check(log.String(), HasLen, 0)
+	log.Reset()
 }
 
 func (s *SnapSuite) TestLintArg(c *C) {
@@ -398,3 +403,29 @@ func (s *SnapSuite) TestFixupArg(c *C) {
 	// Trailing ">s" is fixed to just >.
 	c.Check(snap.FixupArg("<option>s"), Equals, "<option>")
 }
+
+func (s *SnapSuite) TestSetsUserAgent(c *C) {
+	testServerHit := false
+	s.RedirectClientToTestServer(func(w http.ResponseWriter, r *http.Request) {
+		c.Check(r.Header.Get("User-Agent"), Matches, "snapd/.*")
+		testServerHit = true
+
+		fmt.Fprintln(w, `{"type": "error", "result": {"message": "cannot do something"}}`)
+	})
+	restore := mockArgs("snap", "install", "foo")
+	defer restore()
+
+	_ = snap.RunMain()
+	c.Assert(testServerHit, Equals, true)
+}
+
+func (s *SnapSuite) TestCompletionHandlerSkipsHidden(c *C) {
+	snap.MarkForNoCompletion(snap.HiddenCmd("bar yadda yack", false))
+	snap.MarkForNoCompletion(snap.HiddenCmd("bar yack yack yack", true))
+	snap.CompletionHandler([]flags.Completion{
+		{Item: "foo", Description: "foo yadda yadda"},
+		{Item: "bar", Description: "bar yadda yack"},
+		{Item: "baz", Description: "bar yack yack yack"},
+	})
+	c.Check(s.Stdout(), Equals, "foo\nbaz\n")
+}
