@@ -42,7 +42,9 @@ const accountControlConnectedPlugAppArmor = `
 /{,usr/}sbin/chpasswd ixr,
 /{,usr/}sbin/user{add,del} ixr,
 
-# Only allow modifying the non-system extrausers database
+# Allow modifying the non-system extrausers NSS database. The extrausers
+# database is used on Ubuntu Core devices to manage both privileged and
+# unprivileged users (since /etc/passwd, /etc/group, etc are all read-only).
 /var/lib/extrausers/ r,
 /var/lib/extrausers/** rwkl,
 
@@ -52,6 +54,9 @@ const accountControlConnectedPlugAppArmor = `
 /etc/default/nss r,
 /etc/pam.d/{,*} r,
 
+# Needed by chpasswd
+/lib/@{multiarch}/security/* ixr,
+
 # Useradd needs netlink
 network netlink raw,
 
@@ -94,7 +99,7 @@ func makeAccountControlSecCompSnippet() (string, error) {
 	return snippet, nil
 }
 
-func (iface *accountControlInterface) SecCompConnectedPlug(spec *seccomp.Specification, plug *interfaces.Plug, Attrs map[string]interface{}, slot *interfaces.Slot, slotAttrs map[string]interface{}) error {
+func (iface *accountControlInterface) SecCompConnectedPlug(spec *seccomp.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
 	if iface.secCompSnippet == "" {
 		// Cache the snippet after it's successfully computed once
 		snippet, err := makeAccountControlSecCompSnippet()
@@ -117,6 +122,5 @@ func init() {
 		connectedPlugAppArmor: accountControlConnectedPlugAppArmor,
 		// handled by SecCompConnectedPlug
 		connectedPlugSecComp: "",
-		reservedForOS:        true,
 	}})
 }
