@@ -33,7 +33,25 @@ const cpuControlConnectedPlugAppArmor = `
 # Description: This interface allows for setting CPU tunables
 /sys/devices/system/cpu/**/ r,
 /sys/devices/system/cpu/cpu*/online rw,
-/sys/devices/system/cpu/smt/control rw,
+/sys/devices/system/cpu/smt/*       r,
+/sys/devices/system/cpu/smt/control w,
+
+# https://www.kernel.org/doc/html/latest/admin-guide/pm/cpufreq.html#policy-interface-in-sysfs
+/sys/devices/system/cpu/cpufreq/{,**} r,
+/sys/devices/system/cpu/cpufreq/policy*/energy_performance_preference w,
+/sys/devices/system/cpu/cpufreq/policy*/scaling_governor w,
+/sys/devices/system/cpu/cpufreq/policy*/scaling_max_freq w,
+/sys/devices/system/cpu/cpufreq/policy*/scaling_min_freq w,
+/sys/devices/system/cpu/cpufreq/policy*/scaling_setspeed w,
+/sys/devices/system/cpu/cpufreq/boost w,
+
+# https://www.kernel.org/doc/html/latest/admin-guide/pm/intel_pstate.html#user-space-interface-in-sysfs
+/sys/devices/system/cpu/intel_pstate/{,*} r,
+/sys/devices/system/cpu/intel_pstate/hwp_dynamic_boost w,
+/sys/devices/system/cpu/intel_pstate/max_perf_pct w,
+/sys/devices/system/cpu/intel_pstate/min_perf_pct w,
+/sys/devices/system/cpu/intel_pstate/no_turbo w,
+/sys/devices/system/cpu/intel_pstate/status w,
 `
 
 func init() {
@@ -44,6 +62,5 @@ func init() {
 		implicitOnClassic:     true,
 		baseDeclarationSlots:  cpuControlBaseDeclarationSlots,
 		connectedPlugAppArmor: cpuControlConnectedPlugAppArmor,
-		reservedForOS:         true,
 	})
 }
