@@ -20,16 +20,13 @@
 package builtin
 
 import (
-	"bytes"
-	"fmt"
-
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/apparmor"
 	"github.com/snapcore/snapd/interfaces/mount"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/release"
-	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/strutil"
 )
 
 const desktopSummary = `allows access to basic graphical desktop resources`
@@ -60,6 +57,8 @@ dbus (send)
 owner @{HOME}/.local/share/fonts/{,**} r,
 /var/cache/fontconfig/   r,
 /var/cache/fontconfig/** mr,
+# some applications are known to mmap fonts
+/usr/{,local/}share/fonts/** m,
 
 # subset of gnome abstraction
 /etc/gtk-3.0/settings.ini r,
@@ -108,7 +107,23 @@ dbus (receive)
     bus=session
     path=/org/freedesktop/Notifications
     interface=org.freedesktop.Notifications
-    member={ActionInvoked,NotificationClosed}
+    member={ActionInvoked,NotificationClosed,NotificationReplied}
+    peer=(label=unconfined),
+
+# KDE Plasma's Inhibited property indicating "do not disturb" mode
+# https://invent.kde.org/plasma/plasma-workspace/-/blob/master/libnotificationmanager/dbus/org.freedesktop.Notifications.xml#L42
+dbus (send)
+    bus=session
+    path=/org/freedesktop/Notifications
+    interface=org.freedesktop.DBus.Properties
+    member="Get{,All}"
+    peer=(label=unconfined),
+
+dbus (receive)
+    bus=session
+    path=/org/freedesktop/Notifications
+    interface=org.freedesktop.DBus.Properties
+    member=PropertiesChanged
     peer=(label=unconfined),
 
 # DesktopAppInfo Launched
@@ -134,9 +149,43 @@ dbus (send)
   member="Get{,All}"
   peer=(label=unconfined),
 
+# Allow accessing the GNOME crypto services prompt APIs as used by
+# applications using libgcr (such as pinentry-gnome3) for secure pin
+# entry to unlock GPG keys etc. See:
+# https://developer.gnome.org/gcr/unstable/GcrPrompt.html
+# https://developer.gnome.org/gcr/unstable/GcrSecretExchange.html
+dbus (send)
+    bus=session
+    path=/org/gnome/keyring/Prompter
+    interface=org.gnome.keyring.internal.Prompter
+    member="{BeginPrompting,PerformPrompt,StopPrompting}"
+    peer=(label=unconfined),
+
+# While the DBus path is not snap-specific, by the time an application
+# registers the prompt path via DBus, Gcr will check that it isn't
+# already in use and send the client an error if it is. See:
+# https://github.com/snapcore/snapd/pull/7673#issuecomment-592229711
+dbus (receive)
+    bus=session
+    path=/org/gnome/keyring/Prompt/p[0-9]*
+    interface=org.gnome.keyring.internal.Prompter.Callback
+    member="{PromptReady,PromptDone}"
+    peer=(label=unconfined),
+
 # Allow use of snapd's internal 'xdg-open'
 /usr/bin/xdg-open ixr,
-/usr/share/applications/{,*} r,
+# While /usr/share/applications comes from the base runtime of the snap, it
+# has some things that snaps actually need, so allow access to those and deny
+# access to the others
+/usr/share/applications/ r,
+/usr/share/applications/mimeapps.list r,
+/usr/share/applications/xdg-open.desktop r,
+# silence noisy denials from desktop files in core* snaps that aren't usable by
+# snaps
+deny /usr/share/applications/python*.desktop r,
+deny /usr/share/applications/vim.desktop r,
+deny /usr/share/applications/snap-handle-link.desktop r,  # core16
+
 dbus (send)
     bus=session
     path=/
@@ -181,13 +230,17 @@ dbus (send)
     bus=session
     path=/io/snapcraft/Settings
     interface=io.snapcraft.Settings
-    member={Check,Get,Set}
+    member={Check,CheckSub,Get,GetSub,Set,SetSub}
     peer=(label=unconfined),
 
-## Allow access to xdg-document-portal file system.  Access control is
-## handled by bind mounting a snap-specific sub-tree to this location.
-owner /run/user/[0-9]*/doc/ r,
-owner /run/user/[0-9]*/doc/** rw,
+# Allow access to xdg-document-portal file system.  Access control is
+# handled by bind mounting a snap-specific sub-tree to this location
+# (ie, this is /run/user/<uid>/doc/by-app/snap.@{SNAP_INSTANCE_NAME}
+# on the host).
+owner /run/user/[0-9]*/doc/{,*/} r,
+# Allow rw access without owner match to the documents themselves since
+# the user guided the access and can specify anything DAC allows.
+/run/user/[0-9]*/doc/*/** rw,
 
 # Allow access to xdg-desktop-portal and xdg-document-portal
 dbus (receive, send)
@@ -201,48 +254,68 @@ dbus (receive, send)
     interface=org.freedesktop.DBus.Properties
     path=/org/freedesktop/portal/{desktop,documents}{,/**}
     peer=(label=unconfined),
+
+# The portals service is normally running and newer versions of
+# xdg-desktop-portal include AssumedAppArmor=unconfined. Since older
+# systems don't have this and because gtkfilechoosernativeportal.c relies on
+# service activation, allow sends to peer=(name=org.freedesktop.portal.Desktop)
+# for service activation.
+dbus (send)
+    bus=session
+    interface=org.freedesktop.portal.*
+    path=/org/freedesktop/portal/{desktop,documents}{,/**}
+    peer=(name=org.freedesktop.portal.Desktop),
+dbus (send)
+    bus=session
+    interface=org.freedesktop.DBus.Properties
+    path=/org/freedesktop/portal/{desktop,documents}{,/**}
+    peer=(name=org.freedesktop.portal.Desktop),
+
+# These accesses are noisy and applications can't do anything with the found
+# icon files, so explicitly deny to silence the denials
+deny /var/lib/snapd/desktop/icons/{,**/} r,
+
+# These accesses occur when flatpaks are on the system since it updates
+# XDG_DATA_DIRS to contain $HOME/.local/share/flatpak/exports/share. Until
+# we have better XDG_DATA_DIRS handling, silence these noisy denials.
+# https://github.com/snapcrafters/discord/issues/23#issuecomment-637607843
+deny @{HOME}/.local/share/flatpak/exports/share/** r,
+
+# Allow access to the IBus portal (IBUS_USE_PORTAL=1)
+dbus (send)
+      bus=session
+      path=/org/freedesktop/IBus
+      interface=org.freedesktop.IBus.Portal
+      member=CreateInputContext
+      peer=(name=org.freedesktop.portal.IBus),
+
+dbus (send, receive)
+      bus=session
+      path=/org/freedesktop/IBus/InputContext_[0-9]*
+      interface=org.freedesktop.IBus.InputContext
+      peer=(label=unconfined),
 `
 
-type desktopInterface struct{}
-
-func (iface *desktopInterface) Name() string {
-	return "desktop"
-}
-
-func (iface *desktopInterface) StaticInfo() interfaces.StaticInfo {
-	return interfaces.StaticInfo{
-		Summary:              desktopSummary,
-		ImplicitOnClassic:    true,
-		BaseDeclarationSlots: desktopBaseDeclarationSlots,
-	}
-}
-
-func (iface *desktopInterface) BeforePrepareSlot(slot *snap.SlotInfo) error {
-	return sanitizeSlotReservedForOS(iface, slot)
-}
-
-func (iface *desktopInterface) AutoConnect(*snap.PlugInfo, *snap.SlotInfo) bool {
-	// allow what declarations allowed
-	return true
+type desktopInterface struct {
+	commonInterface
 }
 
 func (iface *desktopInterface) fontconfigDirs() []string {
-	return []string{
+	fontDirs := []string{
 		dirs.SystemFontsDir,
 		dirs.SystemLocalFontsDir,
-		dirs.SystemFontconfigCacheDir,
 	}
+	return append(fontDirs, dirs.SystemFontconfigCacheDirs...)
 }
 
 func (iface *desktopInterface) AppArmorConnectedPlug(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
 	spec.AddSnippet(desktopConnectedPlugAppArmor)
 
 	// Allow mounting document portal
-	var buf bytes.Buffer
-	fmt.Fprintf(&buf, "  # Mount the document portal\n")
-	fmt.Fprintf(&buf, "  mount options=(bind) /run/user/[0-9]*/doc/by-app/snap.%s/ -> /run/user/[0-9]*/doc/,\n", plug.Snap().InstanceName())
-	fmt.Fprintf(&buf, "  umount /run/user/[0-9]*/doc/,\n\n")
-	spec.AddUpdateNS(buf.String())
+	emit := spec.AddUpdateNSf
+	emit("  # Mount the document portal\n")
+	emit("  mount options=(bind) /run/user/[0-9]*/doc/by-app/snap.%s/ -> /run/user/[0-9]*/doc/,\n", plug.Snap().InstanceName())
+	emit("  umount /run/user/[0-9]*/doc/,\n\n")
 
 	if !release.OnClassic {
 		// We only need the font mount rules on classic systems
@@ -251,14 +324,12 @@ func (iface *desktopInterface) AppArmorConnectedPlug(spec *apparmor.Specificatio
 
 	// Allow mounting fonts
 	for _, dir := range iface.fontconfigDirs() {
-		var buf bytes.Buffer
 		source := "/var/lib/snapd/hostfs" + dir
 		target := dirs.StripRootDir(dir)
-		fmt.Fprintf(&buf, "  # Read-only access to %s\n", target)
-		fmt.Fprintf(&buf, "  mount options=(bind) %s/ -> %s/,\n", source, target)
-		fmt.Fprintf(&buf, "  remount options=(bind, ro) %s/,\n", target)
-		fmt.Fprintf(&buf, "  umount %s/,\n\n", target)
-		spec.AddUpdateNS(buf.String())
+		emit("  # Read-only access to %s\n", target)
+		emit("  mount options=(bind) %s/ -> %s/,\n", source, target)
+		emit("  remount options=(bind, ro) %s/,\n", target)
+		emit("  umount %s/,\n\n", target)
 	}
 
 	return nil
@@ -281,6 +352,19 @@ func (iface *desktopInterface) MountConnectedPlug(spec *mount.Specification, plu
 		if !osutil.IsDirectory(dir) {
 			continue
 		}
+		if release.DistroLike("arch", "fedora") {
+			// XXX: on Arch and Fedora 32+ there is a known
+			// incompatibility between the binary fonts cache files
+			// and ones expected by desktop snaps; even though the
+			// cache format level is same for both, the host
+			// generated cache files cause instability, segfaults or
+			// incorrect rendering of fonts, for this reason do not
+			// mount the cache directories on those distributions,
+			// see https://bugs.launchpad.net/snapd/+bug/1877109
+			if strutil.ListContains(dirs.SystemFontconfigCacheDirs, dir) {
+				continue
+			}
+		}
 		// Since /etc/fonts/fonts.conf in the snap mount ns is the same
 		// as on the host, we need to preserve the original directory
 		// paths for the fontconfig runtime to poke the correct
@@ -296,5 +380,12 @@ func (iface *desktopInterface) MountConnectedPlug(spec *mount.Specification, plu
 }
 
 func init() {
-	registerIface(&desktopInterface{})
+	registerIface(&desktopInterface{
+		commonInterface: commonInterface{
+			name:                 "desktop",
+			summary:              desktopSummary,
+			implicitOnClassic:    true,
+			baseDeclarationSlots: desktopBaseDeclarationSlots,
+		},
+	})
 }
