@@ -21,10 +21,12 @@ package builtin
 
 import (
 	"fmt"
+	"os"
 
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/apparmor"
 	"github.com/snapcore/snapd/interfaces/systemd"
+	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/snap"
 )
 
@@ -63,10 +65,6 @@ func (iface *gpioInterface) StaticInfo() interfaces.StaticInfo {
 
 // BeforePrepareSlot checks the slot definition is valid
 func (iface *gpioInterface) BeforePrepareSlot(slot *snap.SlotInfo) error {
-	if err := sanitizeSlotReservedForOSOrGadget(iface, slot); err != nil {
-		return err
-	}
-
 	// Must have a GPIO number
 	number, ok := slot.Attrs["number"]
 	if !ok {
@@ -93,12 +91,18 @@ func (iface *gpioInterface) AppArmorConnectedPlug(spec *apparmor.Specification,
 	// requires symlinks to be dereferenced, evaluate the GPIO
 	// path and add the correct absolute path to the AppArmor snippet.
 	dereferencedPath, err := evalSymlinks(path)
+	if err != nil && os.IsNotExist(err) {
+		// If the specific gpio is not available there is no point
+		// exporting it, we should also not fail because this
+		// will block snapd updates (LP: 1866424)
+		logger.Noticef("cannot export not existing gpio %s", path)
+		return nil
+	}
 	if err != nil {
 		return err
 	}
 	spec.AddSnippet(fmt.Sprintf("%s/* rwk,", dereferencedPath))
 	return nil
-
 }
 
 func (iface *gpioInterface) SystemdConnectedSlot(spec *systemd.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
