@@ -62,8 +62,6 @@ owner @{HOME}/.local/share/fonts/{,**} r,
 
 # subset of gnome abstraction
 /etc/gnome/defaults.list r,
-/usr/share/gnome/applications/             r,
-/usr/share/applications/mimeinfo.cache     r,
 
 /etc/gtk-*/*                               r,
 /usr/lib{,32,64}/gtk-*/**                  mr,
@@ -80,22 +78,29 @@ owner @{HOME}/.local/share/fonts/{,**} r,
 /usr/share/icons/*/index.theme             rk,
 /usr/share/pixmaps/                        r,
 /usr/share/pixmaps/**                      r,
-/usr/share/unity/icons/**                  r,
-/usr/share/thumbnailer/icons/**            r,
-/usr/share/themes/**                       r,
 
 # The snapcraft desktop part may look for schema files in various locations, so
 # allow reading system installed schemas.
 /usr/share/glib*/schemas/{,*}              r,
-/usr/share/gnome/glib*/schemas/{,*}        r,
-/usr/share/ubuntu/glib*/schemas/{,*}       r,
 
 # Snappy's 'xdg-open' talks to the snapd-xdg-open service which currently works
 # only in environments supporting dbus-send (eg, X11). In the future once
 # snappy's xdg-open supports all snaps images, this access may move to another
-# interface.
+# interface. This is duplicated from desktop for compatibility with existing
+# snaps.
 /usr/bin/xdg-open ixr,
-/usr/share/applications/{,*} r,
+# While /usr/share/applications comes from the base runtime of the snap, it
+# has some things that snaps actually need, so allow access to those and deny
+# access to the others. This is duplicated from desktop for compatibility with
+# existing snaps.
+/usr/share/applications/ r,
+/usr/share/applications/mimeapps.list r,
+/usr/share/applications/xdg-open.desktop r,
+# silence noisy denials from desktop files in core* snaps that aren't usable by
+# snaps
+deny /usr/share/applications/python*.desktop r,
+deny /usr/share/applications/vim.desktop r,
+deny /usr/share/applications/snap-handle-link.desktop r,  # core16
 
 # This allow access to the first version of the snapd-xdg-open
 # version which was shipped outside of snapd
@@ -120,7 +125,7 @@ dbus (send)
     bus=session
     path=/io/snapcraft/Settings
     interface=io.snapcraft.Settings
-    member={Check,Get,Set}
+    member={Check,CheckSub,Get,GetSub,Set,SetSub}
     peer=(label=unconfined),
 
 # input methods (ibus)
@@ -135,6 +140,15 @@ unix (connect, receive, send)
      type=stream
      peer=(addr="@/tmp/ibus/dbus-*"),
 
+# abstract path in ibus >= 1.5.22 uses $XDG_CACHE_HOME (ie, @{HOME}/.cache)
+# This should use this, but due to LP: #1856738 we cannot
+#unix (connect, receive, send)
+#    type=stream
+#    peer=(addr="@@{HOME}/.cache/ibus/dbus-*"),
+unix (connect, receive, send)
+     type=stream
+     peer=(addr="@/home/*/.cache/ibus/dbus-*"),
+
 
 # input methods (mozc)
 # allow communicating with mozc server (TODO: investigate if allows sniffing)
@@ -371,6 +385,13 @@ dbus (receive)
     member=Introspect
     peer=(label=unconfined),
 
+dbus (receive)
+    bus=session
+    path=/com/canonical/dbusmenu
+    interface=org.freedesktop.DBus.Properties
+    member=Get*
+    peer=(label=unconfined),
+
 # app-indicators
 dbus (send)
     bus=session
@@ -444,7 +465,23 @@ dbus (receive)
     bus=session
     path=/org/freedesktop/Notifications
     interface=org.freedesktop.Notifications
-    member={ActionInvoked,NotificationClosed}
+    member={ActionInvoked,NotificationClosed,NotificationReplied}
+    peer=(label=unconfined),
+
+# KDE Plasma's Inhibited property indicating "do not disturb" mode
+# https://invent.kde.org/plasma/plasma-workspace/-/blob/master/libnotificationmanager/dbus/org.freedesktop.Notifications.xml#L42
+dbus (send)
+    bus=session
+    path=/org/freedesktop/Notifications
+    interface=org.freedesktop.DBus.Properties
+    member="Get{,All}"
+    peer=(label=unconfined),
+
+dbus (receive)
+    bus=session
+    path=/org/freedesktop/Notifications
+    interface=org.freedesktop.DBus.Properties
+    member=PropertiesChanged
     peer=(label=unconfined),
 
 dbus (send)
@@ -476,15 +513,14 @@ dbus (receive)
     member=Get*
     peer=(label=unconfined),
 
-# unity messaging menu
-# first, allow finding the desktop file
-/usr/share/applications/ r,
-# this leaks the names of snaps with desktop files
-/var/lib/snapd/desktop/applications/ r,
-/var/lib/snapd/desktop/applications/mimeinfo.cache r,
-# parallel-installs: when @{SNAP_INSTANCE_NAME} == @{SNAP_NAME},
-# this leaks read access to desktop files of parallel installs of the snap
-/var/lib/snapd/desktop/applications/@{SNAP_INSTANCE_NAME}_*.desktop r,
+###SNAP_DESKTOP_FILE_RULES###
+# Snaps are unable to use the data in mimeinfo.cache (since they can't execute
+# the returned desktop file themselves). unity messaging menu doesn't require
+# mimeinfo.cache and xdg-mime will fallback to reading the desktop files
+# directly to look for MimeType. Since reading the snap's own desktop files is
+# allowed, we can safely deny access to this file (and xdg-mime will either
+# return one of the snap's mimetypes, or none).
+deny /var/lib/snapd/desktop/applications/mimeinfo.cache r,
 
 # then allow talking to Unity DBus service
 dbus (send)
@@ -641,17 +677,23 @@ func (iface *unity7Interface) StaticInfo() interfaces.StaticInfo {
 }
 
 func (iface *unity7Interface) AppArmorConnectedPlug(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
-	// Unity7 will take the desktop filename and convert all '-' (and '.',
-	// but we don't care about that here because the rule above already
-	// does that) to '_'. Since we know that the desktop filename starts
-	// with the snap name, perform this conversion on the snap name.
+	// Unity7 will take the desktop filename and convert all '-' and '+'
+	// (and '.', but we don't care about that here because the rule above
+	// already does that) to '_'. Since we know that the desktop filename
+	// starts with the snap name, perform this conversion on the snap name.
 	//
 	// parallel-installs: UNITY_SNAP_NAME is used in the context of dbus
 	// mediation, this unintentionally opens access to dbus paths of keyed
 	// instances of @{SNAP_NAME} to @{SNAP_NAME} snap
-	new := strings.Replace(plug.Snap().InstanceName(), "-", "_", -1)
+	new := strings.Replace(plug.Snap().DesktopPrefix(), "-", "_", -1)
+	new = strings.Replace(new, "+", "_", -1)
 	old := "###UNITY_SNAP_NAME###"
 	snippet := strings.Replace(unity7ConnectedPlugAppArmor, old, new, -1)
+
+	old = "###SNAP_DESKTOP_FILE_RULES###"
+	new = strings.Join(getDesktopFileRules(plug.Snap().DesktopPrefix()), "\n")
+	snippet = strings.Replace(snippet, old, new+"\n", -1)
+
 	spec.AddSnippet(snippet)
 	return nil
 }
@@ -661,10 +703,6 @@ func (iface *unity7Interface) SecCompConnectedPlug(spec *seccomp.Specification,
 	return nil
 }
 
-func (iface *unity7Interface) BeforePrepareSlot(slot *snap.SlotInfo) error {
-	return sanitizeSlotReservedForOS(iface, slot)
-}
-
 func (iface *unity7Interface) AutoConnect(*snap.PlugInfo, *snap.SlotInfo) bool {
 	// allow what declarations allowed
 	return true
