@@ -27,7 +27,7 @@ import (
 	"github.com/snapcore/snapd/snap"
 )
 
-// Sanitize plug with a given snapd interface.
+// BeforePreparePlug sanitizes a plug with a given snapd interface.
 func BeforePreparePlug(iface Interface, plugInfo *snap.PlugInfo) error {
 	if iface.Name() != plugInfo.Interface {
 		return fmt.Errorf("cannot sanitize plug %q (interface %q) using interface %q",
@@ -40,6 +40,13 @@ func BeforePreparePlug(iface Interface, plugInfo *snap.PlugInfo) error {
 	return err
 }
 
+// ByName returns an Interface for the given interface name. Note that in order for
+// this to work properly, the package "interfaces/builtin" must also eventually be
+// imported to populate the full list of interfaces.
+var ByName = func(name string) (iface Interface, err error) {
+	panic("ByName is unset, import interfaces/builtin to initialize this")
+}
+
 // PlugRef is a reference to a plug.
 type PlugRef struct {
 	Snap string `json:"snap"`
@@ -51,6 +58,14 @@ func (ref PlugRef) String() string {
 	return fmt.Sprintf("%s:%s", ref.Snap, ref.Name)
 }
 
+// SortsBefore returns true when plug should be sorted before the other
+func (ref PlugRef) SortsBefore(other PlugRef) bool {
+	if ref.Snap != other.Snap {
+		return ref.Snap < other.Snap
+	}
+	return ref.Name < other.Name
+}
+
 // Sanitize slot with a given snapd interface.
 func BeforePrepareSlot(iface Interface, slotInfo *snap.SlotInfo) error {
 	if iface.Name() != slotInfo.Interface {
@@ -75,6 +90,14 @@ func (ref SlotRef) String() string {
 	return fmt.Sprintf("%s:%s", ref.Snap, ref.Name)
 }
 
+// SortsBefore returns true when slot should be sorted before the other
+func (ref SlotRef) SortsBefore(other SlotRef) bool {
+	if ref.Snap != other.Snap {
+		return ref.Snap < other.Snap
+	}
+	return ref.Name < other.Name
+}
+
 // Interfaces holds information about a list of plugs, slots and their connections.
 type Interfaces struct {
 	Plugs       []*snap.PlugInfo
@@ -110,6 +133,14 @@ func (conn *ConnRef) ID() string {
 	return fmt.Sprintf("%s:%s %s:%s", conn.PlugRef.Snap, conn.PlugRef.Name, conn.SlotRef.Snap, conn.SlotRef.Name)
 }
 
+// SortsBefore returns true when connection should be sorted before the other
+func (conn *ConnRef) SortsBefore(other *ConnRef) bool {
+	if conn.PlugRef != other.PlugRef {
+		return conn.PlugRef.SortsBefore(other.PlugRef)
+	}
+	return conn.SlotRef.SortsBefore(other.SlotRef)
+}
+
 // ParseConnRef parses an ID string
 func ParseConnRef(id string) (*ConnRef, error) {
 	var conn ConnRef
@@ -173,6 +204,27 @@ type StaticInfo struct {
 	BaseDeclarationSlots string
 }
 
+// PermanentPlugServiceSnippets will return the set of snippets for the systemd
+// service unit that should be generated for a snap with the specified plug.
+// The list returned is not unique, callers must de-duplicate themselves.
+// The plug is provided because the snippet may depend on plug attributes for
+// example. The plug is sanitized before the snippets are returned.
+func PermanentPlugServiceSnippets(iface Interface, plug *snap.PlugInfo) (snips []string, err error) {
+	// sanitize the plug first
+	err = BeforePreparePlug(iface, plug)
+	if err != nil {
+		return nil, err
+	}
+
+	type serviceSnippetPlugger interface {
+		ServicePermanentPlug(plug *snap.PlugInfo) []string
+	}
+	if iface, ok := iface.(serviceSnippetPlugger); ok {
+		snips = iface.ServicePermanentPlug(plug)
+	}
+	return snips, nil
+}
+
 // StaticInfoOf returns the static-info of the given interface.
 func StaticInfoOf(iface Interface) (si StaticInfo) {
 	type metaDataProvider interface {
@@ -210,9 +262,9 @@ const (
 	SecurityUDev SecuritySystem = "udev"
 	// SecurityMount identifies the mount security system.
 	SecurityMount SecuritySystem = "mount"
-	// SecurityKMod identifies the kernel modules security system
+	// SecurityKMod identifies the kernel modules security system.
 	SecurityKMod SecuritySystem = "kmod"
-	// SecuritySystemd identifies the systemd services security system
+	// SecuritySystemd identifies the systemd services security system.
 	SecuritySystemd SecuritySystem = "systemd"
 )
 
@@ -232,13 +284,3 @@ func ValidateDBusBusName(busName string) error {
 	}
 	return nil
 }
-
-// UnknownPlugSlotError is an error reported when plug or slot cannot be found.
-type UnknownPlugSlotError struct {
-	Msg string
-}
-
-// Error returns the message associated with unknown plug or slot error.
-func (e *UnknownPlugSlotError) Error() string {
-	return e.Msg
-}
