@@ -36,7 +36,6 @@ import (
 	"bytes"
 	"fmt"
 	"os"
-	"os/exec"
 	"path/filepath"
 	"runtime"
 	"strings"
@@ -44,44 +43,41 @@ import (
 	"github.com/snapcore/snapd/arch"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/interfaces"
-	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/release"
+	"github.com/snapcore/snapd/sandbox/apparmor"
+	"github.com/snapcore/snapd/sandbox/seccomp"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snapdtool"
 	"github.com/snapcore/snapd/strutil"
+	"github.com/snapcore/snapd/timings"
 )
 
 var (
-	osReadlink               = os.Readlink
-	kernelFeatures           = release.SecCompActions
-	ubuntuKernelArchitecture = arch.UbuntuKernelArchitecture
-	releaseInfoId            = release.ReleaseInfo.ID
-	releaseInfoVersionId     = release.ReleaseInfo.VersionID
-	requiresSocketcall       = requiresSocketcallImpl
+	kernelFeatures         = seccomp.Actions
+	dpkgKernelArchitecture = arch.DpkgKernelArchitecture
+	releaseInfoId          = release.ReleaseInfo.ID
+	releaseInfoVersionId   = release.ReleaseInfo.VersionID
+	requiresSocketcall     = requiresSocketcallImpl
+
+	snapSeccompVersionInfo = snapSeccompVersionInfoImpl
+	seccompCompilerLookup  = snapdtool.InternalToolPath
 )
 
-func seccompToBpfPath() string {
-	// FIXME: use cmd.InternalToolPath here once:
-	//   https://github.com/snapcore/snapd/pull/3512
-	// is merged
-	snapSeccomp := filepath.Join(dirs.DistroLibExecDir, "snap-seccomp")
+func snapSeccompVersionInfoImpl(c Compiler) (seccomp.VersionInfo, error) {
+	return c.VersionInfo()
+}
 
-	exe, err := osReadlink("/proc/self/exe")
-	if err != nil {
-		logger.Noticef("cannot read /proc/self/exe: %v, using default snap-seccomp command", err)
-		return snapSeccomp
-	}
-	if !strings.HasPrefix(exe, dirs.SnapMountDir) {
-		return snapSeccomp
-	}
-
-	// if we are re-execed, then snap-seccomp is at the same location
-	// as snapd
-	return filepath.Join(filepath.Dir(exe), "snap-seccomp")
+type Compiler interface {
+	Compile(in, out string) error
+	VersionInfo() (seccomp.VersionInfo, error)
 }
 
 // Backend is responsible for maintaining seccomp profiles for snap-confine.
-type Backend struct{}
+type Backend struct {
+	snapSeccomp Compiler
+	versionInfo seccomp.VersionInfo
+}
 
 var globalProfileLE = []byte{
 	0x20, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x15, 0x00, 0x00, 0x04, 0x3e, 0x00, 0x00, 0xc0,
@@ -116,8 +112,11 @@ func isBigEndian() bool {
 	return false
 }
 
-// Initialize ensures that the global profile is on disk.
-func (b *Backend) Initialize() error {
+// Initialize ensures that the global profile is on disk and interrogates
+// libseccomp wrapper to generate a version string that will be used to
+// determine if we need to recompile seccomp policy due to system
+// changes outside of snapd.
+func (b *Backend) Initialize(*interfaces.SecurityBackendOptions) error {
 	dir := dirs.SnapSeccompDir
 	fname := "global.bin"
 	var globalProfile []byte
@@ -126,8 +125,8 @@ func (b *Backend) Initialize() error {
 	} else {
 		globalProfile = globalProfileLE
 	}
-	content := map[string]*osutil.FileState{
-		fname: {Content: globalProfile, Mode: 0644},
+	content := map[string]osutil.FileState{
+		fname: &osutil.MemoryFileState{Content: globalProfile, Mode: 0644},
 	}
 	if err := os.MkdirAll(dir, 0755); err != nil {
 		return fmt.Errorf("cannot create directory for seccomp profiles %q: %s", dir, err)
@@ -136,6 +135,17 @@ func (b *Backend) Initialize() error {
 	if err != nil {
 		return fmt.Errorf("cannot synchronize global seccomp profile: %s", err)
 	}
+
+	b.snapSeccomp, err = seccomp.NewCompiler(seccompCompilerLookup)
+	if err != nil {
+		return fmt.Errorf("cannot initialize seccomp profile compiler: %v", err)
+	}
+
+	versionInfo, err := snapSeccompVersionInfo(b.snapSeccomp)
+	if err != nil {
+		return fmt.Errorf("cannot obtain snap-seccomp version information: %v", err)
+	}
+	b.versionInfo = versionInfo
 	return nil
 }
 
@@ -144,13 +154,100 @@ func (b *Backend) Name() interfaces.SecuritySystem {
 	return interfaces.SecuritySecComp
 }
 
+func bpfSrcPath(srcName string) string {
+	return filepath.Join(dirs.SnapSeccompDir, srcName)
+}
+
+func bpfBinPath(srcName string) string {
+	return filepath.Join(dirs.SnapSeccompDir, strings.TrimSuffix(srcName, ".src")+".bin")
+}
+
+func parallelCompile(compiler Compiler, profiles []string) error {
+	if len(profiles) == 0 {
+		// no profiles, nothing to do
+		return nil
+	}
+
+	profilesQueue := make(chan string, len(profiles))
+	numWorkers := runtime.NumCPU()
+	if numWorkers >= 2 {
+		numWorkers -= 1
+	}
+	if numWorkers > len(profiles) {
+		numWorkers = len(profiles)
+	}
+	resultsBufferSize := numWorkers * 2
+	if resultsBufferSize > len(profiles) {
+		resultsBufferSize = len(profiles)
+	}
+	res := make(chan error, resultsBufferSize)
+
+	// launch as many workers as we have CPUs
+	for i := 0; i < numWorkers; i++ {
+		go func() {
+			for {
+				profile, ok := <-profilesQueue
+				if !ok {
+					break
+				}
+				in := bpfSrcPath(profile)
+				out := bpfBinPath(profile)
+				// remove the old profile first so that we are
+				// not loading it accidentally should the
+				// compilation fail
+				if err := os.Remove(out); err != nil && !os.IsNotExist(err) {
+					res <- err
+					continue
+				}
+
+				// snap-seccomp uses AtomicWriteFile internally, on failure the
+				// output file is unlinked
+				if err := compiler.Compile(in, out); err != nil {
+					res <- fmt.Errorf("cannot compile %s: %v", in, err)
+				} else {
+					res <- nil
+				}
+			}
+		}()
+	}
+
+	for _, p := range profiles {
+		profilesQueue <- p
+	}
+	// signal workers to exit
+	close(profilesQueue)
+
+	var firstErr error
+	for i := 0; i < len(profiles); i++ {
+		maybeErr := <-res
+		if maybeErr != nil && firstErr == nil {
+			firstErr = maybeErr
+		}
+
+	}
+
+	// not expecting any more results
+	close(res)
+
+	if firstErr != nil {
+		for _, p := range profiles {
+			out := bpfBinPath(p)
+			// unlink all profiles that could have been successfully
+			// compiled
+			os.Remove(out)
+		}
+
+	}
+	return firstErr
+}
+
 // Setup creates seccomp profiles specific to a given snap.
 // The snap can be in developer mode to make security violations non-fatal to
 // the offending application process.
 //
 // This method should be called after changing plug, slots, connections between
 // them or application present in the snap.
-func (b *Backend) Setup(snapInfo *snap.Info, opts interfaces.ConfinementOptions, repo *interfaces.Repository) error {
+func (b *Backend) Setup(snapInfo *snap.Info, opts interfaces.ConfinementOptions, repo *interfaces.Repository, tm timings.Measurer) error {
 	snapName := snapInfo.InstanceName()
 	// Get the snippets that apply to this snap
 	spec, err := repo.SnapSpecification(b.Name(), snapName)
@@ -164,28 +261,32 @@ func (b *Backend) Setup(snapInfo *snap.Info, opts interfaces.ConfinementOptions,
 		return fmt.Errorf("cannot obtain expected security files for snap %q: %s", snapName, err)
 	}
 
-	glob := interfaces.SecurityTagGlob(snapName)
+	glob := interfaces.SecurityTagGlob(snapName) + ".src"
 	dir := dirs.SnapSeccompDir
 	if err := os.MkdirAll(dir, 0755); err != nil {
 		return fmt.Errorf("cannot create directory for seccomp profiles %q: %s", dir, err)
 	}
-	_, _, err = osutil.EnsureDirState(dir, glob, content)
+	// There is a delicate interaction between `snap run`, `snap-confine`
+	// and compilation of profiles:
+	// - whenever profiles need to be rebuilt due to system-key change,
+	//   `snap run` detects the system-key mismatch and waits for snapd
+	//   (system key is only updated once all security backends have
+	//   finished their job)
+	// - whenever the binary file does not exist, `snap-confine` will poll
+	//   and wait for SNAP_CONFINE_MAX_PROFILE_WAIT, if the profile does not
+	//   appear in that time, `snap-confine` will fail and die
+	changed, removed, err := osutil.EnsureDirState(dir, glob, content)
 	if err != nil {
 		return fmt.Errorf("cannot synchronize security files for snap %q: %s", snapName, err)
 	}
-
-	for baseName := range content {
-		in := filepath.Join(dirs.SnapSeccompDir, baseName)
-		out := filepath.Join(dirs.SnapSeccompDir, strings.TrimSuffix(baseName, ".src")+".bin")
-
-		seccompToBpf := seccompToBpfPath()
-		cmd := exec.Command(seccompToBpf, "compile", in, out)
-		if output, err := cmd.CombinedOutput(); err != nil {
-			return osutil.OutputErr(output, err)
+	for _, c := range removed {
+		err := os.Remove(bpfBinPath(c))
+		if err != nil && !os.IsNotExist(err) {
+			return err
 		}
 	}
 
-	return nil
+	return parallelCompile(b.snapSeccomp, changed)
 }
 
 // Remove removes seccomp profiles of a given snap.
@@ -198,33 +299,68 @@ func (b *Backend) Remove(snapName string) error {
 	return nil
 }
 
+// Obtain the privilege dropping snippet
+func uidGidChownSnippet(name string) (string, error) {
+	tmp := strings.Replace(privDropAndChownSyscalls, "###USERNAME###", name, -1)
+	return strings.Replace(tmp, "###GROUP###", name, -1), nil
+}
+
 // deriveContent combines security snippets collected from all the interfaces
 // affecting a given snap into a content map applicable to EnsureDirState.
-func (b *Backend) deriveContent(spec *Specification, opts interfaces.ConfinementOptions, snapInfo *snap.Info) (content map[string]*osutil.FileState, err error) {
+func (b *Backend) deriveContent(spec *Specification, opts interfaces.ConfinementOptions, snapInfo *snap.Info) (content map[string]osutil.FileState, err error) {
 	// Some base snaps and systems require the socketcall() in the default
 	// template
 	addSocketcall := requiresSocketcall(snapInfo.Base)
 
+	var uidGidChownSyscalls bytes.Buffer
+	if len(snapInfo.SystemUsernames) == 0 {
+		uidGidChownSyscalls.WriteString(barePrivDropSyscalls)
+	} else {
+		for _, id := range snapInfo.SystemUsernames {
+			syscalls, err := uidGidChownSnippet(id.Name)
+			if err != nil {
+				return nil, fmt.Errorf(`cannot calculate syscalls for "%s": %s`, id, err)
+			}
+			uidGidChownSyscalls.WriteString(syscalls)
+		}
+		uidGidChownSyscalls.WriteString(rootSetUidGidSyscalls)
+	}
+
 	for _, hookInfo := range snapInfo.Hooks {
 		if content == nil {
-			content = make(map[string]*osutil.FileState)
+			content = make(map[string]osutil.FileState)
 		}
 		securityTag := hookInfo.SecurityTag()
-		addContent(securityTag, opts, spec.SnippetForTag(securityTag), content, addSocketcall)
+
+		path := securityTag + ".src"
+		content[path] = &osutil.MemoryFileState{
+			Content: generateContent(opts, spec.SnippetForTag(securityTag), addSocketcall, b.versionInfo, uidGidChownSyscalls.String()),
+			Mode:    0644,
+		}
 	}
 	for _, appInfo := range snapInfo.Apps {
 		if content == nil {
-			content = make(map[string]*osutil.FileState)
+			content = make(map[string]osutil.FileState)
 		}
 		securityTag := appInfo.SecurityTag()
-		addContent(securityTag, opts, spec.SnippetForTag(securityTag), content, addSocketcall)
+		path := securityTag + ".src"
+		content[path] = &osutil.MemoryFileState{
+			Content: generateContent(opts, spec.SnippetForTag(securityTag), addSocketcall, b.versionInfo, uidGidChownSyscalls.String()),
+			Mode:    0644,
+		}
 	}
 
 	return content, nil
 }
 
-func addContent(securityTag string, opts interfaces.ConfinementOptions, snippetForTag string, content map[string]*osutil.FileState, addSocketcall bool) {
+func generateContent(opts interfaces.ConfinementOptions, snippetForTag string, addSocketcall bool, versionInfo seccomp.VersionInfo, uidGidChownSyscalls string) []byte {
 	var buffer bytes.Buffer
+
+	if versionInfo != "" {
+		buffer.WriteString("# snap-seccomp version information:\n")
+		fmt.Fprintf(&buffer, "# %s\n", versionInfo)
+	}
+
 	if opts.Classic && !opts.JailMode {
 		// NOTE: This is understood by snap-confine
 		buffer.WriteString("@unrestricted\n")
@@ -232,18 +368,19 @@ func addContent(securityTag string, opts interfaces.ConfinementOptions, snippetF
 	if opts.DevMode && !opts.JailMode {
 		// NOTE: This is understood by snap-confine
 		buffer.WriteString("@complain\n")
-		if !release.SecCompSupportsAction("log") {
+		if !seccomp.SupportsAction("log") {
 			buffer.WriteString("# complain mode logging unavailable\n")
 		}
 	}
 
 	buffer.Write(defaultTemplate)
 	buffer.WriteString(snippetForTag)
+	buffer.WriteString(uidGidChownSyscalls)
 
-	// For systems with force-devmode we need to apply a workaround
-	// to avoid failing hooks. See description in template.go for
-	// more details.
-	if release.ReleaseInfo.ForceDevMode() {
+	// For systems with partial or missing AppArmor support we need to apply
+	// a workaround to avoid failing hooks. See description in template.go
+	// for more details.
+	if apparmor.ProbedLevel() != apparmor.Full {
 		buffer.WriteString(bindSyscallWorkaround)
 	}
 
@@ -251,11 +388,7 @@ func addContent(securityTag string, opts interfaces.ConfinementOptions, snippetF
 		buffer.WriteString(socketcallSyscallDeprecated)
 	}
 
-	path := fmt.Sprintf("%s.src", securityTag)
-	content[path] = &osutil.FileState{
-		Content: buffer.Bytes(),
-		Mode:    0644,
-	}
+	return buffer.Bytes()
 }
 
 // NewSpecification returns an empty seccomp specification.
@@ -263,16 +396,22 @@ func (b *Backend) NewSpecification() interfaces.Specification {
 	return &Specification{}
 }
 
-// SandboxFeatures returns the list of seccomp features supported by the kernel.
+// SandboxFeatures returns the list of seccomp features supported by the kernel
+// and userspace.
 func (b *Backend) SandboxFeatures() []string {
 	features := kernelFeatures()
 	tags := make([]string, 0, len(features)+1)
 	for _, feature := range features {
-		// Prepend "kernel:" to apparmor kernel features to namespace them and
-		// allow us to introduce our own tags later.
+		// Prepend "kernel:" to apparmor kernel features to namespace
+		// them.
 		tags = append(tags, "kernel:"+feature)
 	}
 	tags = append(tags, "bpf-argument-filtering")
+
+	if res, err := b.versionInfo.HasFeature("bpf-actlog"); err == nil && res {
+		tags = append(tags, "bpf-actlog")
+	}
+
 	return tags
 }
 
@@ -290,7 +429,7 @@ func (b *Backend) SandboxFeatures() []string {
 // - if the kernel architecture is not any of the above, force the use of
 //   socketcall()
 func requiresSocketcallImpl(baseSnap string) bool {
-	switch ubuntuKernelArchitecture() {
+	switch dpkgKernelArchitecture() {
 	case "i386", "s390x":
 		// glibc sysdeps/unix/sysv/linux/i386/kernel-features.h and
 		// sysdeps/unix/sysv/linux/s390/kernel-features.h added the
@@ -344,3 +483,14 @@ func requiresSocketcallImpl(baseSnap string) bool {
 	// If we got here, something went wrong, but if the code above changes
 	// the compiler will complain about the lack of 'return'.
 }
+
+// MockSnapSeccompVersionInfo is for use in tests only.
+func MockSnapSeccompVersionInfo(versionInfo string) (restore func()) {
+	old := snapSeccompVersionInfo
+	snapSeccompVersionInfo = func(c Compiler) (seccomp.VersionInfo, error) {
+		return seccomp.VersionInfo(versionInfo), nil
+	}
+	return func() {
+		snapSeccompVersionInfo = old
+	}
+}
