@@ -37,10 +37,15 @@ import (
 )
 
 // Backend is responsible for maintaining apparmor profiles for ubuntu-core-launcher.
-type Backend struct{}
+type Backend struct {
+	preseed bool
+}
 
 // Initialize does nothing.
-func (b *Backend) Initialize() error {
+func (b *Backend) Initialize(opts *interfaces.SecurityBackendOptions) error {
+	if opts != nil && opts.Preseed {
+		b.preseed = true
+	}
 	return nil
 }
 
@@ -69,16 +74,22 @@ func (b *Backend) Setup(snapInfo *snap.Info, confinement interfaces.ConfinementO
 	}
 	glob := interfaces.InterfaceServiceName(snapName, "*")
 
-	systemd := sysd.New(dirs.GlobalRootDir, sysd.SystemMode, &dummyReporter{})
+	var systemd sysd.Systemd
+	if b.preseed {
+		systemd = sysd.NewEmulationMode(dirs.GlobalRootDir)
+	} else {
+		systemd = sysd.New(sysd.SystemMode, &dummyReporter{})
+	}
+
 	// We need to be carefully here and stop all removed service units before
 	// we remove their files as otherwise systemd is not able to disable/stop
 	// them anymore.
-	if err := disableRemovedServices(systemd, dir, glob, content); err != nil {
+	if err := b.disableRemovedServices(systemd, dir, glob, content); err != nil {
 		logger.Noticef("cannot stop removed services: %s", err)
 	}
 	changed, removed, errEnsure := osutil.EnsureDirState(dir, glob, content)
 	// Reload systemd whenever something is added or removed
-	if len(changed) > 0 || len(removed) > 0 {
+	if !b.preseed && (len(changed) > 0 || len(removed) > 0) {
 		err := systemd.DaemonReload()
 		if err != nil {
 			logger.Noticef("cannot reload systemd state: %s", err)
@@ -89,10 +100,12 @@ func (b *Backend) Setup(snapInfo *snap.Info, confinement interfaces.ConfinementO
 		if err := systemd.Enable(service); err != nil {
 			logger.Noticef("cannot enable service %q: %s", service, err)
 		}
-		// If we have a new service here which isn't started yet the restart
-		// operation will start it.
-		if err := systemd.Restart(service, 10*time.Second); err != nil {
-			logger.Noticef("cannot restart service %q: %s", service, err)
+		if !b.preseed {
+			// If we have a new service here which isn't started yet the restart
+			// operation will start it.
+			if err := systemd.Restart(service, 10*time.Second); err != nil {
+				logger.Noticef("cannot restart service %q: %s", service, err)
+			}
 		}
 	}
 	return errEnsure
@@ -100,7 +113,14 @@ func (b *Backend) Setup(snapInfo *snap.Info, confinement interfaces.ConfinementO
 
 // Remove disables, stops and removes systemd services of a given snap.
 func (b *Backend) Remove(snapName string) error {
-	systemd := sysd.New(dirs.GlobalRootDir, sysd.SystemMode, &dummyReporter{})
+	var systemd sysd.Systemd
+	if b.preseed {
+		// removing while preseeding is not a viable scenario, but implemented
+		// for completness.
+		systemd = sysd.NewEmulationMode(dirs.GlobalRootDir)
+	} else {
+		systemd = sysd.New(sysd.SystemMode, &dummyReporter{})
+	}
 	// Remove all the files matching snap glob
 	glob := interfaces.InterfaceServiceName(snapName, "*")
 	_, removed, errEnsure := osutil.EnsureDirState(dirs.SnapServicesDir, glob, nil)
@@ -108,12 +128,14 @@ func (b *Backend) Remove(snapName string) error {
 		if err := systemd.Disable(service); err != nil {
 			logger.Noticef("cannot disable service %q: %s", service, err)
 		}
-		if err := systemd.Stop(service, 5*time.Second); err != nil {
-			logger.Noticef("cannot stop service %q: %s", service, err)
+		if !b.preseed {
+			if err := systemd.Stop(service, 5*time.Second); err != nil {
+				logger.Noticef("cannot stop service %q: %s", service, err)
+			}
 		}
 	}
 	// Reload systemd whenever something is removed
-	if len(removed) > 0 {
+	if !b.preseed && len(removed) > 0 {
 		err := systemd.DaemonReload()
 		if err != nil {
 			logger.Noticef("cannot reload systemd state: %s", err)
@@ -133,14 +155,14 @@ func (b *Backend) SandboxFeatures() []string {
 }
 
 // deriveContent computes .service files based on requests made to the specification.
-func deriveContent(spec *Specification, snapInfo *snap.Info) map[string]*osutil.FileState {
+func deriveContent(spec *Specification, snapInfo *snap.Info) map[string]osutil.FileState {
 	services := spec.Services()
 	if len(services) == 0 {
 		return nil
 	}
-	content := make(map[string]*osutil.FileState)
+	content := make(map[string]osutil.FileState)
 	for name, service := range services {
-		content[name] = &osutil.FileState{
+		content[name] = &osutil.MemoryFileState{
 			Content: []byte(service.String()),
 			Mode:    0644,
 		}
@@ -148,7 +170,7 @@ func deriveContent(spec *Specification, snapInfo *snap.Info) map[string]*osutil.
 	return content
 }
 
-func disableRemovedServices(systemd sysd.Systemd, dir, glob string, content map[string]*osutil.FileState) error {
+func (b *Backend) disableRemovedServices(systemd sysd.Systemd, dir, glob string, content map[string]osutil.FileState) error {
 	paths, err := filepath.Glob(filepath.Join(dir, glob))
 	if err != nil {
 		return err
@@ -159,8 +181,10 @@ func disableRemovedServices(systemd sysd.Systemd, dir, glob string, content map[
 			if err := systemd.Disable(service); err != nil {
 				logger.Noticef("cannot disable service %q: %s", service, err)
 			}
-			if err := systemd.Stop(service, 5*time.Second); err != nil {
-				logger.Noticef("cannot stop service %q: %s", service, err)
+			if !b.preseed {
+				if err := systemd.Stop(service, 5*time.Second); err != nil {
+					logger.Noticef("cannot stop service %q: %s", service, err)
+				}
 			}
 		}
 	}
