@@ -30,6 +30,13 @@ import (
 	"time"
 
 	"github.com/snapcore/snapd/osutil/sys"
+	"github.com/snapcore/snapd/strutil"
+)
+
+var (
+	StreamsEqualChunked  = streamsEqualChunked
+	FilesAreEqualChunked = filesAreEqualChunked
+	SudoersFile          = sudoersFile
 )
 
 func MockUserLookup(mock func(name string) (*user.User, error)) func() {
@@ -61,6 +68,14 @@ func MockSyscallKill(f func(int, syscall.Signal) error) func() {
 	}
 }
 
+func MockSyscallStatfs(f func(string, *syscall.Statfs_t) error) func() {
+	oldSyscallStatfs := syscallStatfs
+	syscallStatfs = f
+	return func() {
+		syscallStatfs = oldSyscallStatfs
+	}
+}
+
 func MockSyscallGetpgid(f func(int) (int, error)) func() {
 	oldSyscallGetpgid := syscallGetpgid
 	syscallGetpgid = f
@@ -102,6 +117,11 @@ func SetUnsafeIO(b bool) func() {
 	}
 }
 
+func GetUnsafeIO() bool {
+	// a getter so that tests do not attempt to modify that directly
+	return snapdUnsafeIO
+}
+
 func MockOsReadlink(f func(string) (string, error)) func() {
 	realOsReadlink := osReadlink
 	osReadlink = f
@@ -109,23 +129,6 @@ func MockOsReadlink(f func(string) (string, error)) func() {
 	return func() { osReadlink = realOsReadlink }
 }
 
-//MockMountInfo mocks content of /proc/self/mountinfo read by IsHomeUsingNFS
-func MockMountInfo(text string) (restore func()) {
-	old := procSelfMountInfo
-	f, err := ioutil.TempFile("", "mountinfo")
-	if err != nil {
-		panic(fmt.Errorf("cannot open temporary file: %s", err))
-	}
-	if err := ioutil.WriteFile(f.Name(), []byte(text), 0644); err != nil {
-		panic(fmt.Errorf("cannot write mock mountinfo file: %s", err))
-	}
-	procSelfMountInfo = f.Name()
-	return func() {
-		os.Remove(procSelfMountInfo)
-		procSelfMountInfo = old
-	}
-}
-
 // MockEtcFstab mocks content of /etc/fstab read by IsHomeUsingNFS
 func MockEtcFstab(text string) (restore func()) {
 	old := etcFstab
@@ -176,14 +179,44 @@ func MockFindGidNoFallback(mock func(name string) (uint64, error)) (restore func
 	return func() { findGidNoGetentFallback = old }
 }
 
-func MockFindUid(mock func(name string) (uint64, error)) (restore func()) {
-	old := findUid
-	findUid = mock
-	return func() { findUid = old }
+const MaxSymlinkTries = maxSymlinkTries
+
+var ParseRawEnvironment = parseRawEnvironment
+
+// ParseRawExpandableEnv returns a new expandable environment parsed from key=value strings.
+func ParseRawExpandableEnv(entries []string) (ExpandableEnv, error) {
+	om := strutil.NewOrderedMap()
+	for _, entry := range entries {
+		key, value, err := parseEnvEntry(entry)
+		if err != nil {
+			return ExpandableEnv{}, err
+		}
+		if om.Get(key) != "" {
+			return ExpandableEnv{}, fmt.Errorf("cannot overwrite earlier value of %q", key)
+		}
+		om.Set(key, value)
+	}
+	return ExpandableEnv{OrderedMap: om}, nil
 }
 
-func MockFindGid(mock func(name string) (uint64, error)) (restore func()) {
-	old := findGid
-	findGid = mock
-	return func() { findGid = old }
+// this is weird to use in a test, but it is so that we can test the actual
+// implementation of LoadMountInfo, which normally panics during tests if not
+// properly mocked
+func MountInfoMustMock(new bool) (restore func()) {
+	old := mountInfoMustMockInTests
+	mountInfoMustMockInTests = new
+	return func() {
+		mountInfoMustMockInTests = old
+	}
+}
+
+// this should not be used except to test the actual implementation logic of
+// LoadMountInfo, if you are trying to mock /proc/self/mountinfo in a test,
+// use MockMountInfo(), which is exported and the right way to do that.
+func MockProcSelfMountInfoLocation(new string) (restore func()) {
+	old := procSelfMountInfo
+	procSelfMountInfo = new
+	return func() {
+		procSelfMountInfo = old
+	}
 }
