@@ -33,6 +33,7 @@ import (
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/asserts/assertstest"
 	"github.com/snapcore/snapd/gadget"
+	"github.com/snapcore/snapd/gadget/quantity"
 	"github.com/snapcore/snapd/overlord/assertstate"
 	"github.com/snapcore/snapd/overlord/assertstate/assertstatetest"
 	"github.com/snapcore/snapd/overlord/auth"
@@ -1339,27 +1340,28 @@ volumes:
 	defer restore()
 
 	gadgetUpdateCalled := false
-	restore = devicestate.MockGadgetUpdate(func(current, update gadget.GadgetData, path string, policy gadget.UpdatePolicyFunc) error {
+	restore = devicestate.MockGadgetUpdate(func(current, update gadget.GadgetData, path string, policy gadget.UpdatePolicyFunc, _ gadget.ContentUpdateObserver) error {
 		gadgetUpdateCalled = true
 		c.Check(policy, NotNil)
 		c.Check(reflect.ValueOf(policy).Pointer(), Equals, reflect.ValueOf(gadget.RemodelUpdatePolicy).Pointer())
 		c.Check(current, DeepEquals, gadget.GadgetData{
 			Info: &gadget.Info{
-				Volumes: map[string]gadget.Volume{
+				Volumes: map[string]*gadget.Volume{
 					"pc": {
 						Bootloader: "grub",
+						Schema:     "gpt",
 						Structure: []gadget.VolumeStructure{{
 							Name:       "foo",
 							Type:       "00000000-0000-0000-0000-0000deadcafe",
-							Size:       10 * gadget.SizeMiB,
+							Size:       10 * quantity.SizeMiB,
 							Filesystem: "ext4",
 							Content: []gadget.VolumeContent{
-								{Source: "foo-content", Target: "/"},
+								{UnresolvedSource: "foo-content", Target: "/"},
 							},
 						}, {
 							Name: "bare-one",
 							Type: "bare",
-							Size: gadget.SizeMiB,
+							Size: quantity.SizeMiB,
 							Content: []gadget.VolumeContent{
 								{Image: "bare.img"},
 							},
@@ -1371,21 +1373,22 @@ volumes:
 		})
 		c.Check(update, DeepEquals, gadget.GadgetData{
 			Info: &gadget.Info{
-				Volumes: map[string]gadget.Volume{
+				Volumes: map[string]*gadget.Volume{
 					"pc": {
 						Bootloader: "grub",
+						Schema:     "gpt",
 						Structure: []gadget.VolumeStructure{{
 							Name:       "foo",
 							Type:       "00000000-0000-0000-0000-0000deadcafe",
-							Size:       10 * gadget.SizeMiB,
+							Size:       10 * quantity.SizeMiB,
 							Filesystem: "ext4",
 							Content: []gadget.VolumeContent{
-								{Source: "new-foo-content", Target: "/"},
+								{UnresolvedSource: "new-foo-content", Target: "/"},
 							},
 						}, {
 							Name: "bare-one",
 							Type: "bare",
-							Size: gadget.SizeMiB,
+							Size: quantity.SizeMiB,
 							Content: []gadget.VolumeContent{
 								{Image: "new-bare-content.img"},
 							},
@@ -1485,7 +1488,7 @@ func (s *deviceMgrRemodelSuite) TestRemodelGadgetAssetsParanoidCheck(c *C) {
 	defer restore()
 
 	gadgetUpdateCalled := false
-	restore = devicestate.MockGadgetUpdate(func(current, update gadget.GadgetData, path string, policy gadget.UpdatePolicyFunc) error {
+	restore = devicestate.MockGadgetUpdate(func(current, update gadget.GadgetData, path string, policy gadget.UpdatePolicyFunc, _ gadget.ContentUpdateObserver) error {
 		return errors.New("unexpected call")
 	})
 	defer restore()
