@@ -20,15 +20,12 @@
 package ctlcmd_test
 
 import (
+	"context"
 	"fmt"
 	"sort"
 
-	"golang.org/x/net/context"
-
 	. "gopkg.in/check.v1"
 
-	"github.com/snapcore/snapd/asserts"
-	"github.com/snapcore/snapd/asserts/sysdb"
 	"github.com/snapcore/snapd/client"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/overlord/auth"
@@ -37,6 +34,7 @@ import (
 	"github.com/snapcore/snapd/overlord/hookstate/hooktest"
 	"github.com/snapcore/snapd/overlord/servicestate"
 	"github.com/snapcore/snapd/overlord/snapstate"
+	"github.com/snapcore/snapd/overlord/snapstate/snapstatetest"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/snaptest"
@@ -50,41 +48,49 @@ type fakeStore struct {
 	storetest.Store
 }
 
-func (f *fakeStore) SnapAction(_ context.Context, currentSnaps []*store.CurrentSnap, actions []*store.SnapAction, user *auth.UserState, opts *store.RefreshOptions) ([]*snap.Info, error) {
+func (f *fakeStore) SnapAction(_ context.Context, currentSnaps []*store.CurrentSnap, actions []*store.SnapAction, assertQuery store.AssertionQuery, user *auth.UserState, opts *store.RefreshOptions) ([]store.SnapActionResult, []store.AssertionResult, error) {
+	if assertQuery != nil {
+		panic("no assertion query support")
+	}
 	if actions[0].Action == "install" {
-		installs := make([]*snap.Info, 0, len(actions))
+		installs := make([]store.SnapActionResult, 0, len(actions))
 		for _, a := range actions {
 			snapName, instanceKey := snap.SplitInstanceName(a.InstanceName)
 			if instanceKey != "" {
 				panic(fmt.Sprintf("unexpected instance name %q in snap install action", a.InstanceName))
 			}
 
-			installs = append(installs, &snap.Info{
+			installs = append(installs, store.SnapActionResult{Info: &snap.Info{
+				DownloadInfo: snap.DownloadInfo{
+					Size: 1,
+				},
 				SideInfo: snap.SideInfo{
 					RealName: snapName,
 					Revision: snap.R(2),
 				},
 				Architectures: []string{"all"},
-			})
+			}})
 		}
 
-		return installs, nil
+		return installs, nil, nil
 	}
 
-	return []*snap.Info{{
+	snaps := []store.SnapActionResult{{Info: &snap.Info{
 		SideInfo: snap.SideInfo{
 			RealName: "test-snap",
 			Revision: snap.R(2),
 			SnapID:   "test-snap-id",
 		},
 		Architectures: []string{"all"},
-	}, {SideInfo: snap.SideInfo{
-		RealName: "other-snap",
-		Revision: snap.R(2),
-		SnapID:   "other-snap-id",
-	},
+	}}, {Info: &snap.Info{
+		SideInfo: snap.SideInfo{
+			RealName: "other-snap",
+			Revision: snap.R(2),
+			SnapID:   "other-snap-id",
+		},
 		Architectures: []string{"all"},
-	}}, nil
+	}}}
+	return snaps, nil, nil
 }
 
 type servicectlSuite struct {
@@ -124,7 +130,7 @@ apps:
 `
 
 func mockServiceChangeFunc(testServiceControlInputs func(appInfos []*snap.AppInfo, inst *servicestate.Instruction)) func() {
-	return ctlcmd.MockServicestateControlFunc(func(st *state.State, appInfos []*snap.AppInfo, inst *servicestate.Instruction, context *hookstate.Context) ([]*state.TaskSet, error) {
+	return ctlcmd.MockServicestateControlFunc(func(st *state.State, appInfos []*snap.AppInfo, inst *servicestate.Instruction, flags *servicestate.Flags, context *hookstate.Context) ([]*state.TaskSet, error) {
 		testServiceControlInputs(appInfos, inst)
 		return nil, fmt.Errorf("forced error")
 	})
@@ -189,14 +195,7 @@ func (s *servicectlSuite) SetUpTest(c *C) {
 
 	s.st.Set("seeded", true)
 	s.st.Set("refresh-privacy-key", "privacy-key")
-	snapstate.Model = func(*state.State) (*asserts.Model, error) {
-		return sysdb.GenericClassicModel(), nil
-	}
-}
-
-func (s *servicectlSuite) TearDownTest(c *C) {
-	s.BaseTest.TearDownTest(c)
-	snapstate.Model = nil
+	s.AddCleanup(snapstatetest.UseFallbackDeviceModel())
 }
 
 func (s *servicectlSuite) TestStopCommand(c *C) {
@@ -310,6 +309,47 @@ func (s *servicectlSuite) TestConflictingChange(c *C) {
 	c.Check(err, ErrorMatches, `snap "test-snap" has "conflicting change" change in progress`)
 }
 
+var (
+	installTaskKinds = []string{
+		"prerequisites",
+		"download-snap",
+		"validate-snap",
+		"mount-snap",
+		"copy-snap-data",
+		"setup-profiles",
+		"link-snap",
+		"auto-connect",
+		"set-auto-aliases",
+		"setup-aliases",
+		"run-hook[install]",
+		"start-snap-services",
+		"run-hook[configure]",
+		"run-hook[check-health]",
+	}
+
+	refreshTaskKinds = []string{
+		"prerequisites",
+		"download-snap",
+		"validate-snap",
+		"mount-snap",
+		"run-hook[pre-refresh]",
+		"stop-snap-services",
+		"remove-aliases",
+		"unlink-current-snap",
+		"copy-snap-data",
+		"setup-profiles",
+		"link-snap",
+		"auto-connect",
+		"set-auto-aliases",
+		"setup-aliases",
+		"run-hook[post-refresh]",
+		"start-snap-services",
+		"cleanup",
+		"run-hook[configure]",
+		"run-hook[check-health]",
+	}
+)
+
 func (s *servicectlSuite) TestQueuedCommands(c *C) {
 	s.st.Lock()
 
@@ -318,8 +358,8 @@ func (s *servicectlSuite) TestQueuedCommands(c *C) {
 	c.Assert(err, IsNil)
 	c.Check(installed, DeepEquals, []string{"one", "two"})
 	c.Assert(tts, HasLen, 2)
-	c.Assert(tts[0].Tasks(), HasLen, 13)
-	c.Assert(tts[1].Tasks(), HasLen, 13)
+	c.Assert(taskKinds(tts[0].Tasks()), DeepEquals, installTaskKinds)
+	c.Assert(taskKinds(tts[1].Tasks()), DeepEquals, installTaskKinds)
 	chg.AddAll(tts[0])
 	chg.AddAll(tts[1])
 
@@ -345,14 +385,107 @@ func (s *servicectlSuite) TestQueuedCommands(c *C) {
 	s.st.Lock()
 	defer s.st.Unlock()
 
-	for i := 1; i <= 2; i++ {
-		laneTasks := chg.LaneTasks(i)
-		c.Assert(laneTasks, HasLen, 16)
+	expectedTaskKinds := append(installTaskKinds, "exec-command", "service-control", "exec-command", "service-control", "exec-command", "service-control")
+	checkLaneTasks := func(lane int) {
+		laneTasks := chg.LaneTasks(lane)
+		c.Assert(taskKinds(laneTasks), DeepEquals, expectedTaskKinds)
 		c.Check(laneTasks[12].Summary(), Matches, `Run configure hook of .* snap if present`)
-		c.Check(laneTasks[13].Summary(), Equals, "stop of [test-snap.test-service]")
-		c.Check(laneTasks[14].Summary(), Equals, "start of [test-snap.test-service]")
-		c.Check(laneTasks[15].Summary(), Equals, "restart of [test-snap.test-service]")
+		c.Check(laneTasks[14].Summary(), Equals, "stop of [test-snap.test-service]")
+		c.Check(laneTasks[16].Summary(), Equals, "start of [test-snap.test-service]")
+		c.Check(laneTasks[18].Summary(), Equals, "restart of [test-snap.test-service]")
 	}
+	checkLaneTasks(1)
+	checkLaneTasks(2)
+}
+
+func (s *servicectlSuite) testQueueCommandsOrdering(c *C, finalTaskKind string) {
+	s.st.Lock()
+
+	chg := s.st.NewChange("seeding change", "seeding change")
+	finalTask := s.st.NewTask(finalTaskKind, "")
+	chg.AddTask(finalTask)
+	configure := s.st.NewTask("run-hook", "")
+	chg.AddTask(configure)
+
+	s.st.Unlock()
+
+	setup := &hookstate.HookSetup{Snap: "test-snap", Revision: snap.R(1), Hook: "configure"}
+	context, err := hookstate.NewContext(configure, configure.State(), setup, s.mockHandler, "")
+	c.Assert(err, IsNil)
+
+	_, _, err = ctlcmd.Run(context, []string{"stop", "test-snap.test-service"}, 0)
+	c.Check(err, IsNil)
+	_, _, err = ctlcmd.Run(context, []string{"start", "test-snap.test-service"}, 0)
+	c.Check(err, IsNil)
+
+	s.st.Lock()
+	defer s.st.Unlock()
+
+	var finalWaitTasks []string
+	for _, t := range finalTask.WaitTasks() {
+		taskInfo := fmt.Sprintf("%s:%s", t.Kind(), t.Summary())
+		finalWaitTasks = append(finalWaitTasks, taskInfo)
+
+		var wait []string
+		var hasRunHook bool
+		for _, wt := range t.WaitTasks() {
+			if wt.Kind() != "run-hook" {
+				taskInfo = fmt.Sprintf("%s:%s", wt.Kind(), wt.Summary())
+				wait = append(wait, taskInfo)
+			} else {
+				hasRunHook = true
+			}
+		}
+		c.Assert(hasRunHook, Equals, true)
+
+		switch t.Kind() {
+		case "exec-command":
+			var argv []string
+			c.Assert(t.Get("argv", &argv), IsNil)
+			c.Check(argv, HasLen, 3)
+			switch argv[1] {
+			case "stop":
+				c.Check(wait, HasLen, 0)
+			case "start":
+				c.Check(wait, DeepEquals, []string{
+					`exec-command:stop of [test-snap.test-service]`,
+					`service-control:Run service command "stop" for services ["test-service"] of snap "test-snap"`})
+			default:
+				c.Fatalf("unexpected command: %q", argv[1])
+			}
+		case "service-control":
+			var sa servicestate.ServiceAction
+			c.Assert(t.Get("service-action", &sa), IsNil)
+			c.Check(sa.Services, DeepEquals, []string{"test-service"})
+			switch sa.Action {
+			case "stop":
+				c.Check(wait, DeepEquals, []string{
+					"exec-command:stop of [test-snap.test-service]"})
+			case "start":
+				c.Check(wait, DeepEquals, []string{
+					"exec-command:start of [test-snap.test-service]",
+					"exec-command:stop of [test-snap.test-service]",
+					`service-control:Run service command "stop" for services ["test-service"] of snap "test-snap"`})
+			}
+		default:
+			c.Fatalf("unexpected task: %s", t.Kind())
+		}
+
+	}
+	c.Check(finalWaitTasks, DeepEquals, []string{
+		`exec-command:stop of [test-snap.test-service]`,
+		`service-control:Run service command "stop" for services ["test-service"] of snap "test-snap"`,
+		`exec-command:start of [test-snap.test-service]`,
+		`service-control:Run service command "start" for services ["test-service"] of snap "test-snap"`})
+	c.Check(finalTask.HaltTasks(), HasLen, 0)
+}
+
+func (s *servicectlSuite) TestQueuedCommandsRunBeforeMarkSeeded(c *C) {
+	s.testQueueCommandsOrdering(c, "mark-seeded")
+}
+
+func (s *servicectlSuite) TestQueuedCommandsRunBeforeSetModel(c *C) {
+	s.testQueueCommandsOrdering(c, "set-model")
 }
 
 func (s *servicectlSuite) TestQueuedCommandsUpdateMany(c *C) {
@@ -365,19 +498,21 @@ func (s *servicectlSuite) TestQueuedCommandsUpdateMany(c *C) {
 	s.st.Lock()
 
 	chg := s.st.NewChange("update many change", "update change")
-	installed, tts, err := snapstate.UpdateMany(context.TODO(), s.st, []string{"test-snap", "other-snap"}, 0, nil)
+	installed, tts, err := snapstate.UpdateMany(context.Background(), s.st, []string{"test-snap", "other-snap"}, 0, nil)
 	c.Assert(err, IsNil)
 	sort.Strings(installed)
 	c.Check(installed, DeepEquals, []string{"other-snap", "test-snap"})
-	c.Assert(tts, HasLen, 2)
-	c.Assert(tts[0].Tasks(), HasLen, 18)
-	c.Assert(tts[1].Tasks(), HasLen, 18)
+	c.Assert(tts, HasLen, 3)
+	c.Assert(taskKinds(tts[0].Tasks()), DeepEquals, refreshTaskKinds)
+	c.Assert(taskKinds(tts[1].Tasks()), DeepEquals, refreshTaskKinds)
+	c.Assert(taskKinds(tts[2].Tasks()), DeepEquals, []string{"check-rerefresh"})
+	c.Assert(tts[2].Tasks()[0].Kind(), Equals, "check-rerefresh")
 	chg.AddAll(tts[0])
 	chg.AddAll(tts[1])
 
 	s.st.Unlock()
 
-	for _, ts := range tts {
+	for _, ts := range tts[:2] {
 		tsTasks := ts.Tasks()
 		// assumes configure task is last
 		task := tsTasks[len(tsTasks)-1]
@@ -397,13 +532,17 @@ func (s *servicectlSuite) TestQueuedCommandsUpdateMany(c *C) {
 	s.st.Lock()
 	defer s.st.Unlock()
 
+	expectedTaskKinds := append(refreshTaskKinds, "exec-command", "service-control", "exec-command", "service-control", "exec-command", "service-control")
 	for i := 1; i <= 2; i++ {
 		laneTasks := chg.LaneTasks(i)
-		c.Assert(laneTasks, HasLen, 21)
+		c.Assert(taskKinds(laneTasks), DeepEquals, expectedTaskKinds)
 		c.Check(laneTasks[17].Summary(), Matches, `Run configure hook of .* snap if present`)
-		c.Check(laneTasks[18].Summary(), Equals, "stop of [test-snap.test-service]")
-		c.Check(laneTasks[19].Summary(), Equals, "start of [test-snap.test-service]")
-		c.Check(laneTasks[20].Summary(), Equals, "restart of [test-snap.test-service]")
+		c.Check(laneTasks[19].Summary(), Equals, "stop of [test-snap.test-service]")
+		c.Check(laneTasks[20].Summary(), Equals, `Run service command "stop" for services ["test-service"] of snap "test-snap"`)
+		c.Check(laneTasks[21].Summary(), Equals, "start of [test-snap.test-service]")
+		c.Check(laneTasks[22].Summary(), Equals, `Run service command "start" for services ["test-service"] of snap "test-snap"`)
+		c.Check(laneTasks[23].Summary(), Equals, "restart of [test-snap.test-service]")
+		c.Check(laneTasks[24].Summary(), Equals, `Run service command "restart" for services ["test-service"] of snap "test-snap"`)
 	}
 }
 
@@ -411,9 +550,9 @@ func (s *servicectlSuite) TestQueuedCommandsSingleLane(c *C) {
 	s.st.Lock()
 
 	chg := s.st.NewChange("install change", "install change")
-	ts, err := snapstate.Install(s.st, "one", "", snap.R(1), 0, snapstate.Flags{})
+	ts, err := snapstate.Install(context.Background(), s.st, "one", &snapstate.RevisionOptions{Revision: snap.R(1)}, 0, snapstate.Flags{})
 	c.Assert(err, IsNil)
-	c.Assert(ts.Tasks(), HasLen, 13)
+	c.Assert(taskKinds(ts.Tasks()), DeepEquals, installTaskKinds)
 	chg.AddAll(ts)
 
 	s.st.Unlock()
@@ -437,11 +576,11 @@ func (s *servicectlSuite) TestQueuedCommandsSingleLane(c *C) {
 	defer s.st.Unlock()
 
 	laneTasks := chg.LaneTasks(0)
-	c.Assert(laneTasks, HasLen, 16)
+	c.Assert(taskKinds(laneTasks), DeepEquals, append(installTaskKinds, "exec-command", "service-control", "exec-command", "service-control", "exec-command", "service-control"))
 	c.Check(laneTasks[12].Summary(), Matches, `Run configure hook of .* snap if present`)
-	c.Check(laneTasks[13].Summary(), Equals, "stop of [test-snap.test-service]")
-	c.Check(laneTasks[14].Summary(), Equals, "start of [test-snap.test-service]")
-	c.Check(laneTasks[15].Summary(), Equals, "restart of [test-snap.test-service]")
+	c.Check(laneTasks[14].Summary(), Equals, "stop of [test-snap.test-service]")
+	c.Check(laneTasks[16].Summary(), Equals, "start of [test-snap.test-service]")
+	c.Check(laneTasks[18].Summary(), Equals, "restart of [test-snap.test-service]")
 }
 
 func (s *servicectlSuite) TestTwoServices(c *C) {
