@@ -32,6 +32,7 @@ import (
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/hotplug"
 	"github.com/snapcore/snapd/logger"
+	"github.com/snapcore/snapd/overlord/hookstate"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/snap"
@@ -80,7 +81,7 @@ func (m *InterfaceManager) doSetupProfiles(task *state.Task, tomb *tomb.Tomb) er
 	task.State().Lock()
 	defer task.State().Unlock()
 
-	perfTimings := timings.NewForTask(task)
+	perfTimings := state.TimingsForTask(task)
 	defer perfTimings.Save(task.State())
 
 	// Get snap.Info from bits handed by the snap manager.
@@ -94,6 +95,10 @@ func (m *InterfaceManager) doSetupProfiles(task *state.Task, tomb *tomb.Tomb) er
 		return err
 	}
 
+	if len(snapInfo.BadInterfaces) > 0 {
+		task.State().Warnf("%s", snap.BadInterfacesSummary(snapInfo))
+	}
+
 	// We no longer do/need core-phase-2, see
 	//   https://github.com/snapcore/snapd/pull/5301
 	// This code is just here to deal with old state that may still
@@ -209,7 +214,7 @@ func (m *InterfaceManager) doRemoveProfiles(task *state.Task, tomb *tomb.Tomb) e
 	st.Lock()
 	defer st.Unlock()
 
-	perfTimings := timings.NewForTask(task)
+	perfTimings := state.TimingsForTask(task)
 	defer perfTimings.Save(st)
 
 	// Get SnapSetup for this snap. This is gives us the name of the snap.
@@ -254,7 +259,7 @@ func (m *InterfaceManager) undoSetupProfiles(task *state.Task, tomb *tomb.Tomb)
 	st.Lock()
 	defer st.Unlock()
 
-	perfTimings := timings.NewForTask(task)
+	perfTimings := state.TimingsForTask(task)
 	defer perfTimings.Save(st)
 
 	var corePhase2 bool
@@ -386,7 +391,7 @@ func (m *InterfaceManager) doConnect(task *state.Task, _ *tomb.Tomb) error {
 	st.Lock()
 	defer st.Unlock()
 
-	perfTimings := timings.NewForTask(task)
+	perfTimings := state.TimingsForTask(task)
 	defer perfTimings.Save(st)
 
 	plugRef, slotRef, err := getPlugAndSlotRefs(task)
@@ -461,11 +466,14 @@ func (m *InterfaceManager) doConnect(task *state.Task, _ *tomb.Tomb) error {
 	// policy "connection" rules, other auto-connections obey the
 	// "auto-connection" rules
 	if autoConnect && !byGadget {
-		autochecker, err := newAutoConnectChecker(st, deviceCtx)
+		autochecker, err := newAutoConnectChecker(st, task, m.repo, deviceCtx)
 		if err != nil {
 			return err
 		}
-		policyChecker = autochecker.check
+		policyChecker = func(plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) (bool, error) {
+			ok, _, err := autochecker.check(plug, slot)
+			return ok, err
+		}
 	} else {
 		policyCheck, err := newConnectChecker(st, deviceCtx)
 		if err != nil {
@@ -494,6 +502,15 @@ func (m *InterfaceManager) doConnect(task *state.Task, _ *tomb.Tomb) error {
 		logger.Debugf("Connect handler: skipping setupSnapSecurity for snaps %q and %q", plug.Snap.InstanceName(), slot.Snap.InstanceName())
 	}
 
+	// For undo handler. We need to remember old state of the connection only
+	// if undesired flag is set because that means there was a remembered
+	// inactive connection already and we should restore its properties
+	// in case of undo. Otherwise we don't have to keep old-conn because undo
+	// can simply delete any trace of the connection.
+	if old, ok := conns[connRef.ID()]; ok && old.Undesired {
+		task.Set("old-conn", old)
+	}
+
 	conns[connRef.ID()] = &connState{
 		Interface:        conn.Interface(),
 		StaticPlugAttrs:  conn.Plug.StaticAttrs(),
@@ -517,7 +534,7 @@ func (m *InterfaceManager) doDisconnect(task *state.Task, _ *tomb.Tomb) error {
 	st.Lock()
 	defer st.Unlock()
 
-	perfTimings := timings.NewForTask(task)
+	perfTimings := state.TimingsForTask(task)
 	defer perfTimings.Save(st)
 
 	plugRef, slotRef, err := getPlugAndSlotRefs(task)
@@ -525,11 +542,19 @@ func (m *InterfaceManager) doDisconnect(task *state.Task, _ *tomb.Tomb) error {
 		return err
 	}
 
+	cref := interfaces.ConnRef{PlugRef: plugRef, SlotRef: slotRef}
+
 	conns, err := getConns(st)
 	if err != nil {
 		return err
 	}
 
+	// forget flag can be passed with snap disconnect --forget
+	var forget bool
+	if err := task.Get("forget", &forget); err != nil && err != state.ErrNoState {
+		return fmt.Errorf("internal error: cannot read 'forget' flag: %s", err)
+	}
+
 	var snapStates []snapstate.SnapState
 	for _, instanceName := range []string{plugRef.Snap, slotRef.Snap} {
 		var snapst snapstate.SnapState
@@ -544,10 +569,27 @@ func (m *InterfaceManager) doDisconnect(task *state.Task, _ *tomb.Tomb) error {
 		}
 	}
 
+	conn, ok := conns[cref.ID()]
+	if !ok {
+		return fmt.Errorf("internal error: connection %q not found in state", cref.ID())
+	}
+
+	// store old connection for undo
+	task.Set("old-conn", conn)
+
 	err = m.repo.Disconnect(plugRef.Snap, plugRef.Name, slotRef.Snap, slotRef.Name)
 	if err != nil {
+		_, notConnected := err.(*interfaces.NotConnectedError)
+		_, noPlugOrSlot := err.(*interfaces.NoPlugOrSlotError)
+		// not connected, just forget it.
+		if forget && (notConnected || noPlugOrSlot) {
+			delete(conns, cref.ID())
+			setConns(st, conns)
+			return nil
+		}
 		return fmt.Errorf("snapd changed, please retry the operation: %v", err)
 	}
+
 	for _, snapst := range snapStates {
 		snapInfo, err := snapst.CurrentInfo()
 		if err != nil {
@@ -559,15 +601,6 @@ func (m *InterfaceManager) doDisconnect(task *state.Task, _ *tomb.Tomb) error {
 		}
 	}
 
-	cref := interfaces.ConnRef{PlugRef: plugRef, SlotRef: slotRef}
-	conn, ok := conns[cref.ID()]
-	if !ok {
-		return fmt.Errorf("internal error: connection %q not found in state", cref.ID())
-	}
-
-	// store old connection for undo
-	task.Set("old-conn", conn)
-
 	// "auto-disconnect" flag indicates it's a disconnect triggered automatically as part of snap removal;
 	// such disconnects should not set undesired flag and instead just remove the connection.
 	var autoDisconnect bool
@@ -583,6 +616,8 @@ func (m *InterfaceManager) doDisconnect(task *state.Task, _ *tomb.Tomb) error {
 	}
 
 	switch {
+	case forget:
+		delete(conns, cref.ID())
 	case byHotplug:
 		conn.HotplugGone = true
 		conns[cref.ID()] = conn
@@ -606,7 +641,7 @@ func (m *InterfaceManager) undoDisconnect(task *state.Task, _ *tomb.Tomb) error
 	st.Lock()
 	defer st.Unlock()
 
-	perfTimings := timings.NewForTask(task)
+	perfTimings := state.TimingsForTask(task)
 	defer perfTimings.Save(st)
 
 	var oldconn connState
@@ -618,6 +653,11 @@ func (m *InterfaceManager) undoDisconnect(task *state.Task, _ *tomb.Tomb) error
 		return err
 	}
 
+	var forget bool
+	if err := task.Get("forget", &forget); err != nil && err != state.ErrNoState {
+		return fmt.Errorf("internal error: cannot read 'forget' flag: %s", err)
+	}
+
 	plugRef, slotRef, err := getPlugAndSlotRefs(task)
 	if err != nil {
 		return err
@@ -640,10 +680,18 @@ func (m *InterfaceManager) undoDisconnect(task *state.Task, _ *tomb.Tomb) error
 	connRef := &interfaces.ConnRef{PlugRef: plugRef, SlotRef: slotRef}
 
 	plug := m.repo.Plug(connRef.PlugRef.Snap, connRef.PlugRef.Name)
+	slot := m.repo.Slot(connRef.SlotRef.Snap, connRef.SlotRef.Name)
+	if forget && (plug == nil || slot == nil) {
+		// we were trying to forget an inactive connection that was
+		// referring to a non-existing plug or slot; just restore it
+		// in the conns state but do not reconnect via repository.
+		conns[connRef.ID()] = &oldconn
+		setConns(st, conns)
+		return nil
+	}
 	if plug == nil {
 		return fmt.Errorf("snap %q has no %q plug", connRef.PlugRef.Snap, connRef.PlugRef.Name)
 	}
-	slot := m.repo.Slot(connRef.SlotRef.Snap, connRef.SlotRef.Name)
 	if slot == nil {
 		return fmt.Errorf("snap %q has no %q slot", connRef.SlotRef.Snap, connRef.SlotRef.Name)
 	}
@@ -673,6 +721,9 @@ func (m *InterfaceManager) undoConnect(task *state.Task, _ *tomb.Tomb) error {
 	st.Lock()
 	defer st.Unlock()
 
+	perfTimings := state.TimingsForTask(task)
+	defer perfTimings.Save(st)
+
 	plugRef, slotRef, err := getPlugAndSlotRefs(task)
 	if err != nil {
 		return err
@@ -682,8 +733,66 @@ func (m *InterfaceManager) undoConnect(task *state.Task, _ *tomb.Tomb) error {
 	if err != nil {
 		return err
 	}
-	delete(conns, connRef.ID())
+
+	var old connState
+	err = task.Get("old-conn", &old)
+	if err != nil && err != state.ErrNoState {
+		return err
+	}
+	if err == nil {
+		conns[connRef.ID()] = &old
+	} else {
+		delete(conns, connRef.ID())
+	}
 	setConns(st, conns)
+
+	if err := m.repo.Disconnect(connRef.PlugRef.Snap, connRef.PlugRef.Name, connRef.SlotRef.Snap, connRef.SlotRef.Name); err != nil {
+		return err
+	}
+
+	var delayedSetupProfiles bool
+	if err := task.Get("delayed-setup-profiles", &delayedSetupProfiles); err != nil && err != state.ErrNoState {
+		return err
+	}
+	if delayedSetupProfiles {
+		logger.Debugf("Connect undo handler: skipping setupSnapSecurity for snaps %q and %q", connRef.PlugRef.Snap, connRef.SlotRef.Snap)
+		return nil
+	}
+
+	plug := m.repo.Plug(connRef.PlugRef.Snap, connRef.PlugRef.Name)
+	if plug == nil {
+		return fmt.Errorf("internal error: snap %q has no %q plug", connRef.PlugRef.Snap, connRef.PlugRef.Name)
+	}
+	slot := m.repo.Slot(connRef.SlotRef.Snap, connRef.SlotRef.Name)
+	if slot == nil {
+		return fmt.Errorf("internal error: snap %q has no %q slot", connRef.SlotRef.Snap, connRef.SlotRef.Name)
+	}
+
+	var plugSnapst snapstate.SnapState
+	err = snapstate.Get(st, plugRef.Snap, &plugSnapst)
+	if err == state.ErrNoState {
+		return fmt.Errorf("internal error: snap %q is no longer available", plugRef.Snap)
+	}
+	if err != nil {
+		return err
+	}
+	var slotSnapst snapstate.SnapState
+	err = snapstate.Get(st, slotRef.Snap, &slotSnapst)
+	if err == state.ErrNoState {
+		return fmt.Errorf("internal error: snap %q is no longer available", slotRef.Snap)
+	}
+	if err != nil {
+		return err
+	}
+	slotOpts := confinementOptions(slotSnapst.Flags)
+	if err := m.setupSnapSecurity(task, slot.Snap, slotOpts, perfTimings); err != nil {
+		return err
+	}
+	plugOpts := confinementOptions(plugSnapst.Flags)
+	if err := m.setupSnapSecurity(task, plug.Snap, plugOpts, perfTimings); err != nil {
+		return err
+	}
+
 	return nil
 }
 
@@ -693,25 +802,6 @@ var contentLinkRetryTimeout = 30 * time.Second
 // timeout for retrying hotplug-related tasks
 var hotplugRetryTimeout = 300 * time.Millisecond
 
-// defaultContentProviders returns a dict of the default-providers for the
-// content plugs for the given instanceName
-func (m *InterfaceManager) defaultContentProviders(instanceName string) map[string]bool {
-	plugs := m.repo.Plugs(instanceName)
-	defaultProviders := make(map[string]bool, len(plugs))
-	for _, plug := range plugs {
-		if plug.Interface == "content" {
-			var s string
-			if err := plug.Attr("content", &s); err == nil && s != "" {
-				var dprovider string
-				if err := plug.Attr("default-provider", &dprovider); err == nil && dprovider != "" {
-					defaultProviders[dprovider] = true
-				}
-			}
-		}
-	}
-	return defaultProviders
-}
-
 func obsoleteCorePhase2SetupProfiles(kind string, task *state.Task) (bool, error) {
 	if kind != "setup-profiles" {
 		return false, nil
@@ -914,21 +1004,33 @@ func waitChainSearch(startT, searchT *state.Task) bool {
 // The "delayed-setup-profiles" flag is set on the connect tasks to
 // indicate that doConnect handler should not set security backends up
 // because this will be done later by the setup-profiles task.
-func batchConnectTasks(st *state.State, snapsup *snapstate.SnapSetup, conns map[string]*interfaces.ConnRef, autoconnect bool) (*state.TaskSet, error) {
+func batchConnectTasks(st *state.State, snapsup *snapstate.SnapSetup, conns map[string]*interfaces.ConnRef, connOpts map[string]*connectOpts) (ts *state.TaskSet, hasInterfaceHooks bool, err error) {
 	setupProfiles := st.NewTask("setup-profiles", fmt.Sprintf(i18n.G("Setup snap %q (%s) security profiles for auto-connections"), snapsup.InstanceName(), snapsup.Revision()))
 	setupProfiles.Set("snap-setup", snapsup)
 
-	ts := state.NewTaskSet()
-	for _, conn := range conns {
-		connectTs, err := connect(st, conn.PlugRef.Snap, conn.PlugRef.Name, conn.SlotRef.Snap, conn.SlotRef.Name, connectOpts{AutoConnect: autoconnect, DelayedSetupProfiles: true})
+	ts = state.NewTaskSet()
+	for connID, conn := range conns {
+		var opts connectOpts
+		if providedOpts := connOpts[connID]; providedOpts != nil {
+			opts = *providedOpts
+		} else {
+			// default
+			opts.AutoConnect = true
+		}
+		opts.DelayedSetupProfiles = true
+		connectTs, err := connect(st, conn.PlugRef.Snap, conn.PlugRef.Name, conn.SlotRef.Snap, conn.SlotRef.Name, opts)
 		if err != nil {
-			return nil, fmt.Errorf("internal error: auto-connect of %q failed: %s", conn, err)
+			return nil, false, fmt.Errorf("internal error: auto-connect of %q failed: %s", conn, err)
+		}
+
+		if len(connectTs.Tasks()) > 1 {
+			hasInterfaceHooks = true
 		}
 
 		// setup-profiles needs to wait for the main "connect" task
 		connectTask, _ := connectTs.Edge(ConnectTaskEdge)
 		if connectTask == nil {
-			return nil, fmt.Errorf("internal error: no 'connect' task found for %q", conn)
+			return nil, false, fmt.Errorf("internal error: no 'connect' task found for %q", conn)
 		}
 		setupProfiles.WaitFor(connectTask)
 
@@ -942,7 +1044,39 @@ func batchConnectTasks(st *state.State, snapsup *snapstate.SnapSetup, conns map[
 	if len(ts.Tasks()) > 0 {
 		ts.AddTask(setupProfiles)
 	}
-	return ts, nil
+	return ts, hasInterfaceHooks, nil
+}
+
+// firstTaskAfterBootWhenPreseeding finds the first task to be run for thisSnap
+// on first boot after mark-preseeded task, this is always the install hook.
+// It is an internal error if install hook for thisSnap cannot be found.
+func firstTaskAfterBootWhenPreseeding(thisSnap string, markPreseeded *state.Task) (*state.Task, error) {
+	if markPreseeded.Change() == nil {
+		return nil, fmt.Errorf("internal error: %s task not in change", markPreseeded.Kind())
+	}
+	for _, ht := range markPreseeded.HaltTasks() {
+		if ht.Kind() == "run-hook" {
+			var hs hookstate.HookSetup
+			if err := ht.Get("hook-setup", &hs); err != nil {
+				return nil, fmt.Errorf("internal error: cannot get hook setup: %v", err)
+			}
+			if hs.Hook == "install" && hs.Snap == thisSnap {
+				return ht, nil
+			}
+		}
+	}
+	return nil, fmt.Errorf("internal error: cannot find install hook for snap %q", thisSnap)
+}
+
+func filterForSlot(slot *snap.SlotInfo) func(candSlots []*snap.SlotInfo) []*snap.SlotInfo {
+	return func(candSlots []*snap.SlotInfo) []*snap.SlotInfo {
+		for _, candSlot := range candSlots {
+			if candSlot.String() == slot.String() {
+				return []*snap.SlotInfo{slot}
+			}
+		}
+		return nil
+	}
 }
 
 // doAutoConnect creates task(s) to connect the given snap to viable candidates.
@@ -967,24 +1101,32 @@ func (m *InterfaceManager) doAutoConnect(task *state.Task, _ *tomb.Tomb) error {
 	}
 
 	// The previous task (link-snap) may have triggered a restart,
-	// if this is the case we can only procceed once the restart
+	// if this is the case we can only proceed once the restart
 	// has happened or we may not have all the interfaces of the
 	// new core/base snap.
-	if err := snapstate.WaitRestart(task, snapsup); err != nil {
+	if err := snapstate.FinishRestart(task, snapsup); err != nil {
 		return err
 	}
 
 	snapName := snapsup.InstanceName()
 
-	autochecker, err := newAutoConnectChecker(st, deviceCtx)
+	autochecker, err := newAutoConnectChecker(st, task, m.repo, deviceCtx)
 	if err != nil {
 		return err
 	}
 
+	gadgectConnect := newGadgetConnect(st, task, m.repo, snapName, deviceCtx)
+
 	// wait for auto-install, started by prerequisites code, for
 	// the default-providers of content ifaces so we can
-	// auto-connect to them
-	defaultProviders := m.defaultContentProviders(snapName)
+	// auto-connect to them; snapstate prerequisites does a bit
+	// more filtering than this so defaultProviders here can
+	// contain some more snaps; should not be an issue in practice
+	// given the check below checks for same chain and we don't
+	// forcefully wait for defaultProviders; we just retry for
+	// things in the intersection between defaultProviders here and
+	// snaps with not ready link-snap|setup-profiles tasks
+	defaultProviders := snap.DefaultContentProviders(m.repo.Plugs(snapName))
 	for _, chg := range st.Changes() {
 		if chg.Status().Ready() {
 			continue
@@ -1000,7 +1142,8 @@ func (m *InterfaceManager) doAutoConnect(task *state.Task, _ *tomb.Tomb) error {
 				// Only retry if the task that installs the
 				// content provider is not waiting for us
 				// (or this will just hang forever).
-				if defaultProviders[snapsup.InstanceName()] && !inSameChangeWaitChain(task, t) {
+				_, ok := defaultProviders[snapsup.InstanceName()]
+				if ok && !inSameChangeWaitChain(task, t) {
 					return &state.Retry{After: contentLinkRetryTimeout}
 				}
 			}
@@ -1010,59 +1153,36 @@ func (m *InterfaceManager) doAutoConnect(task *state.Task, _ *tomb.Tomb) error {
 	plugs := m.repo.Plugs(snapName)
 	slots := m.repo.Slots(snapName)
 	newconns := make(map[string]*interfaces.ConnRef, len(plugs)+len(slots))
+	var connOpts map[string]*connectOpts
+
+	conflictError := func(retry *state.Retry, err error) error {
+		if retry != nil {
+			task.Logf("Waiting for conflicting change in progress: %s", retry.Reason)
+			return retry // will retry
+		}
+		return fmt.Errorf("auto-connect conflict check failed: %v", err)
+	}
+
+	// Consider gadget connections, we want to remember them in
+	// any case with "by-gadget" set, so they should be processed
+	// before the auto-connection ones.
+	if err := gadgectConnect.addGadgetConnections(newconns, conns, conflictError); err != nil {
+		return err
+	}
+	if len(newconns) > 0 {
+		connOpts = make(map[string]*connectOpts, len(newconns))
+		byGadgetOpts := &connectOpts{AutoConnect: true, ByGadget: true}
+		for key := range newconns {
+			connOpts[key] = byGadgetOpts
+		}
+	}
 
 	// Auto-connect all the plugs
-	for _, plug := range plugs {
-		candidates := m.repo.AutoConnectCandidateSlots(snapName, plug.Name, autochecker.check)
-		if len(candidates) == 0 {
-			continue
-		}
-		// If we are in a core transition we may have both the old ubuntu-core
-		// snap and the new core snap providing the same interface. In that
-		// situation we want to ignore any candidates in ubuntu-core and simply
-		// go with those from the new core snap.
-		if len(candidates) == 2 {
-			switch {
-			case candidates[0].Snap.InstanceName() == "ubuntu-core" && candidates[1].Snap.InstanceName() == "core":
-				candidates = candidates[1:2]
-			case candidates[1].Snap.InstanceName() == "ubuntu-core" && candidates[0].Snap.InstanceName() == "core":
-				candidates = candidates[0:1]
-			}
-		}
-		if len(candidates) != 1 {
-			crefs := make([]string, len(candidates))
-			for i, candidate := range candidates {
-				crefs[i] = candidate.String()
-			}
-			task.Logf("cannot auto-connect plug %s, candidates found: %s", plug, strings.Join(crefs, ", "))
-			continue
-		}
-		slot := candidates[0]
-		connRef := interfaces.NewConnRef(plug, slot)
-		key := connRef.ID()
-		if _, ok := conns[key]; ok {
-			// Suggested connection already exist (or has Undesired flag set) so don't clobber it.
-			// NOTE: we don't log anything here as this is a normal and common condition.
-			continue
-		}
-
-		ignore, err := findSymmetricAutoconnectTask(st, plug.Snap.InstanceName(), slot.Snap.InstanceName(), task)
-		if err != nil {
-			return err
-		}
-
-		if ignore {
-			continue
-		}
-
-		if err := checkAutoconnectConflicts(st, task, plug.Snap.InstanceName(), slot.Snap.InstanceName()); err != nil {
-			if retry, ok := err.(*state.Retry); ok {
-				task.Logf("Waiting for conflicting change in progress: %s", retry.Reason)
-				return err // will retry
-			}
-			return fmt.Errorf("auto-connect conflict check failed: %s", err)
-		}
-		newconns[connRef.ID()] = connRef
+	cannotAutoConnectLog := func(plug *snap.PlugInfo, candRefs []string) string {
+		return fmt.Sprintf("cannot auto-connect plug %s, candidates found: %s", plug, strings.Join(candRefs, ", "))
+	}
+	if err := autochecker.addAutoConnections(newconns, plugs, nil, conns, cannotAutoConnectLog, conflictError); err != nil {
+		return err
 	}
 	// Auto-connect all the slots
 	for _, slot := range slots {
@@ -1071,58 +1191,55 @@ func (m *InterfaceManager) doAutoConnect(task *state.Task, _ *tomb.Tomb) error {
 			continue
 		}
 
-		for _, plug := range candidates {
-			// make sure slot is the only viable
-			// connection for plug, same check as if we were
-			// considering auto-connections from plug
-			candSlots := m.repo.AutoConnectCandidateSlots(plug.Snap.InstanceName(), plug.Name, autochecker.check)
-
-			if len(candSlots) != 1 || candSlots[0].String() != slot.String() {
-				crefs := make([]string, len(candSlots))
-				for i, candidate := range candSlots {
-					crefs[i] = candidate.String()
-				}
-				task.Logf("cannot auto-connect slot %s to %s, candidates found: %s", slot, plug, strings.Join(crefs, ", "))
-				continue
-			}
-
-			connRef := interfaces.NewConnRef(plug, slot)
-			key := connRef.ID()
-			if _, ok := conns[key]; ok {
-				// Suggested connection already exist (or has Undesired flag set) so don't clobber it.
-				// NOTE: we don't log anything here as this is a normal and common condition.
-				continue
-			}
-			if _, ok := newconns[key]; ok {
-				continue
-			}
-
-			ignore, err := findSymmetricAutoconnectTask(st, plug.Snap.InstanceName(), slot.Snap.InstanceName(), task)
-			if err != nil {
-				return err
-			}
-
-			if ignore {
-				continue
-			}
-
-			if err := checkAutoconnectConflicts(st, task, plug.Snap.InstanceName(), slot.Snap.InstanceName()); err != nil {
-				if retry, ok := err.(*state.Retry); ok {
-					task.Logf("Waiting for conflicting change in progress: %s", retry.Reason)
-					return err // will retry
-				}
-				return fmt.Errorf("auto-connect conflict check failed: %s", err)
-			}
-			newconns[connRef.ID()] = connRef
+		cannotAutoConnectLog := func(plug *snap.PlugInfo, candRefs []string) string {
+			return fmt.Sprintf("cannot auto-connect slot %s to plug %s, candidates found: %s", slot, plug, strings.Join(candRefs, ", "))
+		}
+		if err := autochecker.addAutoConnections(newconns, candidates, filterForSlot(slot), conns, cannotAutoConnectLog, conflictError); err != nil {
+			return err
 		}
 	}
 
-	autoconnect := true
-	autots, err := batchConnectTasks(st, snapsup, newconns, autoconnect)
+	autots, hasInterfaceHooks, err := batchConnectTasks(st, snapsup, newconns, connOpts)
 	if err != nil {
 		return err
 	}
 
+	// If interface hooks are not present then connects can be executed during
+	// preseeding.
+	// Otherwise we will run all connects, their hooks and setup-profiles after
+	// preseeding (on first boot). Note, we may be facing multiple connections
+	// here where only some have hooks; however there is no point in running
+	// those without hooks before mark-preseeded, because only setup-profiles is
+	// performance-critical and it still needs to run after those with hooks.
+	if m.preseed && hasInterfaceHooks {
+		for _, t := range st.Tasks() {
+			if t.Kind() == "mark-preseeded" {
+				markPreseeded := t
+				// consistency check
+				if markPreseeded.Status() != state.DoStatus {
+					return fmt.Errorf("internal error: unexpected state of mark-preseeded task: %s", markPreseeded.Status())
+				}
+
+				firstTaskAfterBoot, err := firstTaskAfterBootWhenPreseeding(snapsup.InstanceName(), markPreseeded)
+				if err != nil {
+					return err
+				}
+				// first task of the snap that normally runs on first boot
+				// needs to wait on connects & interface hooks.
+				firstTaskAfterBoot.WaitAll(autots)
+
+				// connect tasks and interface hooks need to wait for end of preseeding
+				// (they need to run on first boot, not during preseeding).
+				autots.WaitFor(markPreseeded)
+				t.Change().AddAll(autots)
+				task.SetStatus(state.DoneStatus)
+				st.EnsureBefore(0)
+				return nil
+			}
+		}
+		return fmt.Errorf("internal error: mark-preseeded task not found in preseeding mode")
+	}
+
 	if len(autots.Tasks()) > 0 {
 		snapstate.InjectTasks(task, autots)
 
@@ -1266,88 +1383,6 @@ func (m *InterfaceManager) undoTransitionUbuntuCore(t *state.Task, _ *tomb.Tomb)
 	return m.transitionConnectionsCoreMigration(st, newName, oldName)
 }
 
-// doGadgetConnect creates task(s) to follow the interface connection instructions from the gadget.
-func (m *InterfaceManager) doGadgetConnect(task *state.Task, _ *tomb.Tomb) error {
-	st := task.State()
-	st.Lock()
-	defer st.Unlock()
-
-	deviceCtx, err := snapstate.DeviceCtx(st, task, nil)
-	if err != nil {
-		return err
-	}
-
-	conns, err := getConns(st)
-	if err != nil {
-		return err
-	}
-
-	gconns, err := snapstate.GadgetConnections(st, deviceCtx)
-	if err != nil {
-		return err
-	}
-
-	gconnts := state.NewTaskSet()
-	var newconns []*interfaces.ConnRef
-
-	// consider the gadget connect instructions
-	for _, gconn := range gconns {
-		plugSnapName, err := resolveSnapIDToName(st, gconn.Plug.SnapID)
-		if err != nil {
-			return err
-		}
-		plug := m.repo.Plug(plugSnapName, gconn.Plug.Plug)
-		if plug == nil {
-			task.Logf("gadget connect: ignoring missing plug %s:%s", gconn.Plug.SnapID, gconn.Plug.Plug)
-			continue
-		}
-
-		slotSnapName, err := resolveSnapIDToName(st, gconn.Slot.SnapID)
-		if err != nil {
-			return err
-		}
-		slot := m.repo.Slot(slotSnapName, gconn.Slot.Slot)
-		if slot == nil {
-			task.Logf("gadget connect: ignoring missing slot %s:%s", gconn.Slot.SnapID, gconn.Slot.Slot)
-			continue
-		}
-
-		connRef := interfaces.NewConnRef(plug, slot)
-		key := connRef.ID()
-		if _, ok := conns[key]; ok {
-			// Gadget connection already exist (or has Undesired flag set) so don't clobber it.
-			continue
-		}
-
-		if err := checkAutoconnectConflicts(st, task, plug.Snap.InstanceName(), slot.Snap.InstanceName()); err != nil {
-			if retry, ok := err.(*state.Retry); ok {
-				task.Logf("gadget connect will be retried: %s", retry.Reason)
-				return err // will retry
-			}
-			return fmt.Errorf("gadget connect conflict check failed: %s", err)
-		}
-		newconns = append(newconns, connRef)
-	}
-
-	// Create connect tasks and interface hooks
-	for _, conn := range newconns {
-		ts, err := connect(st, conn.PlugRef.Snap, conn.PlugRef.Name, conn.SlotRef.Snap, conn.SlotRef.Name, connectOpts{AutoConnect: true, ByGadget: true})
-		if err != nil {
-			return fmt.Errorf("internal error: connect of %q failed: %s", conn, err)
-		}
-		gconnts.AddAll(ts)
-	}
-
-	if len(gconnts.Tasks()) > 0 {
-		snapstate.InjectTasks(task, gconnts)
-
-		st.EnsureBefore(0)
-	}
-
-	task.SetStatus(state.DoneStatus)
-	return nil
-}
-
 // doHotplugConnect creates task(s) to (re)create old connections or auto-connect viable slots in response to hotplug "add" event.
 func (m *InterfaceManager) doHotplugConnect(task *state.Task, _ *tomb.Tomb) error {
 	st := task.State()
@@ -1381,6 +1416,14 @@ func (m *InterfaceManager) doHotplugConnect(task *state.Task, _ *tomb.Tomb) erro
 	// to recreate old connections that are only remembered in the state.
 	connsForDevice := findConnsForHotplugKey(conns, ifaceName, hotplugKey)
 
+	conflictError := func(retry *state.Retry, err error) error {
+		if retry != nil {
+			task.Logf("hotplug connect will be retried: %s", retry.Reason)
+			return retry // will retry
+		}
+		return fmt.Errorf("hotplug-connect conflict check failed: %v", err)
+	}
+
 	// find old connections to recreate
 	var recreate []*interfaces.ConnRef
 	for _, id := range connsForDevice {
@@ -1399,17 +1442,14 @@ func (m *InterfaceManager) doHotplugConnect(task *state.Task, _ *tomb.Tomb) erro
 		}
 
 		if err := checkAutoconnectConflicts(st, task, connRef.PlugRef.Snap, connRef.SlotRef.Snap); err != nil {
-			if retry, ok := err.(*state.Retry); ok {
-				task.Logf("hotplug connect will be retried: %s", retry.Reason)
-				return err // will retry
-			}
-			return fmt.Errorf("hotplug connect conflict check failed: %s", err)
+			retry, _ := err.(*state.Retry)
+			return conflictError(retry, err)
 		}
 		recreate = append(recreate, connRef)
 	}
 
 	// find new auto-connections
-	autochecker, err := newAutoConnectChecker(st, deviceCtx)
+	autochecker, err := newAutoConnectChecker(st, task, m.repo, deviceCtx)
 	if err != nil {
 		return err
 	}
@@ -1417,37 +1457,13 @@ func (m *InterfaceManager) doHotplugConnect(task *state.Task, _ *tomb.Tomb) erro
 	instanceName := slot.Snap.InstanceName()
 	candidates := m.repo.AutoConnectCandidatePlugs(instanceName, slot.Name, autochecker.check)
 
-	var newconns []*interfaces.ConnRef
-	// Auto-connect the slots
-	for _, plug := range candidates {
-		// make sure slot is the only viable
-		// connection for plug, same check as if we were
-		// considering auto-connections from plug
-		candSlots := m.repo.AutoConnectCandidateSlots(plug.Snap.InstanceName(), plug.Name, autochecker.check)
-		if len(candSlots) != 1 || candSlots[0].String() != slot.String() {
-			crefs := make([]string, len(candSlots))
-			for i, candidate := range candSlots {
-				crefs[i] = candidate.String()
-			}
-			task.Logf("cannot auto-connect slot %s to %s, candidates found: %s", slot, plug, strings.Join(crefs, ", "))
-			continue
-		}
-
-		connRef := interfaces.NewConnRef(plug, slot)
-		key := connRef.ID()
-		if _, ok := conns[key]; ok {
-			// existing connection, already considered by connsForDevice loop
-			continue
-		}
-
-		if err := checkAutoconnectConflicts(st, task, plug.Snap.InstanceName(), slot.Snap.InstanceName()); err != nil {
-			if retry, ok := err.(*state.Retry); ok {
-				task.Logf("hotplug connect will be retried: %s", retry.Reason)
-				return err // will retry
-			}
-			return fmt.Errorf("hotplug connect conflict check failed: %s", err)
-		}
-		newconns = append(newconns, connRef)
+	newconns := make(map[string]*interfaces.ConnRef, len(candidates))
+	// Auto-connect the plugs
+	cannotAutoConnectLog := func(plug *snap.PlugInfo, candRefs []string) string {
+		return fmt.Sprintf("cannot auto-connect hotplug slot %s to plug %s, candidates found: %s", slot, plug, strings.Join(candRefs, ", "))
+	}
+	if err := autochecker.addAutoConnections(newconns, candidates, filterForSlot(slot), conns, cannotAutoConnectLog, conflictError); err != nil {
+		return err
 	}
 
 	if len(recreate) == 0 && len(newconns) == 0 {
