@@ -20,21 +20,26 @@
 package snapshotstate_test
 
 import (
+	"context"
 	"encoding/json"
 	"errors"
+	"os"
 	"path/filepath"
 	"sort"
+	"time"
 
-	"golang.org/x/net/context"
 	"gopkg.in/check.v1"
 	"gopkg.in/tomb.v2"
 
 	"github.com/snapcore/snapd/client"
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/logger"
+	"github.com/snapcore/snapd/overlord"
 	"github.com/snapcore/snapd/overlord/snapshotstate"
 	"github.com/snapcore/snapd/overlord/snapshotstate/backend"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/testutil"
 )
 
 func (snapshotSuite) TestManager(c *check.C) {
@@ -54,6 +59,195 @@ func (snapshotSuite) TestManager(c *check.C) {
 	})
 }
 
+func mockDummySnapshot(c *check.C) (restore func()) {
+	shotfile, err := os.Create(filepath.Join(c.MkDir(), "foo.zip"))
+	c.Assert(err, check.IsNil)
+
+	fakeIter := func(_ context.Context, f func(*backend.Reader) error) error {
+		c.Assert(f(&backend.Reader{
+			Snapshot: client.Snapshot{SetID: 1, Snap: "a-snap", SnapID: "a-id", Epoch: snap.Epoch{Read: []uint32{42}, Write: []uint32{17}}},
+			File:     shotfile,
+		}), check.IsNil)
+		return nil
+	}
+
+	restoreBackendIter := snapshotstate.MockBackendIter(fakeIter)
+
+	return func() {
+		shotfile.Close()
+		restoreBackendIter()
+	}
+}
+
+func (snapshotSuite) TestEnsureForgetsSnapshots(c *check.C) {
+	var removedSnapshot string
+	restoreOsRemove := snapshotstate.MockOsRemove(func(fileName string) error {
+		removedSnapshot = fileName
+		return nil
+	})
+	defer restoreOsRemove()
+
+	restore := mockDummySnapshot(c)
+	defer restore()
+
+	st := state.New(nil)
+	runner := state.NewTaskRunner(st)
+	mgr := snapshotstate.Manager(st, runner)
+	c.Assert(mgr, check.NotNil)
+
+	st.Lock()
+	defer st.Unlock()
+
+	st.Set("snapshots", map[uint64]interface{}{
+		1: map[string]interface{}{"expiry-time": "2001-03-11T11:24:00Z"},
+		2: map[string]interface{}{"expiry-time": "2037-02-12T12:50:00Z"},
+	})
+
+	st.Unlock()
+	c.Assert(mgr.Ensure(), check.IsNil)
+	st.Lock()
+
+	// verify expired snapshots were removed
+	var expirations map[uint64]interface{}
+	c.Assert(st.Get("snapshots", &expirations), check.IsNil)
+	c.Check(expirations, check.DeepEquals, map[uint64]interface{}{
+		2: map[string]interface{}{"expiry-time": "2037-02-12T12:50:00Z"}})
+	c.Check(removedSnapshot, check.Matches, ".*/foo.zip")
+}
+
+func (snapshotSuite) TestEnsureForgetsSnapshotsRunsRegularly(c *check.C) {
+	var backendIterCalls int
+	shotfile, err := os.Create(filepath.Join(c.MkDir(), "foo.zip"))
+	c.Assert(err, check.IsNil)
+	fakeIter := func(_ context.Context, f func(*backend.Reader) error) error {
+		c.Assert(f(&backend.Reader{
+			Snapshot: client.Snapshot{SetID: 1, Snap: "a-snap", SnapID: "a-id", Epoch: snap.Epoch{Read: []uint32{42}, Write: []uint32{17}}},
+			File:     shotfile,
+		}), check.IsNil)
+		backendIterCalls++
+		return nil
+	}
+	restoreBackendIter := snapshotstate.MockBackendIter(fakeIter)
+	defer restoreBackendIter()
+
+	restoreOsRemove := snapshotstate.MockOsRemove(func(fileName string) error {
+		return nil
+	})
+	defer restoreOsRemove()
+
+	st := state.New(nil)
+	runner := state.NewTaskRunner(st)
+	mgr := snapshotstate.Manager(st, runner)
+	c.Assert(mgr, check.NotNil)
+
+	storeExpiredSnapshot := func() {
+		st.Lock()
+		// we need at least one snapshot set in the state for forgetExpiredSnapshots to do any work
+		st.Set("snapshots", map[uint64]interface{}{
+			1: map[string]interface{}{"expiry-time": "2001-03-11T11:24:00Z"},
+		})
+		st.Unlock()
+	}
+
+	// consecutive runs of Ensure call the backend just once because of the snapshotExpirationLoopInterval
+	for i := 0; i < 3; i++ {
+		storeExpiredSnapshot()
+		c.Assert(mgr.Ensure(), check.IsNil)
+		c.Check(backendIterCalls, check.Equals, 1)
+	}
+
+	// pretend we haven't run for a while
+	t, err := time.Parse(time.RFC3339, "2002-03-11T11:24:00Z")
+	c.Assert(err, check.IsNil)
+	snapshotstate.SetLastForgetExpiredSnapshotTime(mgr, t)
+	c.Assert(mgr.Ensure(), check.IsNil)
+	c.Check(backendIterCalls, check.Equals, 2)
+
+	c.Assert(mgr.Ensure(), check.IsNil)
+	c.Check(backendIterCalls, check.Equals, 2)
+}
+
+func (snapshotSuite) testEnsureForgetSnapshotsConflict(c *check.C, snapshotOp string) {
+	removeCalled := 0
+	restoreOsRemove := snapshotstate.MockOsRemove(func(string) error {
+		removeCalled++
+		return nil
+	})
+	defer restoreOsRemove()
+
+	restore := mockDummySnapshot(c)
+	defer restore()
+
+	st := state.New(nil)
+	runner := state.NewTaskRunner(st)
+	mgr := snapshotstate.Manager(st, runner)
+	c.Assert(mgr, check.NotNil)
+
+	st.Lock()
+	defer st.Unlock()
+
+	st.Set("snapshots", map[uint64]interface{}{
+		1: map[string]interface{}{"expiry-time": "2001-03-11T11:24:00Z"},
+	})
+
+	var tsk *state.Task
+
+	switch snapshotOp {
+	case "export-snapshot":
+		snapshotstate.SetSnapshotOpInProgress(st, 1, snapshotOp)
+	default:
+		chg := st.NewChange("snapshot-change", "...")
+		tsk = st.NewTask(snapshotOp, "...")
+		tsk.SetStatus(state.DoingStatus)
+		tsk.Set("snapshot-setup", map[string]int{"set-id": 1})
+		chg.AddTask(tsk)
+	}
+
+	st.Unlock()
+	c.Assert(mgr.Ensure(), check.IsNil)
+	st.Lock()
+
+	var expirations map[uint64]interface{}
+	c.Assert(st.Get("snapshots", &expirations), check.IsNil)
+	c.Check(expirations, check.DeepEquals, map[uint64]interface{}{
+		1: map[string]interface{}{"expiry-time": "2001-03-11T11:24:00Z"},
+	})
+	c.Check(removeCalled, check.Equals, 0)
+
+	if tsk != nil {
+		// sanity check of the test setup: snapshot gets removed once conflict goes away
+		tsk.SetStatus(state.DoneStatus)
+	} else {
+		c.Check(snapshotstate.UnsetSnapshotOpInProgress(st, 1), check.Equals, snapshotOp)
+	}
+
+	// pretend we haven't run for a while
+	t, err := time.Parse(time.RFC3339, "2002-03-11T11:24:00Z")
+	c.Assert(err, check.IsNil)
+	snapshotstate.SetLastForgetExpiredSnapshotTime(mgr, t)
+
+	st.Unlock()
+	c.Assert(mgr.Ensure(), check.IsNil)
+	st.Lock()
+
+	expirations = nil
+	c.Assert(st.Get("snapshots", &expirations), check.IsNil)
+	c.Check(removeCalled, check.Equals, 1)
+	c.Check(expirations, check.HasLen, 0)
+}
+
+func (s *snapshotSuite) TestEnsureForgetSnapshotsConflictWithCheckSnapshot(c *check.C) {
+	s.testEnsureForgetSnapshotsConflict(c, "check-snapshot")
+}
+
+func (s *snapshotSuite) TestEnsureForgetSnapshotsConflictWithRestoreSnapshot(c *check.C) {
+	s.testEnsureForgetSnapshotsConflict(c, "restore-snapshot")
+}
+
+func (s *snapshotSuite) TestEnsureForgetSnapshotsConflictWithExportSnapshot(c *check.C) {
+	s.testEnsureForgetSnapshotsConflict(c, "export-snapshot")
+}
+
 func (snapshotSuite) TestFilename(c *check.C) {
 	si := &snap.Info{
 		SideInfo: snap.SideInfo{
@@ -238,6 +432,54 @@ func (snapshotSuite) TestDoSaveFailsBadConfig(c *check.C) {
 	c.Assert(err, check.ErrorMatches, ".* cannot unmarshal .*")
 }
 
+func (snapshotSuite) TestDoSaveFailureRemovesStateEntry(c *check.C) {
+	st := state.New(nil)
+
+	snapInfo := snap.Info{
+		SideInfo: snap.SideInfo{
+			RealName: "a-snap",
+			Revision: snap.R(-1),
+		},
+		Version: "1.33",
+	}
+	defer snapshotstate.MockSnapstateCurrentInfo(func(_ *state.State, snapname string) (*snap.Info, error) {
+		return &snapInfo, nil
+	})()
+	defer snapshotstate.MockConfigGetSnapConfig(func(_ *state.State, snapname string) (*json.RawMessage, error) {
+		return nil, nil
+	})()
+	defer snapshotstate.MockBackendSave(func(_ context.Context, id uint64, si *snap.Info, cfg map[string]interface{}, usernames []string) (*client.Snapshot, error) {
+		var expirations map[uint64]interface{}
+		st.Lock()
+		defer st.Unlock()
+		// verify that prepareSave stored expiration in the state
+		c.Assert(st.Get("snapshots", &expirations), check.IsNil)
+		c.Assert(expirations, check.HasLen, 1)
+		c.Check(expirations[42], check.NotNil)
+		return nil, errors.New("error")
+	})()
+
+	st.Lock()
+
+	task := st.NewTask("save-snapshot", "...")
+	task.Set("snapshot-setup", map[string]interface{}{
+		"set-id": 42,
+		"snap":   "a-snap",
+		"auto":   true,
+	})
+	st.Unlock()
+	err := snapshotstate.DoSave(task, &tomb.Tomb{})
+	c.Assert(err, check.ErrorMatches, "error")
+
+	st.Lock()
+	defer st.Unlock()
+
+	// verify that after backend.Save failure expiration was removed from the state
+	var expirations map[uint64]interface{}
+	c.Assert(st.Get("snapshots", &expirations), check.IsNil)
+	c.Check(expirations, check.HasLen, 0)
+}
+
 type readerSuite struct {
 	task     *state.Task
 	calls    []string
@@ -253,7 +495,7 @@ func (rs *readerSuite) SetUpTest(c *check.C) {
 	rs.task.Set("snapshot-setup", map[string]interface{}{
 		// interestingly restore doesn't use the set-id
 		"snap":     "a-snap",
-		"filename": "/some/file.zip",
+		"filename": "/some/1_file.zip",
 		"users":    []string{"a-user", "b-user"},
 	})
 	st.Unlock()
@@ -272,7 +514,7 @@ func (rs *readerSuite) SetUpTest(c *check.C) {
 			rs.calls = append(rs.calls, "set config")
 			return nil
 		}),
-		snapshotstate.MockBackendOpen(func(string) (*backend.Reader, error) {
+		snapshotstate.MockBackendOpen(func(string, uint64) (*backend.Reader, error) {
 			rs.calls = append(rs.calls, "open")
 			return &backend.Reader{}, nil
 		}),
@@ -306,9 +548,11 @@ func (rs *readerSuite) TestDoRestore(c *check.C) {
 		buf := json.RawMessage(`{"old": "conf"}`)
 		return &buf, nil
 	})()
-	defer snapshotstate.MockBackendOpen(func(filename string) (*backend.Reader, error) {
+	defer snapshotstate.MockBackendOpen(func(filename string, setID uint64) (*backend.Reader, error) {
 		rs.calls = append(rs.calls, "open")
-		c.Check(filename, check.Equals, "/some/file.zip")
+		// set id 0 tells backend.Open to use set id from the filename
+		c.Check(setID, check.Equals, uint64(0))
+		c.Check(filename, check.Equals, "/some/1_file.zip")
 		return &backend.Reader{
 			Snapshot: client.Snapshot{Conf: map[string]interface{}{"hello": "there"}},
 		}, nil
@@ -372,7 +616,7 @@ func (rs *readerSuite) TestDoRestoreFailsOnBadConfig(c *check.C) {
 }
 
 func (rs *readerSuite) TestDoRestoreFailsOpenError(c *check.C) {
-	defer snapshotstate.MockBackendOpen(func(string) (*backend.Reader, error) {
+	defer snapshotstate.MockBackendOpen(func(string, uint64) (*backend.Reader, error) {
 		rs.calls = append(rs.calls, "open")
 		return nil, errors.New("bzzt")
 	})()
@@ -383,7 +627,7 @@ func (rs *readerSuite) TestDoRestoreFailsOpenError(c *check.C) {
 }
 
 func (rs *readerSuite) TestDoRestoreFailsUnserialisableSnapshotConfigError(c *check.C) {
-	defer snapshotstate.MockBackendOpen(func(string) (*backend.Reader, error) {
+	defer snapshotstate.MockBackendOpen(func(string, uint64) (*backend.Reader, error) {
 		rs.calls = append(rs.calls, "open")
 		return &backend.Reader{
 			Snapshot: client.Snapshot{Conf: map[string]interface{}{"hello": func() {}}},
@@ -450,9 +694,11 @@ func (rs *readerSuite) TestCleanupRestore(c *check.C) {
 }
 
 func (rs *readerSuite) TestDoCheck(c *check.C) {
-	defer snapshotstate.MockBackendOpen(func(filename string) (*backend.Reader, error) {
+	defer snapshotstate.MockBackendOpen(func(filename string, setID uint64) (*backend.Reader, error) {
 		rs.calls = append(rs.calls, "open")
-		c.Check(filename, check.Equals, "/some/file.zip")
+		c.Check(filename, check.Equals, "/some/1_file.zip")
+		// set id 0 tells backend.Open to use set id from the filename
+		c.Check(setID, check.Equals, uint64(0))
 		return &backend.Reader{
 			Snapshot: client.Snapshot{Conf: map[string]interface{}{"hello": "there"}},
 		}, nil
@@ -466,12 +712,11 @@ func (rs *readerSuite) TestDoCheck(c *check.C) {
 	err := snapshotstate.DoCheck(rs.task, &tomb.Tomb{})
 	c.Assert(err, check.IsNil)
 	c.Check(rs.calls, check.DeepEquals, []string{"open", "check"})
-
 }
 
 func (rs *readerSuite) TestDoRemove(c *check.C) {
 	defer snapshotstate.MockOsRemove(func(filename string) error {
-		c.Check(filename, check.Equals, "/some/file.zip")
+		c.Check(filename, check.Equals, "/some/1_file.zip")
 		rs.calls = append(rs.calls, "remove")
 		return nil
 	})()
@@ -479,3 +724,82 @@ func (rs *readerSuite) TestDoRemove(c *check.C) {
 	c.Assert(err, check.IsNil)
 	c.Check(rs.calls, check.DeepEquals, []string{"remove"})
 }
+
+func (rs *readerSuite) TestDoForgetRemovesAutomaticSnapshotExpiry(c *check.C) {
+	defer snapshotstate.MockOsRemove(func(filename string) error {
+		return nil
+	})()
+
+	st := state.New(nil)
+	st.Lock()
+	defer st.Unlock()
+
+	task := st.NewTask("forget-snapshot", "...")
+	task.Set("snapshot-setup", map[string]interface{}{
+		"set-id":   1,
+		"filename": "a-file",
+		"snap":     "a-snap",
+	})
+
+	st.Set("snapshots", map[uint64]interface{}{
+		1: map[string]interface{}{
+			"expiry-time": "2001-03-11T11:24:00Z",
+		},
+		2: map[string]interface{}{
+			"expiry-time": "2037-02-12T12:50:00Z",
+		},
+	})
+
+	st.Unlock()
+	c.Assert(snapshotstate.DoForget(task, &tomb.Tomb{}), check.IsNil)
+
+	st.Lock()
+	var expirations map[uint64]interface{}
+	c.Assert(st.Get("snapshots", &expirations), check.IsNil)
+	c.Check(expirations, check.DeepEquals, map[uint64]interface{}{
+		2: map[string]interface{}{
+			"expiry-time": "2037-02-12T12:50:00Z",
+		}})
+}
+
+func (snapshotSuite) TestManagerRunCleanupAbandondedImportsAtStartup(c *check.C) {
+	n := 0
+	restore := snapshotstate.MockBackenCleanupAbandondedImports(func() (int, error) {
+		n++
+		return 0, nil
+	})
+	defer restore()
+
+	o := overlord.Mock()
+	st := o.State()
+	mgr := snapshotstate.Manager(st, state.NewTaskRunner(st))
+	c.Assert(mgr, check.NotNil)
+	o.AddManager(mgr)
+	err := o.Settle(100 * time.Millisecond)
+	c.Assert(err, check.IsNil)
+
+	c.Check(n, check.Equals, 1)
+}
+
+func (snapshotSuite) TestManagerRunCleanupAbandondedImportsAtStartupErrorLogged(c *check.C) {
+	logbuf, restore := logger.MockLogger()
+	defer restore()
+
+	n := 0
+	restore = snapshotstate.MockBackenCleanupAbandondedImports(func() (int, error) {
+		n++
+		return 0, errors.New("some error")
+	})
+	defer restore()
+
+	o := overlord.Mock()
+	st := o.State()
+	mgr := snapshotstate.Manager(st, state.NewTaskRunner(st))
+	c.Assert(mgr, check.NotNil)
+	o.AddManager(mgr)
+	err := o.Settle(100 * time.Millisecond)
+	c.Assert(err, check.IsNil)
+
+	c.Check(n, check.Equals, 1)
+	c.Check(logbuf.String(), testutil.Contains, "cannot cleanup incomplete imports: some error\n")
+}
