@@ -20,10 +20,9 @@
 package snapstate_test
 
 import (
+	"context"
 	"time"
 
-	"golang.org/x/net/context"
-
 	. "gopkg.in/check.v1"
 
 	"github.com/snapcore/snapd/overlord/auth"
@@ -41,7 +40,10 @@ type recordingStore struct {
 	ops []string
 }
 
-func (r *recordingStore) SnapAction(ctx context.Context, currentSnaps []*store.CurrentSnap, actions []*store.SnapAction, user *auth.UserState, opts *store.RefreshOptions) ([]*snap.Info, error) {
+func (r *recordingStore) SnapAction(ctx context.Context, currentSnaps []*store.CurrentSnap, actions []*store.SnapAction, assertQuery store.AssertionQuery, user *auth.UserState, opts *store.RefreshOptions) ([]store.SnapActionResult, []store.AssertionResult, error) {
+	if assertQuery != nil {
+		panic("no assertion query support")
+	}
 	if ctx == nil || !auth.IsEnsureContext(ctx) {
 		panic("Ensure marked context required")
 	}
@@ -54,7 +56,7 @@ func (r *recordingStore) SnapAction(ctx context.Context, currentSnaps []*store.C
 		}
 	}
 	r.ops = append(r.ops, "list-refresh")
-	return nil, nil
+	return nil, nil, nil
 }
 
 type refreshHintsTestSuite struct {
