@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016 Canonical Ltd
+ * Copyright (C) 2016-2020 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -707,7 +707,7 @@ func (ss *stateSuite) TestMethodEntrance(c *C) {
 		func() { st.Tasks() },
 		func() { st.Task("foo") },
 		func() { st.MarshalJSON() },
-		func() { st.Prune(time.Hour, time.Hour, 100) },
+		func() { st.Prune(time.Now(), time.Hour, time.Hour, 100) },
 		func() { st.TaskCount() },
 		func() { st.AllWarnings() },
 		func() { st.PendingWarnings() },
@@ -773,7 +773,8 @@ func (ss *stateSuite) TestPrune(c *C) {
 	st.AddWarning("hello", now, never, time.Nanosecond, state.DefaultRepeatAfter)
 	st.Warnf("hello again")
 
-	st.Prune(pruneWait, abortWait, 100)
+	past := time.Now().AddDate(-1, 0, 0)
+	st.Prune(past, pruneWait, abortWait, 100)
 
 	c.Assert(st.Change(chg1.ID()), Equals, chg1)
 	c.Assert(st.Change(chg2.ID()), IsNil)
@@ -814,7 +815,8 @@ func (ss *stateSuite) TestPruneEmptyChange(c *C) {
 	chg := st.NewChange("abort", "...")
 	state.MockChangeTimes(chg, now.Add(-pruneWait), time.Time{})
 
-	st.Prune(pruneWait, abortWait, 100)
+	past := time.Now().AddDate(-1, 0, 0)
+	st.Prune(past, pruneWait, abortWait, 100)
 	c.Assert(st.Change(chg.ID()), IsNil)
 }
 
@@ -849,13 +851,14 @@ func (ss *stateSuite) TestPruneMaxChangesHappy(c *C) {
 
 	// test that nothing is done when we are within pruneWait and
 	// maxReadyChanges
+	past := time.Now().AddDate(-1, 0, 0)
 	maxReadyChanges := 100
-	st.Prune(pruneWait, abortWait, maxReadyChanges)
+	st.Prune(past, pruneWait, abortWait, maxReadyChanges)
 	c.Assert(st.Changes(), HasLen, 15)
 
 	// but with maxReadyChanges we remove the ready ones
 	maxReadyChanges = 5
-	st.Prune(pruneWait, abortWait, maxReadyChanges)
+	st.Prune(past, pruneWait, abortWait, maxReadyChanges)
 	c.Assert(st.Changes(), HasLen, 10)
 	remaining := map[string]bool{}
 	for _, chg := range st.Changes() {
@@ -891,8 +894,9 @@ func (ss *stateSuite) TestPruneMaxChangesSomeNotReady(c *C) {
 	c.Assert(st.Changes(), HasLen, 10)
 
 	// nothing can be pruned
+	past := time.Now().AddDate(-1, 0, 0)
 	maxChanges := 5
-	st.Prune(1*time.Hour, 3*time.Hour, maxChanges)
+	st.Prune(past, 1*time.Hour, 3*time.Hour, maxChanges)
 	c.Assert(st.Changes(), HasLen, 10)
 }
 
@@ -921,10 +925,44 @@ func (ss *stateSuite) TestPruneMaxChangesHonored(c *C) {
 	//
 	// this test we do not purge the freshly ready change
 	maxChanges := 10
-	st.Prune(1*time.Hour, 3*time.Hour, maxChanges)
+	past := time.Now().AddDate(-1, 0, 0)
+	st.Prune(past, 1*time.Hour, 3*time.Hour, maxChanges)
 	c.Assert(st.Changes(), HasLen, 11)
 }
 
+func (ss *stateSuite) TestPruneHonorsStartOperationTime(c *C) {
+	st := state.New(&fakeStateBackend{})
+	st.Lock()
+	defer st.Unlock()
+
+	now := time.Now()
+
+	startTime := 2 * time.Hour
+	spawnTime := 10 * time.Hour
+	pruneWait := 1 * time.Hour
+	abortWait := 3 * time.Hour
+
+	chg := st.NewChange("change", "...")
+	t := st.NewTask("foo", "")
+	chg.AddTask(t)
+	// change spawned 10h ago
+	state.MockChangeTimes(chg, now.Add(-spawnTime), time.Time{})
+
+	// start operation time is 2h ago, change is not aborted because
+	// it's less than abortWait limit.
+	opTime := now.Add(-startTime)
+	st.Prune(opTime, pruneWait, abortWait, 100)
+	c.Assert(st.Changes(), HasLen, 1)
+	c.Check(chg.Status(), Equals, state.DoStatus)
+
+	// start operation time is 9h ago, change is aborted.
+	startTime = 9 * time.Hour
+	opTime = time.Now().Add(-startTime)
+	st.Prune(opTime, pruneWait, abortWait, 100)
+	c.Assert(st.Changes(), HasLen, 1)
+	c.Check(chg.Status(), Equals, state.HoldStatus)
+}
+
 func (ss *stateSuite) TestRequestRestart(c *C) {
 	b := new(fakeStateBackend)
 	st := state.New(b)
@@ -942,6 +980,49 @@ func (ss *stateSuite) TestRequestRestart(c *C) {
 	c.Check(t, Equals, state.RestartDaemon)
 }
 
+func (ss *stateSuite) TestRequestRestartSystemAndVerifyReboot(c *C) {
+	b := new(fakeStateBackend)
+	st := state.New(b)
+
+	st.Lock()
+	err := st.VerifyReboot("boot-id-1")
+	st.Unlock()
+	c.Assert(err, IsNil)
+
+	ok, t := st.Restarting()
+	c.Check(ok, Equals, false)
+	c.Check(t, Equals, state.RestartUnset)
+
+	st.Lock()
+	st.RequestRestart(state.RestartSystem)
+	st.Unlock()
+
+	c.Check(b.restartRequested, Equals, true)
+
+	ok, t = st.Restarting()
+	c.Check(ok, Equals, true)
+	c.Check(t, Equals, state.RestartSystem)
+
+	var fromBootID string
+	st.Lock()
+	c.Check(st.Get("system-restart-from-boot-id", &fromBootID), IsNil)
+	st.Unlock()
+	c.Check(fromBootID, Equals, "boot-id-1")
+
+	st.Lock()
+	err = st.VerifyReboot("boot-id-1")
+	st.Unlock()
+	c.Check(err, Equals, state.ErrExpectedReboot)
+
+	st.Lock()
+	err = st.VerifyReboot("boot-id-2")
+	st.Unlock()
+	c.Assert(err, IsNil)
+	st.Lock()
+	c.Check(st.Get("system-restart-from-boot-id", &fromBootID), Equals, state.ErrNoState)
+	st.Unlock()
+}
+
 func (ss *stateSuite) TestReadStateInitsCache(c *C) {
 	st, err := state.ReadState(nil, bytes.NewBufferString("{}"))
 	c.Assert(err, IsNil)
@@ -951,3 +1032,21 @@ func (ss *stateSuite) TestReadStateInitsCache(c *C) {
 	st.Cache("key", "value")
 	c.Assert(st.Cached("key"), Equals, "value")
 }
+
+func (ss *stateSuite) TestTimingsSupport(c *C) {
+	st := state.New(nil)
+	st.Lock()
+	defer st.Unlock()
+
+	var tims []int
+
+	err := st.GetMaybeTimings(&tims)
+	c.Assert(err, IsNil)
+	c.Check(tims, IsNil)
+
+	st.SaveTimings([]int{1, 2, 3})
+
+	err = st.GetMaybeTimings(&tims)
+	c.Assert(err, IsNil)
+	c.Check(tims, DeepEquals, []int{1, 2, 3})
+}
