@@ -4,120 +4,51 @@
 . "$TESTSLIB/quiet.sh"
 
 debian_name_package() {
+    #shellcheck source=tests/lib/tools/tests.pkgs.apt.sh
+    . "$TESTSLIB/tools/tests.pkgs.apt.sh"
     for i in "$@"; do
-        case "$i" in
-            man)
-                echo "man-db"
-                ;;
-            *)
-                echo "$i"
-                ;;
-        esac
+        remap_one "$i"
     done
 }
 
 ubuntu_14_04_name_package() {
+    #shellcheck source=tests/lib/tools/tests.pkgs.apt.sh
+    . "$TESTSLIB/tools/tests.pkgs.apt.sh"
     for i in "$@"; do
-        case "$i" in
-            printer-driver-cups-pdf)
-                echo "cups-pdf"
-                ;;
-            *)
-                debian_name_package "$i"
-                ;;
-        esac
+        remap_one "$i"
     done
 }
 
 fedora_name_package() {
+    #shellcheck source=tests/lib/tools/tests.pkgs.dnf-yum.sh
+    . "$TESTSLIB/tools/tests.pkgs.dnf-yum.sh"
     for i in "$@"; do
-        case "$i" in
-            openvswitch-switch)
-                echo "openvswitch"
-                ;;
-            printer-driver-cups-pdf)
-                echo "cups-pdf"
-                ;;
-            python3-gi)
-                echo "python3-gobject"
-                ;;
-            *)
-                echo "$i"
-                ;;
-        esac
+        remap_one "$i"
     done
 }
 
 amazon_name_package() {
+    #shellcheck source=tests/lib/tools/tests.pkgs.dnf-yum.sh
+    . "$TESTSLIB/tools/tests.pkgs.dnf-yum.sh"
     for i in "$@"; do
-        case "$i" in
-            xdelta3)
-                echo "xdelta"
-                ;;
-            openvswitch-switch)
-                echo "openvswitch"
-                ;;
-            *)
-                echo "$i"
-                ;;
-        esac
+        remap_one "$i"
     done
 }
 
 opensuse_name_package() {
+    #shellcheck source=tests/lib/tools/tests.pkgs.zypper.sh
+    . "$TESTSLIB/tools/tests.pkgs.zypper.sh"
     for i in "$@"; do
-        case "$i" in
-            python3-yaml)
-                echo "python3-PyYAML"
-                ;;
-            dbus-x11)
-                echo "dbus-1-x11"
-                ;;
-            printer-driver-cups-pdf)
-                echo "cups-pdf"
-                ;;
-            python3-dbus)
-                # In OpenSUSE Leap 15, this is renamed to python3-dbus-python
-                echo "dbus-1-python3"
-                ;;
-            python3-gi)
-                echo "python3-gobject"
-                ;;
-            *)
-                echo "$i"
-                ;;
-        esac
+        remap_one "$i"
     done
 }
 
 arch_name_package() {
-    case "$1" in
-        python3-yaml)
-            echo "python-yaml"
-            ;;
-        dbus-x11)
-            # no separate dbus-x11 package in arch
-            echo "dbus"
-            ;;
-        printer-driver-cups-pdf)
-            echo "cups-pdf"
-            ;;
-        openvswitch-switch)
-            echo "openvswitch"
-            ;;
-        man)
-            echo "man-db"
-            ;;
-        python3-dbus)
-            echo "python-dbus"
-            ;;
-        python3-gi)
-            echo "python-gobject"
-            ;;
-        *)
-            echo "$1"
-            ;;
-    esac
+    #shellcheck source=tests/lib/tools/tests.pkgs.pacman.sh
+    . "$TESTSLIB/tools/tests.pkgs.pacman.sh"
+    for i in "$@"; do
+        remap_one "$i"
+    done
 }
 
 distro_name_package() {
@@ -181,7 +112,7 @@ distro_install_local_package() {
             quiet dnf -y install --setopt=install_weak_deps=False "$@"
             ;;
         opensuse-*)
-            quiet rpm -i --replacepkgs "$@"
+            quiet zypper in -y --no-recommends --allow-unsigned-rpm "$@"
             ;;
         arch-*)
             pacman -U --noconfirm "$@"
@@ -202,7 +133,7 @@ distro_install_package() {
     # arguments as package names.
     APT_FLAGS=
     DNF_FLAGS=
-    if [[ "$SPREAD_SYSTEM" == fedora-* ]]; then
+    if os.query is-fedora; then
         # Fedora images we use come with a number of preinstalled package, among
         # them gtk3. Those packages are needed to run the tests. The
         # xdg-desktop-portal-gtk package uses this in the spec:
@@ -317,7 +248,9 @@ distro_purge_package() {
 
     case "$SPREAD_SYSTEM" in
         ubuntu-*|debian-*)
-            quiet eatmydata apt-get remove -y --purge -y "$@"
+            # TODO reenable quiet once we have dealt with files being left
+            # behind while purging in prepare
+            eatmydata apt-get remove -y --purge -y "$@"
             ;;
         amazon-*|centos-7-*)
             quiet yum -y remove "$@"
@@ -447,7 +380,7 @@ distro_install_build_snapd(){
 
         # On trusty we may pull in a new hwe-kernel that is needed to run the
         # snapd tests. We need to reboot to actually run this kernel.
-        if [[ "$SPREAD_SYSTEM" = ubuntu-14.04-* ]] && [ "$SPREAD_REBOOT" = 0 ]; then
+        if os.query is-trusty && [ "$SPREAD_REBOOT" = 0 ]; then
             REBOOT
         fi
     elif [ -n "$PPA_VALIDATION_NAME" ]; then
@@ -477,7 +410,7 @@ distro_install_build_snapd(){
                 ;;
             arch-*)
                 # shellcheck disable=SC2125
-                packages="${GOHOME}"/snapd*.pkg.tar.xz
+                packages="${GOHOME}"/snapd*.pkg.tar.*
                 ;;
             *)
                 exit 1
@@ -507,7 +440,7 @@ distro_install_build_snapd(){
                 ;;
         esac
 
-        if [[ "$SPREAD_SYSTEM" == arch-* ]]; then
+        if os.query is-arch-linux; then
             # Arch policy does not allow calling daemon-reloads in package
             # install scripts
             systemctl daemon-reload
@@ -518,6 +451,12 @@ distro_install_build_snapd(){
             fi
         fi
 
+        if [[ "$SPREAD_SYSTEM" == opensuse-tumbleweed-* ]]; then
+            # Package installation applies vendor presets only, which leaves
+            # snapd.apparmor disabled.
+            systemctl enable --now snapd.apparmor.service
+        fi
+
         # On some distributions the snapd.socket is not yet automatically
         # enabled as we don't have a systemd present configuration approved
         # by the distribution for it in place yet.
@@ -602,6 +541,7 @@ pkg_dependencies_ubuntu_classic(){
             ;;
         ubuntu-16.04-32)
             echo "
+                dbus-user-session
                 gccgo-6
                 evolution-data-server
                 fwupd
@@ -612,6 +552,7 @@ pkg_dependencies_ubuntu_classic(){
             ;;
         ubuntu-16.04-64)
             echo "
+                dbus-user-session
                 evolution-data-server
                 fwupd
                 gccgo-6
@@ -627,14 +568,8 @@ pkg_dependencies_ubuntu_classic(){
             ;;
         ubuntu-18.04-64)
             echo "
+                dbus-user-session
                 gccgo-8
-                evolution-data-server
-                fwupd
-                packagekit
-                "
-            ;;
-        ubuntu-19.10-64)
-            echo "
                 evolution-data-server
                 fwupd
                 packagekit
@@ -646,6 +581,12 @@ pkg_dependencies_ubuntu_classic(){
                 evolution-data-server
                 gccgo-9
                 packagekit
+                qemu-utils
+                shellcheck
+                "
+            ;;
+        ubuntu-20.10-64)
+            echo "
                 qemu-utils
                 "
             ;;
@@ -656,13 +597,19 @@ pkg_dependencies_ubuntu_classic(){
             ;;
         debian-*)
             echo "
+                autopkgtest
+                debootstrap
+                dbus-user-session
                 eatmydata
                 evolution-data-server
                 fwupd
+                gcc-multilib
                 libc6-dev-i386
+                linux-libc-dev
                 net-tools
                 packagekit
                 sbuild
+                schroot
                 "
             ;;
     esac
@@ -845,7 +792,7 @@ pkg_dependencies(){
 install_pkg_dependencies(){
     pkgs=$(pkg_dependencies)
     # shellcheck disable=SC2086
-    distro_install_package $pkgs
+    distro_install_package --no-install-recommends $pkgs
 }
 
 # upgrade distribution and indicate if reboot is needed by outputting 'reboot'
