@@ -32,7 +32,9 @@ import (
 	. "gopkg.in/check.v1"
 
 	"github.com/snapcore/snapd/dirs"
-	"github.com/snapcore/snapd/logger"
+	// imported to ensure actual interfaces are defined,
+	// in production this is guaranteed by ifacestate
+	_ "github.com/snapcore/snapd/interfaces/builtin"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/progress"
 	"github.com/snapcore/snapd/snap"
@@ -46,6 +48,8 @@ import (
 )
 
 type servicesTestSuite struct {
+	testutil.DBusTest
+
 	tempdir string
 
 	sysdLog [][]string
@@ -60,6 +64,7 @@ type servicesTestSuite struct {
 var _ = Suite(&servicesTestSuite{})
 
 func (s *servicesTestSuite) SetUpTest(c *C) {
+	s.DBusTest.SetUpTest(c)
 	s.tempdir = c.MkDir()
 	s.sysdLog = nil
 	dirs.SetRootDir(s.tempdir)
@@ -87,28 +92,58 @@ func (s *servicesTestSuite) TearDownTest(c *C) {
 	s.systemctlRestorer()
 	s.delaysRestorer()
 	dirs.SetRootDir("")
+	s.DBusTest.TearDownTest(c)
 }
 
 func (s *servicesTestSuite) TestAddSnapServicesAndRemove(c *C) {
 	info := snaptest.MockSnap(c, packageHello, &snap.SideInfo{Revision: snap.R(12)})
 	svcFile := filepath.Join(s.tempdir, "/etc/systemd/system/snap.hello-snap.svc1.service")
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
+	c.Assert(err, IsNil)
+	c.Check(s.sysdLog, DeepEquals, [][]string{
+		{"daemon-reload"},
+	})
+
+	s.sysdLog = nil
+
+	flags := &wrappers.StartServicesFlags{Enable: true}
+	err = wrappers.StartServices(info.Services(), nil, flags, progress.Null, s.perfTimings)
 	c.Assert(err, IsNil)
 	c.Check(s.sysdLog, DeepEquals, [][]string{
 		{"enable", filepath.Base(svcFile)},
-		{"daemon-reload"},
+		{"start", filepath.Base(svcFile)},
 	})
 
 	content, err := ioutil.ReadFile(svcFile)
 	c.Assert(err, IsNil)
 
-	verbs := []string{"Start", "Stop", "StopPost"}
-	cmds := []string{"", " --command=stop", " --command=post-stop"}
-	for i := range verbs {
-		expected := fmt.Sprintf("Exec%s=/usr/bin/snap run%s hello-snap.svc1", verbs[i], cmds[i])
-		c.Check(string(content), Matches, "(?ms).*^"+regexp.QuoteMeta(expected)) // check.v1 adds ^ and $ around the regexp provided
-	}
+	dir := filepath.Join(dirs.GlobalRootDir, "snap", "hello-snap", "12.mount")
+	c.Assert(string(content), Equals, fmt.Sprintf(`[Unit]
+# Auto-generated, DO NOT EDIT
+Description=Service for snap application hello-snap.svc1
+Requires=%[1]s
+Wants=network.target
+After=%[1]s network.target snapd.apparmor.service
+X-Snappy=yes
+
+[Service]
+EnvironmentFile=-/etc/environment
+ExecStart=/usr/bin/snap run hello-snap.svc1
+SyslogIdentifier=hello-snap.svc1
+Restart=on-failure
+WorkingDirectory=%[2]s/var/snap/hello-snap/12
+ExecStop=/usr/bin/snap run --command=stop hello-snap.svc1
+ExecStopPost=/usr/bin/snap run --command=post-stop hello-snap.svc1
+TimeoutStopSec=30
+Type=forking
+
+[Install]
+WantedBy=multi-user.target
+`,
+		systemd.EscapeUnitNamePath(dir),
+		dirs.GlobalRootDir,
+	))
 
 	s.sysdLog = nil
 	err = wrappers.StopServices(info.Services(), nil, "", progress.Null, s.perfTimings)
@@ -128,6 +163,154 @@ func (s *servicesTestSuite) TestAddSnapServicesAndRemove(c *C) {
 	c.Check(s.sysdLog[1], DeepEquals, []string{"daemon-reload"})
 }
 
+func (s *servicesTestSuite) TestAddSnapServicesWithInterfaceSnippets(c *C) {
+	tt := []struct {
+		comment     string
+		plugSnippet string
+	}{
+		// just single bare interfaces with no attributes
+		{
+			"docker-support",
+			`
+  plugs:
+   - docker-support`,
+		},
+		{
+			"k8s-support",
+			`
+  plugs:
+   - kubernetes-support`,
+		},
+		{
+			"lxd-support",
+			`
+  plugs:
+   - lxd-support
+`,
+		},
+		{
+			"greengrass-support",
+			`
+  plugs:
+   - greengrass-support
+`,
+		},
+
+		// multiple interfaces that require Delegate=true, but only one is
+		// generated
+
+		{
+			"multiple interfaces that require Delegate=true",
+			`
+  plugs:
+   - docker-support
+   - kubernetes-support`,
+		},
+
+		// interfaces with flavor attributes
+
+		{
+			"k8s-support with kubelet",
+			`
+  plugs:
+   - kubelet
+plugs:
+ kubelet:
+  interface: kubernetes-support
+  flavor: kubelet
+`,
+		},
+		{
+			"k8s-support with kubeproxy",
+			`
+  plugs:
+   - kubeproxy
+plugs:
+ kubeproxy:
+  interface: kubernetes-support
+  flavor: kubeproxy
+`,
+		},
+		{
+			"greengrass-support with legacy-container flavor",
+			`
+  plugs:
+   - greengrass
+plugs:
+ greengrass:
+  interface: greengrass-support
+  flavor: legacy-container
+`,
+		},
+	}
+
+	for _, t := range tt {
+		comment := Commentf(t.comment)
+		info := snaptest.MockSnap(c, packageHello+`
+ svc1:
+  daemon: simple
+`+t.plugSnippet,
+			&snap.SideInfo{Revision: snap.R(12)})
+		svcFile := filepath.Join(s.tempdir, "/etc/systemd/system/snap.hello-snap.svc1.service")
+
+		err := wrappers.AddSnapServices(info, nil, progress.Null)
+		c.Assert(err, IsNil, comment)
+		c.Check(s.sysdLog, DeepEquals, [][]string{
+			{"daemon-reload"},
+		}, comment)
+
+		content, err := ioutil.ReadFile(svcFile)
+		c.Assert(err, IsNil, comment)
+
+		dir := filepath.Join(dirs.GlobalRootDir, "snap", "hello-snap", "12.mount")
+		c.Assert(string(content), Equals, fmt.Sprintf(`[Unit]
+# Auto-generated, DO NOT EDIT
+Description=Service for snap application hello-snap.svc1
+Requires=%[1]s
+Wants=network.target
+After=%[1]s network.target snapd.apparmor.service
+X-Snappy=yes
+
+[Service]
+EnvironmentFile=-/etc/environment
+ExecStart=/usr/bin/snap run hello-snap.svc1
+SyslogIdentifier=hello-snap.svc1
+Restart=on-failure
+WorkingDirectory=%[2]s/var/snap/hello-snap/12
+TimeoutStopSec=30
+Type=simple
+Delegate=true
+
+[Install]
+WantedBy=multi-user.target
+`,
+			systemd.EscapeUnitNamePath(dir),
+			dirs.GlobalRootDir,
+		), comment)
+
+		s.sysdLog = nil
+		err = wrappers.StopServices(info.Services(), nil, "", progress.Null, s.perfTimings)
+		c.Assert(err, IsNil, comment)
+		c.Assert(s.sysdLog, HasLen, 2, comment)
+		c.Check(s.sysdLog, DeepEquals, [][]string{
+			{"stop", filepath.Base(svcFile)},
+			{"show", "--property=ActiveState", "snap.hello-snap.svc1.service"},
+		}, comment)
+
+		s.sysdLog = nil
+		err = wrappers.RemoveSnapServices(info, progress.Null)
+		c.Assert(err, IsNil, comment)
+		c.Check(osutil.FileExists(svcFile), Equals, false, comment)
+		c.Assert(s.sysdLog, HasLen, 2, comment)
+		c.Check(s.sysdLog, DeepEquals, [][]string{
+			{"disable", filepath.Base(svcFile)},
+			{"daemon-reload"},
+		}, comment)
+
+		s.sysdLog = nil
+	}
+}
+
 func (s *servicesTestSuite) TestAddSnapServicesAndRemoveUserDaemons(c *C) {
 	info := snaptest.MockSnap(c, packageHello+`
  svc1:
@@ -136,10 +319,9 @@ func (s *servicesTestSuite) TestAddSnapServicesAndRemoveUserDaemons(c *C) {
 `, &snap.SideInfo{Revision: snap.R(12)})
 	svcFile := filepath.Join(s.tempdir, "/etc/systemd/user/snap.hello-snap.svc1.service")
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, IsNil)
 	c.Check(s.sysdLog, DeepEquals, [][]string{
-		{"--user", "--global", "enable", filepath.Base(svcFile)},
 		{"--user", "daemon-reload"},
 	})
 
@@ -187,7 +369,7 @@ func (s *servicesTestSuite) TestRemoveSnapWithSocketsRemovesSocketsService(c *C)
       listen-stream: $SNAP_COMMON/sock2.socket
 `, &snap.SideInfo{Revision: snap.R(12)})
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	err = wrappers.StopServices(info.Services(), nil, "", &progress.Null, s.perfTimings)
@@ -227,7 +409,7 @@ apps:
    daemon: forking
 `, &snap.SideInfo{Revision: snap.R(11)})
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	sysdLog = nil
@@ -270,7 +452,7 @@ apps:
    daemon-scope: user
 `, &snap.SideInfo{Revision: snap.R(11)})
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	sysdLog = nil
@@ -422,6 +604,17 @@ func (s *servicesTestSuite) TestAddSnapServicesWithDisabledServices(c *C) {
 					;;
 			esac
 			;;
+		start)
+			case "$2" in
+				"snap.hello-snap.svc2.service")
+					exit 0
+					;;
+			*)
+					echo "unexpected start of service $2"
+					exit 1
+					;;
+			esac
+			;;
 		daemon-reload)
 			exit 0
 			;;
@@ -436,121 +629,23 @@ func (s *servicesTestSuite) TestAddSnapServicesWithDisabledServices(c *C) {
 	// svc1 will be disabled
 	disabledSvcs := []string{"svc1"}
 
-	err := wrappers.AddSnapServices(info, disabledSvcs, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
+	c.Assert(err, IsNil)
+
+	c.Assert(r.Calls(), DeepEquals, [][]string{
+		{"systemctl", "daemon-reload"},
+	})
+
+	r.ForgetCalls()
+
+	flags := &wrappers.StartServicesFlags{Enable: true}
+	err = wrappers.StartServices(info.Services(), disabledSvcs, flags, progress.Null, s.perfTimings)
 	c.Assert(err, IsNil)
 
 	// only svc2 should be enabled
 	c.Assert(r.Calls(), DeepEquals, [][]string{
 		{"systemctl", "enable", "snap.hello-snap.svc2.service"},
-		{"systemctl", "daemon-reload"},
-	})
-}
-
-func (s *servicesTestSuite) TestAddSnapServicesWithDisabledServicesNowApp(c *C) {
-	info := snaptest.MockSnap(c, packageHello, &snap.SideInfo{Revision: snap.R(12)})
-
-	// mock the logger
-	buf, loggerRestore := logger.MockLogger()
-	defer loggerRestore()
-
-	s.systemctlRestorer()
-	r := testutil.MockCommand(c, "systemctl", `#!/bin/sh
-	if [ "$1" = "--root" ]; then
-	    shift 2
-	fi
-
-	case "$1" in
-		enable)
-			case "$2" in 
-				"snap.hello-snap.svc1.service")
-					exit 0
-					;;
-				*)
-					echo "unexpected enable of service $2"
-					exit 1
-					;;
-			esac
-			;;
-		daemon-reload)
-			exit 0
-			;;
-	    *)
-	        echo "unexpected op $*"
-	        exit 2
-	esac
-	exit 2
-	`)
-	defer r.Restore()
-
-	svcs := []string{"hello"}
-
-	err := wrappers.AddSnapServices(info, svcs, nil, progress.Null)
-	c.Assert(err, IsNil)
-
-	// check the log for the notice
-	c.Assert(buf.String(), Matches, `.*previously disabled service hello is now an app and not a service\n.*`)
-
-	// the cleanup of AddSnapServices will remove written service files and then
-	// call reload, but note that we should catch any non-svc apps before
-	// actually enabling them, so we just see a daemon-reload call and not any
-	// enable calls
-	c.Assert(r.Calls(), DeepEquals, [][]string{
-		{"systemctl", "enable", "snap.hello-snap.svc1.service"},
-		{"systemctl", "daemon-reload"},
-	})
-}
-
-func (s *servicesTestSuite) TestAddSnapServicesWithDisabledServicesMissing(c *C) {
-	info := snaptest.MockSnap(c, packageHello, &snap.SideInfo{Revision: snap.R(12)})
-
-	// mock the logger
-	buf, loggerRestore := logger.MockLogger()
-	defer loggerRestore()
-
-	s.systemctlRestorer()
-	r := testutil.MockCommand(c, "systemctl", `#!/bin/sh
-	if [ "$1" = "--root" ]; then
-	    shift 2
-	fi
-
-	case "$1" in
-		enable)
-			case "$2" in 
-				"snap.hello-snap.svc1.service")
-					exit 0
-					;;
-				*)
-					echo "unexpected enable of service $2"
-					exit 1
-					;;
-			esac
-			;;
-		daemon-reload)
-			exit 0
-			;;
-	    *)
-	        echo "unexpected op $*"
-	        exit 2
-	esac
-	exit 2
-	`)
-	defer r.Restore()
-
-	svcs := []string{"old-disabled-svc"}
-
-	err := wrappers.AddSnapServices(info, svcs, nil, progress.Null)
-	c.Assert(err, IsNil)
-
-	// check the log for the notice
-	c.Assert(buf.String(), Matches, `.*previously disabled service old-disabled-svc no longer exists\n.*`)
-
-	// the cleanup of AddSnapServices will remove written service files and then
-	// call reload, but note that we should catch any non-svc apps before
-	// actually enabling them, so we just see a daemon-reload call and not any
-	// enable calls
-	c.Assert(r.Calls(), DeepEquals, [][]string{
-		{"systemctl", "enable", "snap.hello-snap.svc1.service"},
-		{"systemctl", "daemon-reload"},
+		{"systemctl", "start", "snap.hello-snap.svc2.service"},
 	})
 }
 
@@ -563,7 +658,7 @@ func (s *servicesTestSuite) TestAddSnapServicesWithPreseed(c *C) {
 	r := testutil.MockCommand(c, "systemctl", "exit 1")
 	defer r.Restore()
 
-	err := wrappers.AddSnapServices(info, nil, opts, progress.Null)
+	err := wrappers.AddSnapServices(info, opts, progress.Null)
 	c.Assert(err, IsNil)
 
 	// file was created
@@ -608,7 +703,7 @@ func (s *servicesTestSuite) TestStopServicesWithSockets(c *C) {
       listen-stream: $SNAP_USER_DATA/sock2.socket
 `, &snap.SideInfo{Revision: snap.R(12)})
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	sysServices = nil
@@ -629,11 +724,12 @@ func (s *servicesTestSuite) TestStartServices(c *C) {
 	info := snaptest.MockSnap(c, packageHello, &snap.SideInfo{Revision: snap.R(12)})
 	svcFile := filepath.Join(s.tempdir, "/etc/systemd/system/snap.hello-snap.svc1.service")
 
-	err := wrappers.StartServices(info.Services(), nil, nil, &progress.Null, s.perfTimings)
+	flags := &wrappers.StartServicesFlags{Enable: true}
+	err := wrappers.StartServices(info.Services(), nil, flags, &progress.Null, s.perfTimings)
 	c.Assert(err, IsNil)
 
-	c.Assert(s.sysdLog, DeepEquals, [][]string{
-		{"is-enabled", filepath.Base(svcFile)},
+	c.Check(s.sysdLog, DeepEquals, [][]string{
+		{"enable", filepath.Base(svcFile)},
 		{"start", filepath.Base(svcFile)},
 	})
 }
@@ -646,30 +742,33 @@ func (s *servicesTestSuite) TestStartServicesUserDaemons(c *C) {
 `, &snap.SideInfo{Revision: snap.R(12)})
 	svcFile := filepath.Join(s.tempdir, "/etc/systemd/user/snap.hello-snap.svc1.service")
 
-	err := wrappers.StartServices(info.Services(), nil, nil, &progress.Null, s.perfTimings)
+	flags := &wrappers.StartServicesFlags{Enable: true}
+	err := wrappers.StartServices(info.Services(), nil, flags, &progress.Null, s.perfTimings)
 	c.Assert(err, IsNil)
 
 	c.Assert(s.sysdLog, DeepEquals, [][]string{
-		{"--user", "--global", "is-enabled", filepath.Base(svcFile)},
+		{"--user", "--global", "enable", filepath.Base(svcFile)},
 		{"--user", "start", filepath.Base(svcFile)},
 	})
 }
 
-func (s *servicesTestSuite) TestEnableServices(c *C) {
+func (s *servicesTestSuite) TestStartServicesEnabledConditional(c *C) {
 	info := snaptest.MockSnap(c, packageHello, &snap.SideInfo{Revision: snap.R(12)})
 	svcFile := filepath.Join(s.tempdir, "/etc/systemd/system/snap.hello-snap.svc1.service")
 
-	err := wrappers.EnableSnapServices(info, nil)
-	c.Assert(err, IsNil)
-
-	c.Assert(s.sysdLog, DeepEquals, [][]string{
-		{"enable", filepath.Base(svcFile)},
-	})
+	flags := &wrappers.StartServicesFlags{}
+	c.Check(wrappers.StartServices(info.Services(), nil, flags, progress.Null, s.perfTimings), IsNil)
+	c.Check(s.sysdLog, DeepEquals, [][]string{{"start", filepath.Base(svcFile)}})
 }
 
 func (s *servicesTestSuite) TestNoStartDisabledServices(c *C) {
-	info := snaptest.MockSnap(c, packageHello, &snap.SideInfo{Revision: snap.R(12)})
-	svcFile := filepath.Join(s.tempdir, "/etc/systemd/system/snap.hello-snap.svc1.service")
+	svc2Name := "snap.hello-snap.svc2.service"
+
+	info := snaptest.MockSnap(c, packageHello+`
+ svc2:
+  command: bin/hello
+  daemon: simple
+`, &snap.SideInfo{Revision: snap.R(12)})
 
 	s.systemctlRestorer()
 	r := testutil.MockCommand(c, "systemctl", `#!/bin/sh
@@ -678,15 +777,20 @@ func (s *servicesTestSuite) TestNoStartDisabledServices(c *C) {
 	fi
 
 	case "$1" in
-	    is-enabled)
-	        if [ "$2" = "snap.hello-snap.svc1.service" ]; then
-	            echo "disabled"
-	            exit 1
-	        else
-	            echo "unexpected call $*"
-	            exit 2
-	        fi
-	        ;;
+		start)
+			if [ "$2" = "snap.hello-snap.svc2.service" ]; then
+				exit 0
+			fi
+			echo "unexpected start of service $2"
+			exit 1
+			;;
+		enable)
+			if [ "$2" = "snap.hello-snap.svc2.service" ]; then
+				exit 0
+			fi
+			echo "unexpected enable of service $2"
+			exit 1
+			;;
 	    *)
 	        echo "unexpected call $*"
 	        exit 2
@@ -694,11 +798,12 @@ func (s *servicesTestSuite) TestNoStartDisabledServices(c *C) {
 	`)
 	defer r.Restore()
 
-	err := wrappers.StartServices(info.Services(), nil, nil, &progress.Null, s.perfTimings)
+	flags := &wrappers.StartServicesFlags{Enable: true}
+	err := wrappers.StartServices(info.Services(), []string{"svc1"}, flags, &progress.Null, s.perfTimings)
 	c.Assert(err, IsNil)
-
 	c.Assert(r.Calls(), DeepEquals, [][]string{
-		{"systemctl", "is-enabled", filepath.Base(svcFile)},
+		{"systemctl", "enable", svc2Name},
+		{"systemctl", "start", svc2Name},
 	})
 }
 
@@ -712,7 +817,7 @@ func (s *servicesTestSuite) TestAddSnapMultiServicesFailCreateCleanup(c *C) {
   daemon: potato
 `, &snap.SideInfo{Revision: snap.R(12)})
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, ErrorMatches, ".*potato.*")
 
 	// the services are cleaned up
@@ -730,7 +835,7 @@ func (s *servicesTestSuite) TestAddSnapMultiServicesFailCreateCleanup(c *C) {
 	}
 }
 
-func (s *servicesTestSuite) TestAddSnapMultiServicesFailEnableCleanup(c *C) {
+func (s *servicesTestSuite) TestMultiServicesFailEnableCleanup(c *C) {
 	var sysdLog [][]string
 	svc1Name := "snap.hello-snap.svc1.service"
 	svc2Name := "snap.hello-snap.svc2.service"
@@ -743,6 +848,9 @@ func (s *servicesTestSuite) TestAddSnapMultiServicesFailEnableCleanup(c *C) {
 	r := systemd.MockSystemctl(func(cmd ...string) ([]byte, error) {
 		sysdLog = append(sysdLog, cmd)
 		sdcmd := cmd[0]
+		if sdcmd == "show" {
+			return []byte("ActiveState=inactive"), nil
+		}
 		if len(cmd) >= 2 {
 			sdcmd = cmd[len(cmd)-2]
 		}
@@ -761,7 +869,7 @@ func (s *servicesTestSuite) TestAddSnapMultiServicesFailEnableCleanup(c *C) {
 			default:
 				panic("expected no more than 2 enables")
 			}
-		case "disable", "daemon-reload":
+		case "disable", "daemon-reload", "stop":
 			return nil, nil
 		default:
 			panic("unexpected systemctl command " + sdcmd)
@@ -775,43 +883,38 @@ func (s *servicesTestSuite) TestAddSnapMultiServicesFailEnableCleanup(c *C) {
   daemon: simple
 `, &snap.SideInfo{Revision: snap.R(12)})
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
+	c.Assert(err, IsNil)
+
+	svcFiles, _ = filepath.Glob(filepath.Join(dirs.SnapServicesDir, "snap.hello-snap.*.service"))
+	c.Check(svcFiles, HasLen, 2)
+
+	flags := &wrappers.StartServicesFlags{Enable: true}
+	err = wrappers.StartServices(info.Services(), nil, flags, progress.Null, s.perfTimings)
 	c.Assert(err, ErrorMatches, "failed")
 
-	// the services are cleaned up
-	svcFiles, _ = filepath.Glob(filepath.Join(dirs.SnapServicesDir, "snap.hello-snap.*.service"))
-	c.Check(svcFiles, HasLen, 0)
 	c.Check(sysdLog, DeepEquals, [][]string{
+		{"daemon-reload"}, // from AddSnapServices
 		{"enable", svc1Name},
 		{"enable", svc2Name}, // this one fails
 		{"disable", svc1Name},
-		{"daemon-reload"},
 	})
 }
 
 func (s *servicesTestSuite) TestAddSnapMultiServicesStartFailOnSystemdReloadCleanup(c *C) {
 	// this test might be overdoing it (it's mostly covering the same ground as the previous one), but ... :-)
 	var sysdLog [][]string
-	svc1Name := "snap.hello-snap.svc1.service"
-	svc2Name := "snap.hello-snap.svc2.service"
 
 	// sanity check: there are no service files
 	svcFiles, _ := filepath.Glob(filepath.Join(dirs.SnapServicesDir, "snap.hello-snap.*.service"))
 	c.Check(svcFiles, HasLen, 0)
 
-	first := true
 	r := systemd.MockSystemctl(func(cmd ...string) ([]byte, error) {
 		sysdLog = append(sysdLog, cmd)
-		if len(cmd) < 2 {
+		if cmd[0] == "daemon-reload" {
 			return nil, fmt.Errorf("failed")
 		}
-		if first {
-			first = false
-			if cmd[len(cmd)-1] == svc2Name {
-				// the services are being iterated in the "wrong" order
-				svc1Name, svc2Name = svc2Name, svc1Name
-			}
-		}
+		c.Fatalf("unexpected systemctl call")
 		return nil, nil
 
 	})
@@ -823,27 +926,20 @@ func (s *servicesTestSuite) TestAddSnapMultiServicesStartFailOnSystemdReloadClea
   daemon: simple
 `, &snap.SideInfo{Revision: snap.R(12)})
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, ErrorMatches, "failed")
 
 	// the services are cleaned up
 	svcFiles, _ = filepath.Glob(filepath.Join(dirs.SnapServicesDir, "snap.hello-snap.*.service"))
 	c.Check(svcFiles, HasLen, 0)
 	c.Check(sysdLog, DeepEquals, [][]string{
-		{"enable", svc1Name},
-		{"enable", svc2Name},
 		{"daemon-reload"}, // this one fails
-		{"disable", svc1Name},
-		{"disable", svc2Name},
-		{"daemon-reload"}, // so does this one :-)
+		{"daemon-reload"}, // reload as part of cleanup after removal
 	})
 }
 
 func (s *servicesTestSuite) TestAddSnapMultiUserServicesFailEnableCleanup(c *C) {
 	var sysdLog [][]string
-	svc1Name := "snap.hello-snap.svc1.service"
-	svc2Name := "snap.hello-snap.svc2.service"
-	numEnables := 0
 
 	// sanity check: there are no service files
 	svcFiles, _ := filepath.Glob(filepath.Join(dirs.SnapUserServicesDir, "snap.hello-snap.*.service"))
@@ -862,22 +958,8 @@ func (s *servicesTestSuite) TestAddSnapMultiUserServicesFailEnableCleanup(c *C)
 			sdcmd = cmd[len(cmd)-2]
 		}
 		switch sdcmd {
-		case "enable":
-			numEnables++
-			switch numEnables {
-			case 1:
-				if cmd[len(cmd)-1] == svc2Name {
-					// the services are being iterated in the "wrong" order
-					svc1Name, svc2Name = svc2Name, svc1Name
-				}
-				return nil, nil
-			case 2:
-				return nil, fmt.Errorf("failed")
-			default:
-				panic("expected no more than 2 enables")
-			}
-		case "disable", "daemon-reload":
-			return nil, nil
+		case "daemon-reload":
+			return nil, fmt.Errorf("failed")
 		default:
 			panic("unexpected systemctl command " + sdcmd)
 		}
@@ -895,16 +977,14 @@ func (s *servicesTestSuite) TestAddSnapMultiUserServicesFailEnableCleanup(c *C)
   daemon-scope: user
 `, &snap.SideInfo{Revision: snap.R(12)})
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
-	c.Assert(err, ErrorMatches, "failed")
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
+	c.Assert(err, ErrorMatches, "cannot reload daemon: failed")
 
 	// the services are cleaned up
 	svcFiles, _ = filepath.Glob(filepath.Join(dirs.SnapUserServicesDir, "snap.hello-snap.*.service"))
 	c.Check(svcFiles, HasLen, 0)
 	c.Check(sysdLog, DeepEquals, [][]string{
-		{"--user", "--global", "enable", svc1Name},
-		{"--user", "--global", "enable", svc2Name}, // this one fails
-		{"--user", "--global", "disable", svc1Name},
+		{"--user", "daemon-reload"},
 		{"--user", "daemon-reload"},
 	})
 }
@@ -948,18 +1028,14 @@ func (s *servicesTestSuite) TestAddSnapMultiUserServicesStartFailOnSystemdReload
   daemon-scope: user
 `, &snap.SideInfo{Revision: snap.R(12)})
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, ErrorMatches, "cannot reload daemon: failed")
 
 	// the services are cleaned up
 	svcFiles, _ = filepath.Glob(filepath.Join(dirs.SnapUserServicesDir, "snap.hello-snap.*.service"))
 	c.Check(svcFiles, HasLen, 0)
 	c.Check(sysdLog, DeepEquals, [][]string{
-		{"--user", "--global", "enable", svc1Name},
-		{"--user", "--global", "enable", svc2Name},
 		{"--user", "daemon-reload"}, // this one fails
-		{"--user", "--global", "disable", svc1Name},
-		{"--user", "--global", "disable", svc2Name},
 		{"--user", "daemon-reload"}, // so does this one :-)
 	})
 }
@@ -984,7 +1060,7 @@ func (s *servicesTestSuite) TestAddSnapSocketFiles(c *C) {
 	sock2File := filepath.Join(s.tempdir, "/etc/systemd/system/snap.hello-snap.svc1.sock2.socket")
 	sock3File := filepath.Join(s.tempdir, "/etc/systemd/system/snap.hello-snap.svc1.sock3.socket")
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	expected := fmt.Sprintf(
@@ -1036,7 +1112,7 @@ func (s *servicesTestSuite) TestAddSnapUserSocketFiles(c *C) {
 	sock2File := filepath.Join(s.tempdir, "/etc/systemd/user/snap.hello-snap.svc1.sock2.socket")
 	sock3File := filepath.Join(s.tempdir, "/etc/systemd/user/snap.hello-snap.svc1.sock3.socket")
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	expected := `[Socket]
@@ -1093,18 +1169,22 @@ func (s *servicesTestSuite) TestStartSnapMultiServicesFailStartCleanup(c *C) {
 	if svcs[0].Name == "svc2" {
 		svcs[0], svcs[1] = svcs[1], svcs[0]
 	}
-	err := wrappers.StartServices(svcs, nil, nil, &progress.Null, s.perfTimings)
+
+	flags := &wrappers.StartServicesFlags{Enable: true}
+	err := wrappers.StartServices(svcs, nil, flags, &progress.Null, s.perfTimings)
 	c.Assert(err, ErrorMatches, "failed")
-	c.Assert(sysdLog, HasLen, 8, Commentf("len: %v calls: %v", len(sysdLog), sysdLog))
+	c.Assert(sysdLog, HasLen, 10, Commentf("len: %v calls: %v", len(sysdLog), sysdLog))
 	c.Check(sysdLog, DeepEquals, [][]string{
-		{"is-enabled", svc1Name},
-		{"is-enabled", svc2Name},
+		{"enable", svc1Name},
+		{"enable", svc2Name},
 		{"start", svc1Name},
 		{"start", svc2Name}, // one of the services fails
 		{"stop", svc2Name},
 		{"show", "--property=ActiveState", svc2Name},
 		{"stop", svc1Name},
 		{"show", "--property=ActiveState", svc1Name},
+		{"disable", svc1Name},
+		{"disable", svc2Name},
 	}, Commentf("calls: %v", sysdLog))
 }
 
@@ -1147,12 +1227,13 @@ func (s *servicesTestSuite) TestStartSnapMultiServicesFailStartCleanupWithSocket
 	// ensure desired order
 	apps := []*snap.AppInfo{info.Apps["svc1"], info.Apps["svc2"], info.Apps["svc3"]}
 
-	err := wrappers.StartServices(apps, nil, nil, &progress.Null, s.perfTimings)
+	flags := &wrappers.StartServicesFlags{Enable: true}
+	err := wrappers.StartServices(apps, nil, flags, &progress.Null, s.perfTimings)
 	c.Assert(err, ErrorMatches, "failed")
 	c.Logf("sysdlog: %v", sysdLog)
-	c.Assert(sysdLog, HasLen, 17, Commentf("len: %v calls: %v", len(sysdLog), sysdLog))
+	c.Assert(sysdLog, HasLen, 18, Commentf("len: %v calls: %v", len(sysdLog), sysdLog))
 	c.Check(sysdLog, DeepEquals, [][]string{
-		{"is-enabled", svc1Name},
+		{"enable", svc1Name},
 		{"enable", svc2SocketName},
 		{"start", svc2SocketName},
 		{"enable", svc3SocketName},
@@ -1169,6 +1250,7 @@ func (s *servicesTestSuite) TestStartSnapMultiServicesFailStartCleanupWithSocket
 		{"disable", svc2SocketName},
 		{"stop", svc1Name},
 		{"show", "--property=ActiveState", svc1Name},
+		{"disable", svc1Name},
 	}, Commentf("calls: %v", sysdLog))
 }
 
@@ -1205,12 +1287,13 @@ func (s *servicesTestSuite) TestStartSnapMultiUserServicesFailStartCleanup(c *C)
 	if svcs[0].Name == "svc2" {
 		svcs[0], svcs[1] = svcs[1], svcs[0]
 	}
-	err := wrappers.StartServices(svcs, nil, nil, &progress.Null, s.perfTimings)
+	flags := &wrappers.StartServicesFlags{Enable: true}
+	err := wrappers.StartServices(svcs, nil, flags, &progress.Null, s.perfTimings)
 	c.Assert(err, ErrorMatches, "some user services failed to start")
-	c.Assert(sysdLog, HasLen, 10, Commentf("len: %v calls: %v", len(sysdLog), sysdLog))
+	c.Assert(sysdLog, HasLen, 12, Commentf("len: %v calls: %v", len(sysdLog), sysdLog))
 	c.Check(sysdLog, DeepEquals, [][]string{
-		{"--user", "--global", "is-enabled", svc1Name},
-		{"--user", "--global", "is-enabled", svc2Name},
+		{"--user", "--global", "enable", svc1Name},
+		{"--user", "--global", "enable", svc2Name},
 		{"--user", "start", svc1Name},
 		{"--user", "start", svc2Name}, // one of the services fails
 		// session agent attempts to stop the non-failed services
@@ -1221,6 +1304,8 @@ func (s *servicesTestSuite) TestStartSnapMultiUserServicesFailStartCleanup(c *C)
 		{"--user", "show", "--property=ActiveState", svc2Name},
 		{"--user", "stop", svc1Name},
 		{"--user", "show", "--property=ActiveState", svc1Name},
+		{"--user", "--global", "disable", svc1Name},
+		{"--user", "--global", "disable", svc2Name},
 	}, Commentf("calls: %v", sysdLog))
 }
 
@@ -1255,13 +1340,14 @@ apps:
 	sorted, err := snap.SortServices(svcs)
 	c.Assert(err, IsNil)
 
-	err = wrappers.StartServices(sorted, nil, nil, &progress.Null, s.perfTimings)
+	flags := &wrappers.StartServicesFlags{Enable: true}
+	err = wrappers.StartServices(sorted, nil, flags, &progress.Null, s.perfTimings)
 	c.Assert(err, IsNil)
 	c.Assert(sysdLog, HasLen, 6, Commentf("len: %v calls: %v", len(sysdLog), sysdLog))
 	c.Check(sysdLog, DeepEquals, [][]string{
-		{"is-enabled", svc1Name},
-		{"is-enabled", svc3Name},
-		{"is-enabled", svc2Name},
+		{"enable", svc1Name},
+		{"enable", svc3Name},
+		{"enable", svc2Name},
 		{"start", svc1Name},
 		{"start", svc3Name},
 		{"start", svc2Name},
@@ -1271,13 +1357,13 @@ apps:
 	sorted[1], sorted[0] = sorted[0], sorted[1]
 
 	// we should observe the calls done in the same order as services
-	err = wrappers.StartServices(sorted, nil, nil, &progress.Null, s.perfTimings)
+	err = wrappers.StartServices(sorted, nil, flags, &progress.Null, s.perfTimings)
 	c.Assert(err, IsNil)
 	c.Assert(sysdLog, HasLen, 12, Commentf("len: %v calls: %v", len(sysdLog), sysdLog))
 	c.Check(sysdLog[6:], DeepEquals, [][]string{
-		{"is-enabled", svc3Name},
-		{"is-enabled", svc1Name},
-		{"is-enabled", svc2Name},
+		{"enable", svc3Name},
+		{"enable", svc1Name},
+		{"enable", svc2Name},
 		{"start", svc3Name},
 		{"start", svc1Name},
 		{"start", svc2Name},
@@ -1328,7 +1414,7 @@ func (s *servicesTestSuite) TestServiceAfterBefore(c *C) {
 		},
 	}}
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	for _, check := range checks {
@@ -1368,7 +1454,7 @@ func (s *servicesTestSuite) TestServiceWatchdog(c *C) {
 `
 	info := snaptest.MockSnap(c, snapYaml, &snap.SideInfo{Revision: snap.R(12)})
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	content, err := ioutil.ReadFile(filepath.Join(s.tempdir, "/etc/systemd/system/snap.hello-snap.svc2.service"))
@@ -1398,12 +1484,22 @@ apps:
 	info := snaptest.MockSnap(c, surviveYaml, &snap.SideInfo{Revision: snap.R(1)})
 	survivorFile := filepath.Join(s.tempdir, "/etc/systemd/system/snap.survive-snap.survivor.service")
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, IsNil)
 	c.Check(s.sysdLog, DeepEquals, [][]string{
-		{"enable", filepath.Base(survivorFile)},
 		{"daemon-reload"},
 	})
+	s.sysdLog = nil
+
+	apps := []*snap.AppInfo{info.Apps["survivor"]}
+	flags := &wrappers.StartServicesFlags{Enable: true}
+	err = wrappers.StartServices(apps, nil, flags, progress.Null, s.perfTimings)
+	c.Assert(err, IsNil)
+
+	c.Check(s.sysdLog, DeepEquals, [][]string{
+		{"enable", filepath.Base(survivorFile)},
+		{"start", filepath.Base(survivorFile)},
+	})
 
 	s.sysdLog = nil
 	err = wrappers.StopServices(info.Services(), nil, snap.StopReasonRefresh, progress.Null, s.perfTimings)
@@ -1450,11 +1546,24 @@ apps:
 		info := snaptest.MockSnap(c, surviveYaml, &snap.SideInfo{Revision: snap.R(1)})
 
 		s.sysdLog = nil
-		err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+		err := wrappers.AddSnapServices(info, nil, progress.Null)
+		c.Assert(err, IsNil)
+
+		c.Check(s.sysdLog, DeepEquals, [][]string{
+			{"daemon-reload"},
+		})
+		s.sysdLog = nil
+
+		var apps []*snap.AppInfo
+		for _, a := range info.Apps {
+			apps = append(apps, a)
+		}
+		flags := &wrappers.StartServicesFlags{Enable: true}
+		err = wrappers.StartServices(apps, nil, flags, progress.Null, s.perfTimings)
 		c.Assert(err, IsNil)
 		c.Check(s.sysdLog, DeepEquals, [][]string{
 			{"enable", filepath.Base(survivorFile)},
-			{"daemon-reload"},
+			{"start", filepath.Base(survivorFile)},
 		})
 
 		s.sysdLog = nil
@@ -1512,11 +1621,12 @@ func (s *servicesTestSuite) TestStartSnapSocketEnableStart(c *C) {
 
 	// fix the apps order to make the test stable
 	apps := []*snap.AppInfo{info.Apps["svc1"], info.Apps["svc2"], info.Apps["svc3"]}
-	err := wrappers.StartServices(apps, nil, nil, &progress.Null, s.perfTimings)
+	flags := &wrappers.StartServicesFlags{Enable: true}
+	err := wrappers.StartServices(apps, nil, flags, &progress.Null, s.perfTimings)
 	c.Assert(err, IsNil)
 	c.Assert(s.sysdLog, HasLen, 6, Commentf("len: %v calls: %v", len(s.sysdLog), s.sysdLog))
 	c.Check(s.sysdLog, DeepEquals, [][]string{
-		{"is-enabled", svc1Name},
+		{"enable", svc1Name},
 		{"enable", svc2Sock},
 		{"start", svc2Sock},
 		{"--user", "--global", "enable", svc3Sock},
@@ -1545,11 +1655,12 @@ func (s *servicesTestSuite) TestStartSnapTimerEnableStart(c *C) {
 
 	// fix the apps order to make the test stable
 	apps := []*snap.AppInfo{info.Apps["svc1"], info.Apps["svc2"], info.Apps["svc3"]}
-	err := wrappers.StartServices(apps, nil, nil, &progress.Null, s.perfTimings)
+	flags := &wrappers.StartServicesFlags{Enable: true}
+	err := wrappers.StartServices(apps, nil, flags, &progress.Null, s.perfTimings)
 	c.Assert(err, IsNil)
 	c.Assert(s.sysdLog, HasLen, 6, Commentf("len: %v calls: %v", len(s.sysdLog), s.sysdLog))
 	c.Check(s.sysdLog, DeepEquals, [][]string{
-		{"is-enabled", svc1Name},
+		{"enable", svc1Name},
 		{"enable", svc2Timer},
 		{"start", svc2Timer},
 		{"--user", "--global", "enable", svc3Timer},
@@ -1582,11 +1693,12 @@ func (s *servicesTestSuite) TestStartSnapTimerCleanup(c *C) {
 
 	// fix the apps order to make the test stable
 	apps := []*snap.AppInfo{info.Apps["svc1"], info.Apps["svc2"]}
-	err := wrappers.StartServices(apps, nil, nil, &progress.Null, s.perfTimings)
+	flags := &wrappers.StartServicesFlags{Enable: true}
+	err := wrappers.StartServices(apps, nil, flags, &progress.Null, s.perfTimings)
 	c.Assert(err, ErrorMatches, "failed")
-	c.Assert(sysdLog, HasLen, 10, Commentf("len: %v calls: %v", len(sysdLog), sysdLog))
+	c.Assert(sysdLog, HasLen, 11, Commentf("len: %v calls: %v", len(sysdLog), sysdLog))
 	c.Check(sysdLog, DeepEquals, [][]string{
-		{"is-enabled", svc1Name},
+		{"enable", svc1Name},
 		{"enable", svc2Timer},
 		{"start", svc2Timer}, // this call fails
 		{"stop", svc2Timer},
@@ -1596,6 +1708,7 @@ func (s *servicesTestSuite) TestStartSnapTimerCleanup(c *C) {
 		{"disable", svc2Timer},
 		{"stop", svc1Name},
 		{"show", "--property=ActiveState", svc1Name},
+		{"disable", svc1Name},
 	}, Commentf("calls: %v", sysdLog))
 }
 
@@ -1607,7 +1720,7 @@ func (s *servicesTestSuite) TestAddRemoveSnapWithTimersAddsRemovesTimerFiles(c *
   timer: 10:00-12:00
 `, &snap.SideInfo{Revision: snap.R(12)})
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	app := info.Apps["svc2"]
@@ -1654,7 +1767,7 @@ func (s *servicesTestSuite) TestFailedAddSnapCleansUp(c *C) {
 	})
 	defer r()
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, NotNil)
 
 	c.Logf("services dir: %v", dirs.SnapServicesDir)
@@ -1696,7 +1809,7 @@ apps:
 
 	for i, info := range []*snap.Info{onlyServices, onlySockets, onlyTimers} {
 		s.sysdLog = nil
-		err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+		err := wrappers.AddSnapServices(info, nil, progress.Null)
 		c.Assert(err, IsNil)
 		reloads := 0
 		c.Logf("calls: %v", s.sysdLog)
@@ -1731,19 +1844,33 @@ apps:
   command: bin/hello
   daemon: simple
 `
-
+	svc1Socket := "snap.hello-snap.svc1.sock1.socket"
+	svc2Timer := "snap.hello-snap.svc2.timer"
 	svc3Name := "snap.hello-snap.svc3.service"
 
 	info := snaptest.MockSnap(c, snapYaml, &snap.SideInfo{Revision: snap.R(12)})
 
 	// fix the apps order to make the test stable
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, IsNil)
-	c.Assert(s.sysdLog, HasLen, 2, Commentf("len: %v calls: %v", len(s.sysdLog), s.sysdLog))
 	c.Check(s.sysdLog, DeepEquals, [][]string{
-		// only svc3 gets started during boot
-		{"enable", svc3Name},
 		{"daemon-reload"},
+	})
+	s.sysdLog = nil
+
+	apps := []*snap.AppInfo{info.Apps["svc1"], info.Apps["svc2"], info.Apps["svc3"]}
+	flags := &wrappers.StartServicesFlags{Enable: true}
+	err = wrappers.StartServices(apps, nil, flags, progress.Null, s.perfTimings)
+	c.Assert(err, IsNil)
+
+	c.Assert(s.sysdLog, HasLen, 6, Commentf("len: %v calls: %v", len(s.sysdLog), s.sysdLog))
+	c.Check(s.sysdLog, DeepEquals, [][]string{
+		{"enable", svc3Name},
+		{"enable", svc1Socket},
+		{"start", svc1Socket},
+		{"enable", svc2Timer},
+		{"start", svc2Timer},
+		{"start", svc3Name},
 	}, Commentf("calls: %v", s.sysdLog))
 }
 
@@ -1757,7 +1884,7 @@ func (s *servicesTestSuite) TestServiceRestartDelay(c *C) {
 `
 	info := snaptest.MockSnap(c, snapYaml, &snap.SideInfo{Revision: snap.R(12)})
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	content, err := ioutil.ReadFile(filepath.Join(s.tempdir, "/etc/systemd/system/snap.hello-snap.svc2.service"))
@@ -1772,7 +1899,7 @@ func (s *servicesTestSuite) TestServiceRestartDelay(c *C) {
 func (s *servicesTestSuite) TestAddRemoveSnapServiceWithSnapd(c *C) {
 	info := makeMockSnapdSnap(c)
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Check(err, ErrorMatches, "internal error: adding explicit services for snapd snap is unexpected")
 
 	err = wrappers.RemoveSnapServices(info, progress.Null)
@@ -1790,7 +1917,7 @@ apps:
 	info := snaptest.MockSnap(c, surviveYaml, &snap.SideInfo{Revision: snap.R(1)})
 	srvFile := "snap.test-snap.foo.service"
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	s.sysdLog = nil
@@ -1826,7 +1953,7 @@ func (s *servicesTestSuite) TestStopAndDisableServices(c *C) {
 `, &snap.SideInfo{Revision: snap.R(12)})
 	svcFile := "snap.hello-snap.svc1.service"
 
-	err := wrappers.AddSnapServices(info, nil, nil, progress.Null)
+	err := wrappers.AddSnapServices(info, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	s.sysdLog = nil
